@@ -3,121 +3,81 @@
 #include <unistd.h>
 
 /* This should be defined before #include "utils.h" */
-#define PR_FMT     "dynamic"
-#define PR_DOMAIN  DBG_DYNAMIC
+#define PR_FMT "dynamic"
+#define PR_DOMAIN DBG_DYNAMIC
 
-#include "mcount-arch.h"
+#include "libmcount/dynamic.h"
 #include "libmcount/internal.h"
-#include "utils/utils.h"
+#include "mcount-arch.h"
 #include "utils/symbol.h"
+#include "utils/utils.h"
 
-#define PAGE_SIZE  4096
-#define PAGE_ADDR(a)    ((void *)((a) & ~(PAGE_SIZE - 1)))
-#define PAGE_LEN(a, l)    (a + l - (unsigned long)PAGE_ADDR(a))
-#define XRAY_SECT  "xray_instr_map"
-#define MCOUNTLOC_SECT  "__mcount_loc"
-
-#define CALL_INSN_SIZE  5
-#define JMP_INSN_SIZE   6
-
-/* target instrumentation function it needs to call */
-extern void __fentry__(void);
-extern void __dentry__(void);
-extern void __xray_entry(void);
-extern void __xray_exit(void);
-
-struct xray_instr_map {
-	unsigned long addr;
-	unsigned long entry;
-	unsigned long type;
-	unsigned long count;
-};
-
-enum mcount_x86_dynamic_type {
-	DYNAMIC_NONE,
-	DYNAMIC_PG,
-	DYNAMIC_FENTRY,
-	DYNAMIC_FENTRY_NOP,
-	DYNAMIC_XRAY,
-};
-
-static const char *adi_type_names[] = {
-	"none", "pg", "fentry", "fentry-nop", "xray",
-};
-
-struct arch_dynamic_info {
-	enum mcount_x86_dynamic_type	type;
-	struct xray_instr_map		*xrmap;
-	unsigned long			*mcount_loc;
-	unsigned			xrmap_count;
-	unsigned			nr_mcount_loc;
-};
+static const unsigned char fentry_nop_patt1[] = { 0x67, 0x0f, 0x1f, 0x04, 0x00 };
+static const unsigned char fentry_nop_patt2[] = { 0x0f, 0x1f, 0x44, 0x00, 0x00 };
+static const unsigned char patchable_gcc_nop[] = { 0x90, 0x90, 0x90, 0x90, 0x90 };
+static const unsigned char patchable_clang_nop[] = { 0x0f, 0x1f, 0x44, 0x00, 0x08 };
 
 int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
 {
-	unsigned char trampoline[] = { 0xff, 0x25, 0x02, 0x00, 0x00, 0x00, 0xcc, 0xcc };
+	unsigned char trampoline[] = { 0x3e, 0xff, 0x25, 0x01, 0x00, 0x00, 0x00, 0xcc };
 	unsigned long fentry_addr = (unsigned long)__fentry__;
 	unsigned long xray_entry_addr = (unsigned long)__xray_entry;
 	unsigned long xray_exit_addr = (unsigned long)__xray_exit;
-	struct arch_dynamic_info *adi = mdi->arch;
 	size_t trampoline_size = 16;
 	void *trampoline_check;
 
-	if (adi->type == DYNAMIC_XRAY)
+	if (mdi->type == DYNAMIC_XRAY)
 		trampoline_size *= 2;
 
 	/* find unused 16-byte at the end of the code segment */
-	mdi->trampoline  = ALIGN(mdi->text_addr + mdi->text_size, PAGE_SIZE);
+	mdi->trampoline = ALIGN(mdi->text_addr + mdi->text_size, PAGE_SIZE);
 	mdi->trampoline -= trampoline_size;
 
 	if (unlikely(mdi->trampoline < mdi->text_addr + mdi->text_size)) {
 		mdi->trampoline += trampoline_size;
-		mdi->text_size  += PAGE_SIZE;
+		mdi->text_size += PAGE_SIZE;
 
-		pr_dbg2("adding a page for fentry trampoline at %#lx\n",
-			mdi->trampoline);
+		pr_dbg2("adding a page for fentry trampoline at %#lx\n", mdi->trampoline);
 
 		trampoline_check = mmap((void *)mdi->trampoline, PAGE_SIZE,
 					PROT_READ | PROT_WRITE | PROT_EXEC,
-		     			MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS,
-					-1, 0);
+					MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 
 		if (trampoline_check == MAP_FAILED)
 			pr_err("failed to mmap trampoline for setup");
 	}
 
-	if (mprotect(PAGE_ADDR(mdi->text_addr), 
-			 PAGE_LEN(mdi->text_addr, mdi->text_size),
+	if (mprotect(PAGE_ADDR(mdi->text_addr), PAGE_LEN(mdi->text_addr, mdi->text_size),
 		     PROT_READ | PROT_WRITE | PROT_EXEC)) {
 		pr_dbg("cannot setup trampoline due to protection: %m\n");
 		return -1;
 	}
 
-	if (adi->type == DYNAMIC_XRAY) {
-		/* jmpq  *0x2(%rip)     # <xray_entry_addr> */
+	if (mdi->type == DYNAMIC_XRAY) {
+		/* jmpq  *0x1(%rip)     # <xray_entry_addr> */
 		memcpy((void *)mdi->trampoline, trampoline, sizeof(trampoline));
-		memcpy((void *)mdi->trampoline + sizeof(trampoline),
-		       &xray_entry_addr, sizeof(xray_entry_addr));
+		memcpy((void *)mdi->trampoline + sizeof(trampoline), &xray_entry_addr,
+		       sizeof(xray_entry_addr));
 
-		/* jmpq  *0x2(%rip)     # <xray_exit_addr> */
+		/* jmpq  *0x1(%rip)     # <xray_exit_addr> */
 		memcpy((void *)mdi->trampoline + 16, trampoline, sizeof(trampoline));
-		memcpy((void *)mdi->trampoline + 16 + sizeof(trampoline),
-		       &xray_exit_addr, sizeof(xray_exit_addr));
+		memcpy((void *)mdi->trampoline + 16 + sizeof(trampoline), &xray_exit_addr,
+		       sizeof(xray_exit_addr));
 	}
-	else if (adi->type == DYNAMIC_FENTRY_NOP) {
-		/* jmpq  *0x2(%rip)     # <fentry_addr> */
+	else if (mdi->type == DYNAMIC_FENTRY_NOP || mdi->type == DYNAMIC_PATCHABLE) {
+		/* jmpq  *0x1(%rip)     # <fentry_addr> */
 		memcpy((void *)mdi->trampoline, trampoline, sizeof(trampoline));
-		memcpy((void *)mdi->trampoline + sizeof(trampoline),
-		       &fentry_addr, sizeof(fentry_addr));
+		memcpy((void *)mdi->trampoline + sizeof(trampoline), &fentry_addr,
+		       sizeof(fentry_addr));
 	}
-	else if (adi->type == DYNAMIC_NONE) {
+	else if (mdi->type == DYNAMIC_NONE) {
 #ifdef HAVE_LIBCAPSTONE
 		unsigned long dentry_addr = (unsigned long)__dentry__;
 
 		/* jmpq  *0x2(%rip)     # <dentry_addr> */
 		memcpy((void *)mdi->trampoline, trampoline, sizeof(trampoline));
-		memcpy((void *)mdi->trampoline + sizeof(trampoline),
-		       &dentry_addr, sizeof(dentry_addr));
+		memcpy((void *)mdi->trampoline + sizeof(trampoline), &dentry_addr,
+		       sizeof(dentry_addr));
 #endif
 	}
 	return 0;
@@ -125,72 +85,89 @@ int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
 
 void mcount_cleanup_trampoline(struct mcount_dynamic_info *mdi)
 {
-	if (mprotect(PAGE_ADDR(mdi->text_addr), 
-			 PAGE_LEN(mdi->text_addr, mdi->text_size), PROT_EXEC))
+	if (mprotect(PAGE_ADDR(mdi->text_addr), PAGE_LEN(mdi->text_addr, mdi->text_size),
+		     PROT_READ | PROT_EXEC))
 		pr_err("cannot restore trampoline due to protection");
 }
 
-static void read_xray_map(struct arch_dynamic_info *adi,
-			  struct uftrace_elf_data *elf,
-			  struct uftrace_elf_iter *iter,
-			  unsigned long offset)
+static void read_xray_map(struct mcount_dynamic_info *mdi, struct uftrace_elf_data *elf,
+			  struct uftrace_elf_iter *iter, unsigned long offset)
 {
+	struct xray_instr_map *xrmap;
+	unsigned i;
 	typeof(iter->shdr) *shdr = &iter->shdr;
 
-	adi->xrmap_count = shdr->sh_size / sizeof(*adi->xrmap);
-	adi->xrmap = xmalloc(adi->xrmap_count * sizeof(*adi->xrmap));
+	mdi->nr_patch_target = shdr->sh_size / sizeof(*xrmap);
+	mdi->patch_target = xmalloc(mdi->nr_patch_target * sizeof(*xrmap));
 
 	elf_get_secdata(elf, iter);
-	elf_read_secdata(elf, iter, 0, adi->xrmap, shdr->sh_size);
+	elf_read_secdata(elf, iter, 0, mdi->patch_target, shdr->sh_size);
 
-	/* handle position independent code */
-	if (elf->ehdr.e_type == ET_DYN) {
-		struct xray_instr_map *xrmap;
-		unsigned i;
+	for (i = 0; i < mdi->nr_patch_target; i++) {
+		xrmap = &((struct xray_instr_map *)mdi->patch_target)[i];
 
-		for (i = 0; i < adi->xrmap_count; i++) {
-			xrmap = &adi->xrmap[i];
-
-			xrmap->addr  += offset;
-			xrmap->entry += offset;
+		if (xrmap->version == 2) {
+			xrmap->address += offset + (shdr->sh_offset + i * sizeof(*xrmap));
+			xrmap->function += offset + (shdr->sh_offset + i * sizeof(*xrmap) + 8);
+		}
+		else if (elf->ehdr.e_type == ET_DYN) {
+			xrmap->address += offset;
+			xrmap->function += offset;
 		}
 	}
 }
 
-static void read_mcount_loc(struct arch_dynamic_info *adi,
-			    struct uftrace_elf_data *elf,
-			    struct uftrace_elf_iter *iter,
-			    unsigned long offset)
+static void read_mcount_loc(struct mcount_dynamic_info *mdi, struct uftrace_elf_data *elf,
+			    struct uftrace_elf_iter *iter, unsigned long offset)
 {
 	typeof(iter->shdr) *shdr = &iter->shdr;
 
-	adi->nr_mcount_loc = shdr->sh_size / sizeof(long);
-	adi->mcount_loc = xmalloc(shdr->sh_size);
+	mdi->nr_patch_target = shdr->sh_size / sizeof(long);
+	mdi->patch_target = xmalloc(shdr->sh_size);
 
 	elf_get_secdata(elf, iter);
-	elf_read_secdata(elf, iter, 0, adi->mcount_loc, shdr->sh_size);
+	elf_read_secdata(elf, iter, 0, mdi->patch_target, shdr->sh_size);
 
 	/* symbol has relative address, fix it to match each other */
 	if (elf->ehdr.e_type == ET_EXEC) {
+		unsigned long *mcount_loc = mdi->patch_target;
 		unsigned i;
 
-		for (i = 0; i < adi->nr_mcount_loc; i++) {
-			adi->mcount_loc[i] -= offset;
+		for (i = 0; i < mdi->nr_patch_target; i++) {
+			mcount_loc[i] -= offset;
 		}
 	}
 }
 
-void mcount_arch_find_module(struct mcount_dynamic_info *mdi,
-			     struct symtab *symtab)
+static void read_patchable_loc(struct mcount_dynamic_info *mdi, struct uftrace_elf_data *elf,
+			       struct uftrace_elf_iter *iter, unsigned long offset)
+{
+	typeof(iter->shdr) *shdr = &iter->shdr;
+
+	mdi->nr_patch_target = shdr->sh_size / sizeof(long);
+	mdi->patch_target = xmalloc(shdr->sh_size);
+
+	elf_get_secdata(elf, iter);
+	elf_read_secdata(elf, iter, 0, mdi->patch_target, shdr->sh_size);
+
+	/* symbol has relative address, fix it to match each other */
+	if (elf->ehdr.e_type == ET_EXEC) {
+		unsigned long *patchable_loc = mdi->patch_target;
+		unsigned i;
+
+		for (i = 0; i < mdi->nr_patch_target; i++) {
+			patchable_loc[i] -= offset;
+		}
+	}
+}
+
+void mcount_arch_find_module(struct mcount_dynamic_info *mdi, struct uftrace_symtab *symtab)
 {
 	struct uftrace_elf_data elf;
 	struct uftrace_elf_iter iter;
-	struct arch_dynamic_info *adi;
-	unsigned char fentry_nop_patt1[] = { 0x67, 0x0f, 0x1f, 0x04, 0x00 };
-	unsigned char fentry_nop_patt2[] = { 0x0f, 0x1f, 0x44, 0x00, 0x00 };
 	unsigned i = 0;
 
-	adi = xzalloc(sizeof(*adi));  /* DYNAMIC_NONE */
+	mdi->type = DYNAMIC_NONE;
 
 	if (elf_init(mdi->map->libname, &elf) < 0)
 		goto out;
@@ -198,54 +175,73 @@ void mcount_arch_find_module(struct mcount_dynamic_info *mdi,
 	elf_for_each_shdr(&elf, &iter) {
 		char *shstr = elf_get_name(&elf, &iter, iter.shdr.sh_name);
 
+		if (!strcmp(shstr, PATCHABLE_SECT)) {
+			mdi->type = DYNAMIC_PATCHABLE;
+			read_patchable_loc(mdi, &elf, &iter, mdi->base_addr);
+			goto out;
+		}
+
 		if (!strcmp(shstr, XRAY_SECT)) {
-			adi->type = DYNAMIC_XRAY;
-			read_xray_map(adi, &elf, &iter, mdi->base_addr);
+			mdi->type = DYNAMIC_XRAY;
+			read_xray_map(mdi, &elf, &iter, mdi->base_addr);
 			goto out;
 		}
 
 		if (!strcmp(shstr, MCOUNTLOC_SECT)) {
-			read_mcount_loc(adi, &elf, &iter, mdi->base_addr);
+			read_mcount_loc(mdi, &elf, &iter, mdi->base_addr);
 			/* still needs to check pg or fentry */
 		}
 	}
 
-	/* check first few functions have fentry signature */
+	/*
+	 * check first few functions have fentry or patchable function entry
+	 * signature.
+	 */
 	for (i = 0; i < symtab->nr_sym; i++) {
-		struct sym *sym = &symtab->sym[i];
+		struct uftrace_symbol *sym = &symtab->sym[i];
 		void *code_addr = (void *)sym->addr + mdi->map->start;
 
 		if (sym->type != ST_LOCAL_FUNC && sym->type != ST_GLOBAL_FUNC)
 			continue;
 
-		/* dont' check special functions */
+		/* don't check special functions */
 		if (sym->name[0] == '_')
 			continue;
 
+		/*
+		 * there might be some chances of not having patchable section
+		 * '__patchable_function_entries' but shows the NOPs pattern.
+		 * this can be treated as DYNAMIC_FENTRY_NOP.
+		 */
+		if (!memcmp(code_addr, patchable_gcc_nop, CALL_INSN_SIZE) ||
+		    !memcmp(code_addr, patchable_clang_nop, CALL_INSN_SIZE)) {
+			mdi->type = DYNAMIC_FENTRY_NOP;
+			goto out;
+		}
+
 		/* only support calls to __fentry__ at the beginning */
 		if (!memcmp(code_addr, fentry_nop_patt1, CALL_INSN_SIZE) ||
 		    !memcmp(code_addr, fentry_nop_patt2, CALL_INSN_SIZE)) {
-			adi->type = DYNAMIC_FENTRY_NOP;
+			mdi->type = DYNAMIC_FENTRY_NOP;
 			goto out;
 		}
 	}
 
 	switch (check_trace_functions(mdi->map->libname)) {
 	case TRACE_MCOUNT:
-		adi->type = DYNAMIC_PG;
+		mdi->type = DYNAMIC_PG;
 		break;
 	case TRACE_FENTRY:
-		adi->type = DYNAMIC_FENTRY;
+		mdi->type = DYNAMIC_FENTRY;
 		break;
 	default:
 		break;
 	}
 
 out:
-	pr_dbg("dynamic patch type: %s: %d (%s)\n", basename(mdi->map->libname),
-	       adi->type, adi_type_names[adi->type]);
+	pr_dbg("dynamic patch type: %s: %d (%s)\n", basename(mdi->map->libname), mdi->type,
+	       mdi_type_names[mdi->type]);
 
-	mdi->arch = adi;
 	elf_finish(&elf);
 }
 
@@ -254,18 +250,18 @@ static unsigned long get_target_addr(struct mcount_dynamic_info *mdi, unsigned l
 	return mdi->trampoline - (addr + CALL_INSN_SIZE);
 }
 
-static int patch_fentry_func(struct mcount_dynamic_info *mdi, struct sym *sym)
+static int patch_fentry_code(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym)
 {
-	unsigned char nop1[] = { 0x67, 0x0f, 0x1f, 0x04, 0x00 };
-	unsigned char nop2[] = { 0x0f, 0x1f, 0x44, 0x00, 0x00 };
 	unsigned char *insn = (void *)sym->addr + mdi->map->start;
 	unsigned int target_addr;
 
-	/* only support calls to __fentry__ at the beginning */
-	if (memcmp(insn, nop1, sizeof(nop1)) &&  /* old pattern */
-	    memcmp(insn, nop2, sizeof(nop2))) {  /* new pattern */
-		pr_dbg("skip non-applicable functions: %s\n", sym->name);
-		return INSTRUMENT_FAILED;
+	/* support patchable function entry and __fentry__ at the beginning */
+	if (memcmp(insn, patchable_gcc_nop, sizeof(patchable_gcc_nop)) &&
+	    memcmp(insn, patchable_clang_nop, sizeof(patchable_clang_nop)) &&
+	    memcmp(insn, fentry_nop_patt1, sizeof(fentry_nop_patt1)) &&
+	    memcmp(insn, fentry_nop_patt2, sizeof(fentry_nop_patt2))) {
+		pr_dbg4("skip non-applicable functions: %s\n", sym->name);
+		return INSTRUMENT_SKIPPED;
 	}
 
 	/* get the jump offset to the trampoline */
@@ -278,23 +274,32 @@ static int patch_fentry_func(struct mcount_dynamic_info *mdi, struct sym *sym)
 	/* hopefully we're not patching 'memcpy' itself */
 	memcpy(&insn[1], &target_addr, sizeof(target_addr));
 
-	pr_dbg3("update function '%s' dynamically to call __fentry__\n",
-		sym->name);
+	pr_dbg3("update %p for '%s' function dynamically to call __fentry__\n", insn, sym->name);
 
 	return INSTRUMENT_SUCCESS;
 }
 
-static int update_xray_code(struct mcount_dynamic_info *mdi, struct sym *sym,
+static int patch_fentry_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym)
+{
+	return patch_fentry_code(mdi, sym);
+}
+
+static int patch_patchable_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym)
+{
+	/* it does the same patch logic with fentry. */
+	return patch_fentry_code(mdi, sym);
+}
+
+static int update_xray_code(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym,
 			    struct xray_instr_map *xrmap)
 {
 	unsigned char entry_insn[] = { 0xeb, 0x09 };
-	unsigned char exit_insn[]  = { 0xc3, 0x2e };
-	unsigned char pad[] = { 0x66, 0x0f, 0x1f, 0x84, 0x00,
-				0x00, 0x02, 0x00, 0x00 };
+	unsigned char exit_insn[] = { 0xc3, 0x2e };
+	unsigned char pad[] = { 0x66, 0x0f, 0x1f, 0x84, 0x00, 0x00, 0x02, 0x00, 0x00 };
 	unsigned char nop6[] = { 0x66, 0x0f, 0x1f, 0x44, 0x00, 0x00 };
 	unsigned char nop4[] = { 0x0f, 0x1f, 0x40, 0x00 };
 	unsigned int target_addr;
-	unsigned char *func = (void *)xrmap->addr;
+	unsigned char *func = (void *)xrmap->address;
 	union {
 		unsigned long word;
 		char bytes[8];
@@ -303,63 +308,62 @@ static int update_xray_code(struct mcount_dynamic_info *mdi, struct sym *sym,
 	if (memcmp(func + 2, pad, sizeof(pad)))
 		return INSTRUMENT_FAILED;
 
-	if (xrmap->type == 0) {  /* ENTRY */
+	if (xrmap->kind == 0) { /* ENTRY */
 		if (memcmp(func, entry_insn, sizeof(entry_insn)))
 			return INSTRUMENT_FAILED;
 
-		target_addr = mdi->trampoline - (xrmap->addr + 5);
+		target_addr = mdi->trampoline - (xrmap->address + 5);
 
 		memcpy(func + 5, nop6, sizeof(nop6));
 
 		/* need to write patch_word atomically */
-		patch.bytes[0] = 0xe8;  /* "call" insn */
+		patch.bytes[0] = 0xe8; /* "call" insn */
 		memcpy(&patch.bytes[1], &target_addr, sizeof(target_addr));
 		memcpy(&patch.bytes[5], nop6, 3);
 
 		memcpy(func, patch.bytes, sizeof(patch));
 	}
-	else {  /* EXIT */
+	else { /* EXIT */
 		if (memcmp(func, exit_insn, sizeof(exit_insn)))
 			return INSTRUMENT_FAILED;
 
-		target_addr = mdi->trampoline + 16 - (xrmap->addr + 5);
+		target_addr = mdi->trampoline + 16 - (xrmap->address + 5);
 
 		memcpy(func + 5, nop4, sizeof(nop4));
 
 		/* need to write patch_word atomically */
-		patch.bytes[0] = 0xe9;  /* "jmp" insn */
+		patch.bytes[0] = 0xe9; /* "jmp" insn */
 		memcpy(&patch.bytes[1], &target_addr, sizeof(target_addr));
 		memcpy(&patch.bytes[5], nop4, 3);
 
 		memcpy(func, patch.bytes, sizeof(patch));
 	}
 
-	pr_dbg3("update function '%s' dynamically to call xray functions\n",
-		sym->name);
+	pr_dbg3("update %p for '%s' function %s dynamically to call xray functions\n", func,
+		sym->name, xrmap->kind == 0 ? "entry" : "exit ");
 	return INSTRUMENT_SUCCESS;
 }
 
-static int patch_xray_func(struct mcount_dynamic_info *mdi, struct sym *sym)
+static int patch_xray_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym)
 {
 	unsigned i;
 	int ret = -2;
-	struct arch_dynamic_info *adi = mdi->arch;
 	struct xray_instr_map *xrmap;
 	uint64_t sym_addr = sym->addr + mdi->map->start;
 
 	/* xray provides a pair of entry and exit (or more) */
-	for (i = 0; i < adi->xrmap_count; i++) {
-		xrmap = &adi->xrmap[i];
+	for (i = 0; i < mdi->nr_patch_target; i++) {
+		xrmap = &((struct xray_instr_map *)mdi->patch_target)[i];
 
-		if (xrmap->addr < sym_addr || xrmap->addr >= sym_addr + sym->size)
+		if (xrmap->address < sym_addr || xrmap->address >= sym_addr + sym->size)
 			continue;
 
 		while ((ret = update_xray_code(mdi, sym, xrmap)) == 0) {
-			if (i == adi->xrmap_count - 1)
+			if (i == mdi->nr_patch_target - 1)
 				break;
 			i++;
 
-			if (xrmap->entry != xrmap[1].entry)
+			if (xrmap->function != xrmap[1].function)
 				break;
 			xrmap++;
 		}
@@ -415,15 +419,19 @@ static int patch_xray_func(struct mcount_dynamic_info *mdi, struct sym *sym)
 /*
  * Patch the instruction to the address as given for arguments.
  */
-static void patch_code(struct mcount_dynamic_info *mdi,
-		       uintptr_t addr, uint32_t origin_code_size)
+static void patch_code(struct mcount_dynamic_info *mdi, struct mcount_disasm_info *info)
 {
 	void *origin_code_addr;
 	unsigned char call_insn[] = { 0xe8, 0x00, 0x00, 0x00, 0x00 };
-	uint32_t target_addr = get_target_addr(mdi, addr);
+	uint32_t target_addr = get_target_addr(mdi, info->addr);
 
 	/* patch address */
-	origin_code_addr = (void *)addr;
+	origin_code_addr = (void *)info->addr;
+
+	if (info->has_intel_cet) {
+		origin_code_addr += ENDBR_INSN_SIZE;
+		target_addr = get_target_addr(mdi, info->addr + ENDBR_INSN_SIZE);
+	}
 
 	/* build the instrumentation instruction */
 	memcpy(&call_insn[1], &target_addr, CALL_INSN_SIZE - 1);
@@ -448,32 +456,36 @@ static void patch_code(struct mcount_dynamic_info *mdi,
 	 * dynamic: 0x400554[01]:nop
 	 */
 	memcpy(origin_code_addr, call_insn, CALL_INSN_SIZE);
-	memset(origin_code_addr + CALL_INSN_SIZE, 0x90,  /* NOP */
-	       origin_code_size - CALL_INSN_SIZE);
+	memset(origin_code_addr + CALL_INSN_SIZE, 0x90, /* NOP */
+	       info->orig_size - CALL_INSN_SIZE);
 
 	/* flush icache so that cpu can execute the new insn */
-	__builtin___clear_cache(origin_code_addr,
-				origin_code_addr + origin_code_size);
+	__builtin___clear_cache(origin_code_addr, origin_code_addr + info->orig_size);
 }
 
-static int patch_normal_func(struct mcount_dynamic_info *mdi, struct sym *sym,
+static int patch_normal_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym,
 			     struct mcount_disasm_engine *disasm)
 {
-	uint8_t jmp_insn[14] = { 0xff, 0x25, };
+	uint8_t jmp_insn[15] = {
+		0x3e,
+		0xff,
+		0x25,
+	};
 	uint64_t jmp_target;
-	struct mcount_orig_insn *orig;
 	struct mcount_disasm_info info = {
-		.sym  = sym,
+		.sym = sym,
 		.addr = mdi->map->start + sym->addr,
 	};
+	unsigned call_offset = CALL_INSN_SIZE;
 	int state;
 
 	state = disasm_check_insns(disasm, mdi, &info);
-	if (state != INSTRUMENT_SUCCESS)
+	if (state != INSTRUMENT_SUCCESS) {
+		pr_dbg3("  >> %s: %s\n", state == INSTRUMENT_FAILED ? "FAIL" : "SKIP", sym->name);
 		return state;
+	}
 
-	pr_dbg2("patch normal func: %s (patch size: %d)\n",
-		sym->name, info.orig_size);
+	pr_dbg2("force patch normal func: %s (patch size: %d)\n", sym->name, info.orig_size);
 
 	/*
 	 *  stored origin instruction block:
@@ -486,16 +498,19 @@ static int patch_normal_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 	 *  ----------------------
 	 */
 	jmp_target = info.addr + info.orig_size;
-	memcpy(jmp_insn + JMP_INSN_SIZE, &jmp_target, sizeof(jmp_target));
+	if (info.has_intel_cet) {
+		jmp_target += ENDBR_INSN_SIZE;
+		call_offset += ENDBR_INSN_SIZE;
+	}
+
+	memcpy(jmp_insn + CET_JMP_INSN_SIZE, &jmp_target, sizeof(jmp_target));
 
 	if (info.has_jump)
-		orig = mcount_save_code(&info, jmp_insn, 0);
+		mcount_save_code(&info, call_offset, jmp_insn, 0);
 	else
-		orig = mcount_save_code(&info, jmp_insn, sizeof(jmp_insn));
+		mcount_save_code(&info, call_offset, jmp_insn, sizeof(jmp_insn));
 
-	/* make sure orig->addr same as when called from __dentry__ */
-	orig->addr += CALL_INSN_SIZE;
-	patch_code(mdi, info.addr, info.orig_size);
+	patch_code(mdi, &info);
 
 	return INSTRUMENT_SUCCESS;
 }
@@ -521,12 +536,12 @@ static int unpatch_func(uint8_t *insn, char *name)
 
 	pr_dbg3("unpatch fentry: %s\n", name);
 	memcpy(insn, nop_insn, nop_size);
-	__builtin___clear_cache(insn, insn + nop_size);
+	__builtin___clear_cache((void *)insn, (void *)insn + nop_size);
 
 	return INSTRUMENT_SUCCESS;
 }
 
-static int unpatch_fentry_func(struct mcount_dynamic_info *mdi, struct sym *sym)
+static int unpatch_fentry_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym)
 {
 	uint64_t sym_addr = sym->addr + mdi->map->start;
 
@@ -535,7 +550,7 @@ static int unpatch_fentry_func(struct mcount_dynamic_info *mdi, struct sym *sym)
 
 static int cmp_loc(const void *a, const void *b)
 {
-	const struct sym *sym = a;
+	const struct uftrace_symbol *sym = a;
 	uintptr_t loc = *(uintptr_t *)b;
 
 	if (sym->addr <= loc && loc < sym->addr + sym->size)
@@ -544,18 +559,16 @@ static int cmp_loc(const void *a, const void *b)
 	return sym->addr > loc ? 1 : -1;
 }
 
-static int unpatch_mcount_func(struct mcount_dynamic_info *mdi, struct sym *sym)
+static int unpatch_mcount_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym)
 {
-	struct arch_dynamic_info *adi = mdi->arch;
-
+	unsigned long *mcount_loc = mdi->patch_target;
 	uintptr_t *loc;
 
-	if (adi->nr_mcount_loc != 0) {
-		loc = bsearch(sym, adi->mcount_loc, adi->nr_mcount_loc,
-			       sizeof(*adi->mcount_loc), cmp_loc);
+	if (mdi->nr_patch_target != 0) {
+		loc = bsearch(sym, mcount_loc, mdi->nr_patch_target, sizeof(*mcount_loc), cmp_loc);
 
 		if (loc != NULL) {
-			uint8_t *insn = (uint8_t*) *loc;
+			uint8_t *insn = (uint8_t *)*loc;
 			return unpatch_func(insn + mdi->map->start, sym->name);
 		}
 	}
@@ -563,20 +576,18 @@ static int unpatch_mcount_func(struct mcount_dynamic_info *mdi, struct sym *sym)
 	return INSTRUMENT_SKIPPED;
 }
 
-int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
-		      struct mcount_disasm_engine *disasm,
-		      unsigned min_size)
+int mcount_patch_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym,
+		      struct mcount_disasm_engine *disasm, unsigned min_size)
 {
-	struct arch_dynamic_info *adi = mdi->arch;
 	int result = INSTRUMENT_SKIPPED;
 
-	if (min_size < CALL_INSN_SIZE)
-		min_size = CALL_INSN_SIZE;
+	if (min_size < CALL_INSN_SIZE + 1)
+		min_size = CALL_INSN_SIZE + 1;
 
 	if (sym->size < min_size)
 		return result;
 
-	switch (adi->type) {
+	switch (mdi->type) {
 	case DYNAMIC_XRAY:
 		result = patch_xray_func(mdi, sym);
 		break;
@@ -585,6 +596,10 @@ int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 		result = patch_fentry_func(mdi, sym);
 		break;
 
+	case DYNAMIC_PATCHABLE:
+		result = patch_patchable_func(mdi, sym);
+		break;
+
 	case DYNAMIC_NONE:
 		result = patch_normal_func(mdi, sym, disasm);
 		break;
@@ -595,13 +610,12 @@ int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 	return result;
 }
 
-int mcount_unpatch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
+int mcount_unpatch_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym,
 			struct mcount_disasm_engine *disasm)
 {
-	struct arch_dynamic_info *adi = mdi->arch;
 	int result = INSTRUMENT_SKIPPED;
 
-	switch (adi->type) {
+	switch (mdi->type) {
 	case DYNAMIC_FENTRY:
 		result = unpatch_fentry_func(mdi, sym);
 		break;
@@ -616,12 +630,16 @@ int mcount_unpatch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 	return result;
 }
 
-static void revert_normal_func(struct mcount_dynamic_info *mdi, struct sym *sym,
+static void revert_normal_func(struct mcount_dynamic_info *mdi, struct uftrace_symbol *sym,
 			       struct mcount_disasm_engine *disasm)
 {
 	void *addr = (void *)(uintptr_t)sym->addr + mdi->map->start;
+	uint8_t endbr64[] = { 0xf3, 0x0f, 0x1e, 0xfa };
 	struct mcount_orig_insn *moi;
 
+	if (!memcmp(addr, endbr64, sizeof(endbr64)))
+		addr += sizeof(endbr64);
+
 	moi = mcount_find_insn((uintptr_t)addr + CALL_INSN_SIZE);
 	if (moi == NULL)
 		return;
@@ -643,3 +661,75 @@ void mcount_arch_dynamic_recover(struct mcount_dynamic_info *mdi,
 		free(badsym);
 	}
 }
+
+static bool addr_in_prologue(struct mcount_disasm_info *info, unsigned long addr)
+{
+	return info->addr <= addr && addr < (info->addr + info->orig_size);
+}
+
+int mcount_arch_branch_table_size(struct mcount_disasm_info *info)
+{
+	struct cond_branch_info *jcc_info;
+	int count = 0;
+	int i;
+
+	for (i = 0; i < info->nr_branch; i++) {
+		jcc_info = &info->branch_info[i];
+
+		/* no need to allocate entry for jcc that jump directly to prologue */
+		if (addr_in_prologue(info, jcc_info->branch_target))
+			continue;
+
+		count++;
+	}
+	return count * ARCH_BRANCH_ENTRY_SIZE;
+}
+
+void mcount_arch_patch_branch(struct mcount_disasm_info *info, struct mcount_orig_insn *orig)
+{
+	/*
+	 * The first entry in the table starts right after the out-of-line
+	 * execution buffer.
+	 */
+	uint64_t entry_offset = orig->insn_size;
+	uint8_t trampoline[ARCH_TRAMPOLINE_SIZE] = {
+		0x3e,
+		0xff,
+		0x25,
+	};
+	struct cond_branch_info *jcc_info;
+	unsigned long jcc_target;
+	unsigned long jcc_index;
+	uint32_t disp;
+	int i;
+
+	for (i = 0; i < info->nr_branch; i++) {
+		jcc_info = &info->branch_info[i];
+		jcc_target = jcc_info->branch_target;
+		jcc_index = jcc_info->insn_index;
+
+		/* leave the original disp of jcc that target the prologue as it is */
+		if (addr_in_prologue(info, jcc_target)) {
+			jcc_target -= jcc_info->insn_addr + jcc_info->insn_size;
+			info->insns[jcc_index + 1] = jcc_target;
+			continue;
+		}
+
+		/* setup the branch entry trampoline */
+		memcpy(trampoline + CET_JMP_INSN_SIZE, &jcc_target, sizeof(jcc_target));
+
+		/* write the entry to the branch table */
+		memcpy(orig->insn + entry_offset, trampoline, sizeof(trampoline));
+
+		/* previously, all jcc32 are downgraded to jcc8 */
+		disp = entry_offset - (jcc_index + JCC8_INSN_SIZE);
+		if (disp > SCHAR_MAX) { /* should not happen */
+			pr_err("target is not in reach");
+		}
+
+		/* patch jcc displacement to target corresponding entry in the table */
+		info->insns[jcc_index + 1] = disp;
+
+		entry_offset += ARCH_BRANCH_ENTRY_SIZE;
+	}
+}
