@@ -4,30 +4,30 @@
 #include <sys/mman.h>
 
 /* This should be defined before #include "utils.h" */
-#define PR_FMT     "mcount"
-#define PR_DOMAIN  DBG_MCOUNT
+#define PR_FMT "mcount"
+#define PR_DOMAIN DBG_MCOUNT
 
-#include "uftrace.h"
-#include "libmcount/mcount.h"
 #include "libmcount/internal.h"
-#include "utils/utils.h"
+#include "libmcount/mcount.h"
+#include "uftrace.h"
 #include "utils/symbol.h"
+#include "utils/utils.h"
 
-#define TRAMP_ENT_SIZE    16  /* size of trampoilne for each entry */
-#define TRAMP_PLT0_SIZE   32  /* module id + address of plthook_addr() */
-#define TRAMP_PCREL_JMP   10  /* PC_relative offset for JMP */
-#define TRAMP_IDX_OFFSET  1
-#define TRAMP_JMP_OFFSET  6
+#define TRAMP_ENT_SIZE 16 /* size of trampoilne for each entry */
+#define TRAMP_PLT0_SIZE 32 /* module id + address of plthook_addr() */
+#define TRAMP_PCREL_JMP 10 /* PC_relative offset for JMP */
+#define TRAMP_IDX_OFFSET 1
+#define TRAMP_JMP_OFFSET 6
 
 extern void __weak plt_hooker(void);
-struct plthook_data * mcount_arch_hook_no_plt(struct uftrace_elf_data *elf,
-					      const char *modname,
-					      unsigned long offset)
+struct plthook_data *mcount_arch_hook_no_plt(struct uftrace_elf_data *elf, const char *modname,
+					     unsigned long offset)
 {
 	struct plthook_data *pd;
 	void *trampoline;
 	size_t tramp_len;
 	uint32_t i;
+	/* clang-format off */
 	const uint8_t tramp_plt0[] = {  /* followed by module_id + plthook_addr */
 		/* PUSH module_id */
 		0xff, 0x35, 0xa, 0, 0, 0,
@@ -43,6 +43,7 @@ struct plthook_data * mcount_arch_hook_no_plt(struct uftrace_elf_data *elf,
 		/* should never reach here */
 		0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc,
 	};
+	/* clang-format on */
 	void *plthook_addr = plt_hooker;
 	void *tramp;
 
@@ -50,15 +51,14 @@ struct plthook_data * mcount_arch_hook_no_plt(struct uftrace_elf_data *elf,
 	pd->module_id = (unsigned long)pd;
 	pd->base_addr = offset;
 
-	if (load_elf_dynsymtab(&pd->dsymtab, elf, offset, 0) < 0 ||
-	    pd->dsymtab.nr_sym == 0) {
+	if (load_elf_dynsymtab(&pd->dsymtab, elf, offset, 0) < 0 || pd->dsymtab.nr_sym == 0) {
 		free(pd);
 		return NULL;
 	}
 
 	tramp_len = TRAMP_PLT0_SIZE + pd->dsymtab.nr_sym * TRAMP_ENT_SIZE;
-	trampoline = mmap(NULL, tramp_len, PROT_READ|PROT_WRITE,
-			  MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
+	trampoline =
+		mmap(NULL, tramp_len, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (trampoline == MAP_FAILED) {
 		pr_dbg("mmap failed: %m: ignore libcall hooking\n");
 		free(pd);
@@ -69,8 +69,8 @@ struct plthook_data * mcount_arch_hook_no_plt(struct uftrace_elf_data *elf,
 	pd->resolved_addr = xcalloc(pd->dsymtab.nr_sym, sizeof(long));
 
 	/* add trampoline - save orig addr and replace GOT */
-	pr_dbg2("module: %s (id: %lx), addr = %lx, TRAMPOLINE = %p\n",
-		pd->mod_name, pd->module_id, pd->base_addr, pd->pltgot_ptr);
+	pr_dbg2("module: %s (id: %lx), addr = %lx, TRAMPOLINE = %p\n", pd->mod_name, pd->module_id,
+		pd->base_addr, pd->pltgot_ptr);
 
 	/* setup PLT0 */
 	memcpy(trampoline, tramp_plt0, sizeof(tramp_plt0));
@@ -83,7 +83,7 @@ struct plthook_data * mcount_arch_hook_no_plt(struct uftrace_elf_data *elf,
 	for (i = 0; i < pd->dsymtab.nr_sym; i++) {
 		uint32_t pcrel;
 		Elf64_Rela *rela;
-		struct sym *sym;
+		struct uftrace_symbol *sym;
 		unsigned k;
 		bool skip = false;
 
@@ -108,17 +108,16 @@ struct plthook_data * mcount_arch_hook_no_plt(struct uftrace_elf_data *elf,
 		pcrel = trampoline - (tramp + TRAMP_PCREL_JMP);
 		memcpy(tramp + TRAMP_JMP_OFFSET, &pcrel, sizeof(pcrel));
 
-		rela = (void*)sym->addr;
+		rela = (void *)sym->addr;
 		/* save resolved address in GOT */
-		memcpy(&pd->resolved_addr[i], (void *)rela->r_offset + offset,
-		       sizeof(long));
+		memcpy(&pd->resolved_addr[i], (void *)rela->r_offset + offset, sizeof(long));
 		/* update GOT to point the trampoline */
 		memcpy((void *)rela->r_offset + offset, &tramp, sizeof(long));
 
 		tramp += TRAMP_ENT_SIZE;
 	}
 
-	mprotect(trampoline, tramp_len, PROT_READ|PROT_EXEC);
+	mprotect(trampoline, tramp_len, PROT_READ | PROT_EXEC);
 
 	pd->mod_name = xstrdup(modname);
 
