@@ -1,17 +1,16 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <signal.h>
 #include <inttypes.h>
+#include <signal.h>
+#include <stdio.h>
 #include <stdio_ext.h>
-#include <assert.h>
+#include <stdlib.h>
 
 #include "uftrace.h"
-#include "utils/utils.h"
-#include "utils/symbol.h"
+#include "utils/field.h"
 #include "utils/filter.h"
 #include "utils/fstack.h"
-#include "utils/field.h"
 #include "utils/graph.h"
+#include "utils/symbol.h"
+#include "utils/utils.h"
 
 static LIST_HEAD(output_fields);
 static LIST_HEAD(output_task_fields);
@@ -68,42 +67,42 @@ static void print_addr(struct field_data *fd)
 	/* uftrace records (truncated) 48-bit addresses */
 	int width = sizeof(long) == 4 ? 8 : 12;
 
-	pr_out("%*"PRIx64, width, effective_addr(node->addr));
+	pr_out("%*" PRIx64, width, effective_addr(node->addr));
 }
 
-static struct display_field field_total_time= {
-	.id      = GRAPH_F_TOTAL_TIME,
-	.name    = "total-time",
-	.alias   = "total",
-	.header  = "TOTAL TIME",
-	.length  = 10,
-	.print   = print_total_time,
-	.list    = LIST_HEAD_INIT(field_total_time.list),
+static struct display_field field_total_time = {
+	.id = GRAPH_F_TOTAL_TIME,
+	.name = "total-time",
+	.alias = "total",
+	.header = "TOTAL TIME",
+	.length = 10,
+	.print = print_total_time,
+	.list = LIST_HEAD_INIT(field_total_time.list),
 };
 
-static struct display_field field_self_time= {
-	.id      = GRAPH_F_SELF_TIME,
-	.name    = "self-time",
-	.alias   = "self",
-	.header  = " SELF TIME",
-	.length  = 10,
-	.print   = print_self_time,
-	.list    = LIST_HEAD_INIT(field_self_time.list),
+static struct display_field field_self_time = {
+	.id = GRAPH_F_SELF_TIME,
+	.name = "self-time",
+	.alias = "self",
+	.header = " SELF TIME",
+	.length = 10,
+	.print = print_self_time,
+	.list = LIST_HEAD_INIT(field_self_time.list),
 };
 
 static struct display_field field_addr = {
-	.id      = GRAPH_F_ADDR,
-	.name    = "address",
-	.alias   = "addr",
+	.id = GRAPH_F_ADDR,
+	.name = "address",
+	.alias = "addr",
 #if __SIZEOF_LONG == 4
-	.header  = "  ADDR  ",
-	.length  = 8,
+	.header = "  ADDR  ",
+	.length = 8,
 #else
-	.header  = "   ADDRESS  ",
-	.length  = 12,
+	.header = "   ADDRESS  ",
+	.length = 12,
 #endif
-	.print   = print_addr,
-	.list    = LIST_HEAD_INIT(field_addr.list),
+	.print = print_addr,
+	.list = LIST_HEAD_INIT(field_addr.list),
 };
 
 static void print_task_total_time(struct field_data *fd)
@@ -133,32 +132,32 @@ static void print_task_tid(struct field_data *fd)
 }
 
 static struct display_field field_task_total_time = {
-	.id      = GRAPH_F_TASK_TOTAL_TIME,
-	.name    = "total-time",
-	.alias   = "total",
-	.header  = "TOTAL TIME",
-	.length  = 10,
-	.print   = print_task_total_time,
-	.list    = LIST_HEAD_INIT(field_task_total_time.list),
+	.id = GRAPH_F_TASK_TOTAL_TIME,
+	.name = "total-time",
+	.alias = "total",
+	.header = "TOTAL TIME",
+	.length = 10,
+	.print = print_task_total_time,
+	.list = LIST_HEAD_INIT(field_task_total_time.list),
 };
 
 static struct display_field field_task_self_time = {
-	.id      = GRAPH_F_TASK_SELF_TIME,
-	.name    = "self-time",
-	.alias   = "self",
-	.header  = " SELF TIME",
-	.length  = 10,
-	.print   = print_task_self_time,
-	.list    = LIST_HEAD_INIT(field_task_self_time.list),
+	.id = GRAPH_F_TASK_SELF_TIME,
+	.name = "self-time",
+	.alias = "self",
+	.header = " SELF TIME",
+	.length = 10,
+	.print = print_task_self_time,
+	.list = LIST_HEAD_INIT(field_task_self_time.list),
 };
 
 static struct display_field field_task_tid = {
-	.id      = GRAPH_F_TASK_TID,
-	.name    = "tid",
-	.header  = "   TID  ",
-	.length  = 8,
-	.print   = print_task_tid,
-	.list    = LIST_HEAD_INIT(field_task_tid.list),
+	.id = GRAPH_F_TASK_TID,
+	.name = "tid",
+	.header = "   TID  ",
+	.length = 8,
+	.print = print_task_tid,
+	.list = LIST_HEAD_INIT(field_task_tid.list),
 };
 
 /* index of this table should be matched to display_field_id */
@@ -175,12 +174,14 @@ static struct display_field *field_task_table[] = {
 	&field_task_tid,
 };
 
-static void setup_default_field(struct list_head *fields, struct opts *opts)
+static void setup_default_field(struct list_head *fields, struct uftrace_opts *opts,
+				struct display_field *p_field_table[])
 {
 	add_field(fields, field_table[GRAPH_F_TOTAL_TIME]);
 }
 
-static void setup_default_task_field(struct list_head *fields, struct opts *opts)
+static void setup_default_task_field(struct list_head *fields, struct uftrace_opts *opts,
+				     struct display_field *p_field_table[])
 {
 	add_field(fields, field_task_table[GRAPH_F_TASK_TOTAL_TIME]);
 	add_field(fields, field_task_table[GRAPH_F_TASK_SELF_TIME]);
@@ -211,8 +212,7 @@ static int create_graph(struct uftrace_session *sess, void *func)
 {
 	struct session_graph *graph = xzalloc(sizeof(*graph));
 
-	pr_dbg("create graph for session %.*s (%s)\n",
-	       SESSION_ID_LEN, sess->sid, sess->exename);
+	pr_dbg("create graph for session %.*s (%s)\n", SESSION_ID_LEN, sess->sid, sess->exename);
 
 	graph->func = xstrdup(full_graph ? basename(sess->exename) : func);
 	INIT_LIST_HEAD(&graph->bt_list);
@@ -226,8 +226,7 @@ static int create_graph(struct uftrace_session *sess, void *func)
 	return 0;
 }
 
-static void setup_graph_list(struct uftrace_data *handle, struct opts *opts,
-			     char *func)
+static void setup_graph_list(struct uftrace_data *handle, struct uftrace_opts *opts, char *func)
 {
 	struct session_graph *graph;
 
@@ -240,8 +239,8 @@ static void setup_graph_list(struct uftrace_data *handle, struct opts *opts,
 	}
 }
 
-static struct uftrace_graph * get_graph(struct uftrace_task_reader *task,
-					uint64_t time, uint64_t addr)
+static struct uftrace_graph *get_graph(struct uftrace_task_reader *task, uint64_t time,
+				       uint64_t addr)
 {
 	struct session_graph *graph;
 	struct uftrace_session_link *sessions = &task->h->sessions;
@@ -251,7 +250,7 @@ static struct uftrace_graph * get_graph(struct uftrace_task_reader *task,
 	if (sess == NULL) {
 		struct uftrace_session *fsess = sessions->first;
 
-		if (is_kernel_address(&fsess->symtabs, addr))
+		if (is_kernel_address(&fsess->sym_info, addr))
 			sess = fsess;
 		else
 			return NULL;
@@ -269,8 +268,8 @@ static struct uftrace_graph * get_graph(struct uftrace_task_reader *task,
 
 static int start_graph(struct task_graph *tg);
 
-static struct task_graph * get_task_graph(struct uftrace_task_reader *task,
-					  uint64_t time, uint64_t addr)
+static struct task_graph *get_task_graph(struct uftrace_task_reader *task, uint64_t time,
+					 uint64_t addr)
 {
 	struct task_graph *tg;
 	struct uftrace_graph *graph;
@@ -280,8 +279,7 @@ static struct task_graph * get_task_graph(struct uftrace_task_reader *task,
 	graph = get_graph(task, time, addr);
 
 	if (tg->utg.graph && tg->utg.graph != graph) {
-		pr_dbg("detect new session: %.*s\n",
-		       SESSION_ID_LEN, graph->sess->sid);
+		pr_dbg("detect new session: %.*s\n", SESSION_ID_LEN, graph->sess->sid);
 		tg->utg.new_sess = true;
 	}
 	tg->utg.graph = graph;
@@ -311,7 +309,7 @@ static int save_backtrace_addr(struct task_graph *tg)
 		return 0;
 
 	for (i = len - 1; i >= 0; i--) {
-		struct fstack *fstack = fstack_get(task, i + skip);
+		struct uftrace_fstack *fstack = fstack_get(task, i + skip);
 
 		if (fstack != NULL)
 			addrs[i] = fstack->addr;
@@ -320,8 +318,7 @@ static int save_backtrace_addr(struct task_graph *tg)
 	}
 
 	list_for_each_entry(bt, &graph->bt_list, list) {
-		if (len == bt->len &&
-		    !memcmp(addrs, bt->addr, len * sizeof(*addrs)))
+		if (len == bt->len && !memcmp(addrs, bt->addr, len * sizeof(*addrs)))
 			goto found;
 	}
 
@@ -344,7 +341,7 @@ found:
 static void save_backtrace_time(struct task_graph *tg)
 {
 	struct uftrace_task_reader *task = tg->utg.task;
-	struct fstack *fstack = fstack_get(task, task->stack_count);
+	struct uftrace_fstack *fstack = fstack_get(task, task->stack_count);
 
 	if (tg->bt_curr != NULL && fstack != NULL)
 		tg->bt_curr->time += fstack->total_time;
@@ -356,7 +353,7 @@ static int print_backtrace(struct session_graph *graph)
 {
 	int i = 0, k;
 	struct graph_backtrace *bt;
-	struct sym *sym;
+	struct uftrace_symbol *sym;
 	char *symname;
 
 	list_for_each_entry(bt, &graph->bt_list, list) {
@@ -365,10 +362,9 @@ static int print_backtrace(struct session_graph *graph)
 		pr_out("\n");
 
 		for (k = 0; k < bt->len; k++) {
-			sym = find_symtabs(&graph->ug.sess->symtabs, bt->addr[k]);
+			sym = find_symtabs(&graph->ug.sess->sym_info, bt->addr[k]);
 			if (sym == NULL)
-				sym = session_find_dlsym(graph->ug.sess,
-							 bt->time, bt->addr[k]);
+				sym = session_find_dlsym(graph->ug.sess, bt->time, bt->addr[k]);
 
 			symname = symbol_getname(sym, bt->addr[k]);
 			pr_out("   [%d] %s (%#lx)\n", k, symname, bt->addr[k]);
@@ -436,10 +432,8 @@ static void pr_indent(bool *indent_mask, int indent, bool line)
 	}
 }
 
-static void print_graph_node(struct uftrace_graph *graph,
-			     struct uftrace_graph_node *node,
-			     bool *indent_mask,
-			     int indent, bool needs_line)
+static void print_graph_node(struct uftrace_graph *graph, struct uftrace_graph_node *node,
+			     bool *indent_mask, int indent, bool needs_line)
 {
 	char *symname = node->name;
 	struct uftrace_graph_node *parent = node->parent;
@@ -454,10 +448,15 @@ static void print_graph_node(struct uftrace_graph *graph,
 	pr_indent(indent_mask, indent, needs_line);
 
 	/* FIXME: it should count fork+exec properly */
-	if (full_graph && node == &graph->root)
+	if (full_graph && node == &graph->root) {
 		pr_out("(%d) %s\n", 1, symname);
-	else
-		pr_out("(%d) %s\n", node->nr_calls, symname);
+	}
+	else {
+		pr_out("(%d) %s", node->nr_calls, symname);
+		if (node->loc)
+			pr_gray(" [%s:%d]", node->loc->file->name, node->loc->line);
+		pr_out("\n");
+	}
 
 	if (node->nr_edges > 1) {
 		pr_dbg2("add mask (%d) for %s\n", indent, symname);
@@ -465,8 +464,7 @@ static void print_graph_node(struct uftrace_graph *graph,
 	}
 
 	/* clear parent indent mask at the last node */
-	if (parent && parent->nr_edges > 1 && orig_indent > 0 &&
-	    parent->head.prev == &node->list)
+	if (parent && parent->nr_edges > 1 && orig_indent > 0 && parent->head.prev == &node->list)
 		indent_mask[orig_indent - 1] = false;
 
 	needs_line = (node->nr_edges > 1);
@@ -486,28 +484,34 @@ static void print_graph_node(struct uftrace_graph *graph,
 	pr_dbg2("del mask (%d) for %s\n", orig_indent, symname);
 }
 
-static int print_graph(struct session_graph *graph, struct opts *opts)
+static int print_graph(struct session_graph *graph, struct uftrace_opts *opts)
 {
 	bool *indent_mask;
 
 	/* skip empty graph */
-	if (list_empty(&graph->bt_list) && graph->ug.root.time == 0 &&
-	    graph->ug.root.nr_edges == 0)
+	if (list_empty(&graph->bt_list) && graph->ug.root.time == 0 && graph->ug.root.nr_edges == 0)
 		return 0;
 
-	pr_out("# Function Call Graph for '%s' (session: %.16s)\n",
-	       graph->func, graph->ug.sess->sid);
+	pr_out("# Function Call Graph for '%s' (session: %.16s)\n", graph->func,
+	       graph->ug.sess->sid);
 
 	if (!full_graph && !list_empty(&graph->bt_list)) {
 		pr_out("=============== BACKTRACE ===============\n");
 		print_backtrace(graph);
 	}
 
-	setup_field(&output_fields, opts, &setup_default_field, field_table, ARRAY_SIZE(field_table));
+	setup_field(&output_fields, opts, &setup_default_field, field_table,
+		    ARRAY_SIZE(field_table));
 
 	if (graph->ug.root.time || graph->ug.root.nr_edges) {
 		pr_out("========== FUNCTION CALL GRAPH ==========\n");
-		print_header(&output_fields, "# ", "FUNCTION", 2);
+		print_header(&output_fields, "# ", "FUNCTION", 2, false);
+		if (!list_empty(&output_fields)) {
+			if (opts->srcline)
+				pr_gray(" %s", "[SOURCE]");
+			pr_out("\n");
+		}
+
 		indent_mask = xcalloc(opts->max_stack, sizeof(*indent_mask));
 		print_graph_node(&graph->ug, &graph->ug.root, indent_mask, 0,
 				 graph->ug.root.nr_edges > 1);
@@ -517,19 +521,19 @@ static int print_graph(struct session_graph *graph, struct opts *opts)
 	return 1;
 }
 
-static void build_graph_node(struct opts *opts,
-			     struct uftrace_task_reader *task, uint64_t time,
-			     uint64_t addr, int type, char *func)
+static void build_graph_node(struct uftrace_opts *opts, struct uftrace_task_reader *task,
+			     uint64_t time, uint64_t addr, int type, char *func)
 {
 	struct task_graph *tg;
-	struct sym *sym = NULL;
+	struct uftrace_symbol *sym = NULL;
 	char *name;
+	struct uftrace_dbg_loc *loc = NULL;
 
 	tg = get_task_graph(task, time, addr);
 	if (unlikely(tg->utg.graph == NULL))
 		return;
 
-	sym = find_symtabs(&tg->utg.graph->sess->symtabs, addr);
+	sym = find_symtabs(&tg->utg.graph->sess->sym_info, addr);
 	if (sym == NULL)
 		sym = session_find_dlsym(tg->utg.graph->sess, time, addr);
 
@@ -540,8 +544,11 @@ static void build_graph_node(struct opts *opts,
 		goto out;
 
 	if (tg->enabled) {
-		graph_add_node(&tg->utg, type, name,
-			       sizeof(struct uftrace_graph_node));
+		if (opts->srcline) {
+			loc = task_find_loc_addr(&task->h->sessions, task, time, addr);
+		}
+
+		graph_add_node(&tg->utg, type, name, sizeof(struct uftrace_graph_node), loc);
 	}
 
 	/* cannot find a session for this record */
@@ -563,8 +570,7 @@ out:
 	symbol_putname(sym, name);
 }
 
-static void build_graph(struct opts *opts, struct uftrace_data *handle,
-		       char *func)
+static void build_graph(struct uftrace_opts *opts, struct uftrace_data *handle, char *func)
 {
 	struct uftrace_task_reader *task;
 	struct session_graph *graph;
@@ -585,7 +591,8 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 
 		if (frs->type == UFTRACE_EVENT) {
 			if (frs->addr != EVENT_ID_PERF_SCHED_IN &&
-			    frs->addr != EVENT_ID_PERF_SCHED_OUT)
+			    frs->addr != EVENT_ID_PERF_SCHED_OUT &&
+			    frs->addr != EVENT_ID_PERF_SCHED_OUT_PREEMPT)
 				continue;
 		}
 
@@ -593,7 +600,7 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 			struct uftrace_session *fsess;
 
 			fsess = task->h->sessions.first;
-			addr = get_kernel_address(&fsess->symtabs, addr);
+			addr = get_kernel_address(&fsess->sym_info, addr);
 		}
 
 		if (frs->type == UFTRACE_LOST) {
@@ -607,14 +614,13 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 
 			/* add partial duration of kernel functions before LOST */
 			while (task->stack_count >= task->user_stack_count) {
-				struct fstack *fstack;
+				struct uftrace_fstack *fstack;
 
 				fstack = fstack_get(task, task->stack_count);
 
 				if (fstack_enabled && fstack && fstack->valid &&
 				    !(fstack->flags & FSTACK_FL_NORECORD)) {
-					build_graph_node(opts, task, prev_time,
-							 fstack->addr,
+					build_graph_node(opts, task, prev_time, fstack->addr,
 							 UFTRACE_EXIT, func);
 				}
 
@@ -624,12 +630,10 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 
 			/* force to find a session for kernel function */
 			fsess = task->h->sessions.first;
-			tg = get_task_graph(task, prev_time,
-					    fsess->symtabs.kernel_base + 1);
+			tg = get_task_graph(task, prev_time, fsess->sym_info.kernel_base + 1);
 			tg->utg.lost = true;
 
-			if (tg->enabled && is_kernel_address(&fsess->symtabs,
-							     tg->utg.node->addr))
+			if (tg->enabled && is_kernel_address(&fsess->sym_info, tg->utg.node->addr))
 				pr_dbg("not returning to user after LOST\n");
 
 			continue;
@@ -642,12 +646,13 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 		prev_time = frs->time;
 
 		build_graph_node(opts, task, frs->time, addr, frs->type, func);
+		fstack_check_filter_done(task);
 	}
 
 	/* add duration of remaining functions */
 	for (i = 0; i < handle->nr_tasks; i++) {
 		uint64_t last_time;
-		struct fstack *fstack;
+		struct uftrace_fstack *fstack;
 
 		task = &handle->tasks[i];
 
@@ -677,8 +682,7 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 			if (task->stack_count > 0)
 				fstack[-1].child_time += fstack->total_time;
 
-			build_graph_node(opts, task, last_time, fstack->addr,
-					 UFTRACE_EXIT, func);
+			build_graph_node(opts, task, last_time, fstack->addr, UFTRACE_EXIT, func);
 		}
 	}
 
@@ -707,10 +711,10 @@ struct find_func_data {
 static int find_func(struct uftrace_session *s, void *arg)
 {
 	struct find_func_data *data = arg;
-	struct symtabs *symtabs = &s->symtabs;
+	struct uftrace_sym_info *sinfo = &s->sym_info;
 	struct uftrace_mmap *map;
 
-	for_each_map(symtabs, map) {
+	for_each_map(sinfo, map) {
 		if (map->mod == NULL)
 			continue;
 
@@ -723,8 +727,7 @@ static int find_func(struct uftrace_session *s, void *arg)
 	return data->found;
 }
 
-static void synthesize_depth_trigger(struct opts *opts,
-				     struct uftrace_data *handle,
+static void synthesize_depth_trigger(struct uftrace_opts *opts, struct uftrace_data *handle,
 				     char *func)
 {
 	size_t old_len = opts->trigger ? strlen(opts->trigger) : 0;
@@ -736,9 +739,8 @@ static void synthesize_depth_trigger(struct opts *opts,
 	walk_sessions(&handle->sessions, find_func, &ffd);
 
 	opts->trigger = xrealloc(opts->trigger, old_len + new_len);
-	snprintf(opts->trigger + old_len, new_len,
-		 "%s%s@%sdepth=%d", old_len ? ";" : "",
-		 func, ffd.found ? "" : "kernel,", opts->depth);
+	snprintf(opts->trigger + old_len, new_len, "%s%s@%sdepth=%d", old_len ? ";" : "", func,
+		 ffd.found ? "" : "kernel,", opts->depth);
 }
 
 static void reset_task_runtime(struct uftrace_data *handle)
@@ -756,7 +758,7 @@ static void reset_task_runtime(struct uftrace_data *handle)
 	}
 }
 
-static void graph_build_task(struct opts *opts, struct uftrace_data *handle)
+static void graph_build_task(struct uftrace_opts *opts, struct uftrace_data *handle)
 {
 	struct uftrace_task_reader *task;
 	struct uftrace_task *t;
@@ -787,6 +789,7 @@ static void graph_build_task(struct opts *opts, struct uftrace_data *handle)
 		if (frs->type == UFTRACE_EVENT) {
 			switch (frs->addr) {
 			case EVENT_ID_PERF_SCHED_OUT:
+			case EVENT_ID_PERF_SCHED_OUT_PREEMPT:
 				t->time.stamp = frs->time;
 				break;
 			case EVENT_ID_PERF_SCHED_IN:
@@ -796,6 +799,8 @@ static void graph_build_task(struct opts *opts, struct uftrace_data *handle)
 				break;
 			}
 		}
+
+		fstack_check_filter_done(task);
 	}
 
 	/* update task time if some records were missing */
@@ -813,7 +818,7 @@ static void graph_build_task(struct opts *opts, struct uftrace_data *handle)
 }
 
 /* returns true if any of child has more runtime than the filter */
-static bool check_time_filter(struct uftrace_task *task, struct opts *opts)
+static bool check_time_filter(struct uftrace_task *task, struct uftrace_opts *opts)
 {
 	struct uftrace_task *child;
 
@@ -826,27 +831,22 @@ static bool check_time_filter(struct uftrace_task *task, struct opts *opts)
 	return false;
 }
 
-static bool is_last_child(struct uftrace_task *task,
-			  struct uftrace_task *parent,
-			  struct opts *opts)
+static bool is_last_child(struct uftrace_task *task, struct uftrace_task *parent,
+			  struct uftrace_opts *opts)
 {
-	if (list_is_singular(&parent->children) ||
-	    parent->children.prev == &task->siblings)
+	if (list_is_singular(&parent->children) || parent->children.prev == &task->siblings)
 		return true;
 
 	/* any sibling satisfies the time filter? */
 	list_for_each_entry_continue(task, &parent->children, siblings) {
-		if (task->time.run >= opts->threshold ||
-		    check_time_filter(task, opts))
+		if (task->time.run >= opts->threshold || check_time_filter(task, opts))
 			return false;
 	}
 	return true;
 }
 
-static bool print_task_node(struct uftrace_task *task,
-			    struct uftrace_task *parent,
-			    bool *indent_mask, int indent,
-			    struct opts *opts)
+static bool print_task_node(struct uftrace_task *task, struct uftrace_task *parent,
+			    bool *indent_mask, int indent, struct uftrace_opts *opts)
 {
 	char *name = task->comm;
 	struct uftrace_task *child;
@@ -887,8 +887,7 @@ static bool print_task_node(struct uftrace_task *task,
 		 * In that case we should print the parent even if it's
 		 * shorter than the time filter to show a correct tree.
 		 */
-		if (opts->threshold > child->time.run &&
-		    !check_time_filter(child, opts))
+		if (opts->threshold > child->time.run && !check_time_filter(child, opts))
 			continue;
 
 		indent = orig_indent;
@@ -912,8 +911,7 @@ static bool print_task_node(struct uftrace_task *task,
 
 		blank |= print_task_node(child, task, indent_mask, indent, opts);
 
-		if (&child->siblings != task->children.prev &&
-		    child->pid != task->pid) {
+		if (&child->siblings != task->children.prev && child->pid != task->pid) {
 			/* print blank line after forked child */
 			blank = true;
 		}
@@ -922,7 +920,7 @@ static bool print_task_node(struct uftrace_task *task,
 	return blank;
 }
 
-static int graph_print_task(struct uftrace_data *handle, struct opts *opts)
+static int graph_print_task(struct uftrace_data *handle, struct uftrace_opts *opts)
 {
 	bool *indent_mask;
 	struct uftrace_task *task;
@@ -935,11 +933,11 @@ static int graph_print_task(struct uftrace_data *handle, struct opts *opts)
 
 	task = handle->sessions.first_task;
 
-	setup_field(&output_task_fields, opts, &setup_default_task_field,
-		    field_task_table, ARRAY_SIZE(field_task_table));
+	setup_field(&output_task_fields, opts, &setup_default_task_field, field_task_table,
+		    ARRAY_SIZE(field_task_table));
 
 	pr_out("========== TASK GRAPH ==========\n");
-	print_header(&output_task_fields, "# ", "TASK NAME", 2);
+	print_header(&output_task_fields, "# ", "TASK NAME", 2, true);
 
 	indent_mask = xcalloc(handle->nr_tasks, sizeof(*indent_mask));
 
@@ -952,7 +950,7 @@ static int graph_print_task(struct uftrace_data *handle, struct opts *opts)
 	return 1;
 }
 
-int command_graph(int argc, char *argv[], struct opts *opts)
+int command_graph(int argc, char *argv[], struct uftrace_opts *opts)
 {
 	int ret;
 	struct uftrace_data handle;
@@ -987,6 +985,9 @@ int command_graph(int argc, char *argv[], struct opts *opts)
 
 	fstack_setup_filters(opts, &handle);
 
+	if (format_mode == FORMAT_HTML)
+		pr_out(HTML_HEADER);
+
 	if (opts->show_task) {
 		graph_build_task(opts, &handle);
 		graph_print_task(&handle, opts);
@@ -1022,6 +1023,9 @@ int command_graph(int argc, char *argv[], struct opts *opts)
 	graph_remove_task();
 
 out:
+	if (format_mode == FORMAT_HTML)
+		pr_out(HTML_FOOTER);
+
 	close_data_file(opts, &handle);
 
 	return 0;
