@@ -1,36 +1,35 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdint.h>
-#include <inttypes.h>
-#include <unistd.h>
+#include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <glob.h>
+#include <inttypes.h>
 #include <poll.h>
-#include <assert.h>
-#include <dirent.h>
 #include <pthread.h>
 #include <signal.h>
-#include <glob.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/epoll.h>
+#include <sys/eventfd.h>
+#include <sys/ioctl.h>
 #include <sys/mman.h>
+#include <sys/personality.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/ioctl.h>
-#include <sys/eventfd.h>
-#include <sys/resource.h>
-#include <sys/epoll.h>
-#include <sys/personality.h>
+#include <unistd.h>
 
-#include "uftrace.h"
 #include "libmcount/mcount.h"
-#include "utils/utils.h"
-#include "utils/symbol.h"
-#include "utils/list.h"
+#include "uftrace.h"
 #include "utils/filter.h"
 #include "utils/kernel.h"
+#include "utils/list.h"
 #include "utils/perf.h"
+#include "utils/symbol.h"
+#include "utils/utils.h"
 
 #ifndef EFD_SEMAPHORE
-# define EFD_SEMAPHORE (1 << 0)
+#define EFD_SEMAPHORE (1 << 0)
 #endif
 #define SHMEM_NAME_SIZE (64 - (int)sizeof(struct list_head))
 
@@ -63,17 +62,16 @@ static bool has_perf_event;
 static bool has_sched_event;
 static bool finish_received;
 
-static bool can_use_fast_libmcount(struct opts *opts)
+static bool can_use_fast_libmcount(struct uftrace_opts *opts)
 {
 	if (debug)
 		return false;
 	if (opts->depth != MCOUNT_DEFAULT_DEPTH)
 		return false;
-	if (getenv("UFTRACE_FILTER")    || getenv("UFTRACE_TRIGGER") ||
-	    getenv("UFTRACE_ARGUMENT")  || getenv("UFTRACE_RETVAL") ||
-	    getenv("UFTRACE_PATCH")     || getenv("UFTRACE_SCRIPT") ||
-	    getenv("UFTRACE_AUTO_ARGS") || getenv("UFTRACE_WATCH") ||
-	    getenv("UFTRACE_CALLER")    || getenv("UFTRACE_SIGNAL"))
+	if (getenv("UFTRACE_FILTER") || getenv("UFTRACE_TRIGGER") || getenv("UFTRACE_ARGUMENT") ||
+	    getenv("UFTRACE_RETVAL") || getenv("UFTRACE_PATCH") || getenv("UFTRACE_SCRIPT") ||
+	    getenv("UFTRACE_AUTO_ARGS") || getenv("UFTRACE_WATCH") || getenv("UFTRACE_CALLER") ||
+	    getenv("UFTRACE_SIGNAL") || getenv("UFTRACE_AGENT") || getenv("UFTRACE_LOCATION"))
 		return false;
 	return true;
 }
@@ -81,7 +79,7 @@ static bool can_use_fast_libmcount(struct opts *opts)
 static char *build_debug_domain_string(void)
 {
 	int i, d;
-	static char domain[2*DBG_DOMAIN_MAX + 1];
+	static char domain[2 * DBG_DOMAIN_MAX + 1];
 
 	for (i = 0, d = 0; d < DBG_DOMAIN_MAX; d++) {
 		if (dbg_domain[d]) {
@@ -94,11 +92,10 @@ static char *build_debug_domain_string(void)
 	return domain;
 }
 
-char * get_libmcount_path(struct opts *opts)
+char *get_libmcount_path(struct uftrace_opts *opts)
 {
 	char *libmcount, *lib = xmalloc(PATH_MAX);
-	bool must_use_multi_thread = has_dependency(opts->exename,
-						    "libpthread.so.0");
+	bool must_use_multi_thread = has_dependency(opts->exename, "libpthread.so.0");
 
 	if (opts->nop) {
 		libmcount = "libmcount-nop.so";
@@ -148,7 +145,7 @@ void put_libmcount_path(char *libpath)
 	free(libpath);
 }
 
-static void setup_child_environ(struct opts *opts, int argc, char *argv[])
+static void setup_child_environ(struct uftrace_opts *opts, int argc, char *argv[])
 {
 	char buf[PATH_MAX];
 	char *old_preload, *libpath;
@@ -178,6 +175,16 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 		}
 	}
 
+	if (opts->loc_filter) {
+		char *loc_str = uftrace_clear_kernel(opts->loc_filter);
+
+		if (loc_str) {
+			setenv("UFTRACE_LOCATION", loc_str, 1);
+			setenv("UFTRACE_SRCLINE", "1", 1);
+			free(loc_str);
+		}
+	}
+
 	if (opts->trigger) {
 		char *trigger_str = uftrace_clear_kernel(opts->trigger);
 
@@ -215,11 +222,11 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 			setenv("UFTRACE_PATCH", patch_str, 1);
 			free(patch_str);
 		}
+	}
 
-		if (opts->size_filter) {
-			snprintf(buf, sizeof(buf), "%d", opts->size_filter);
-			setenv("UFTRACE_PATCH_SIZE", buf, 1);
-		}
+	if (opts->size_filter) {
+		snprintf(buf, sizeof(buf), "%d", opts->size_filter);
+		setenv("UFTRACE_MIN_SIZE", buf, 1);
 	}
 
 	if (opts->event) {
@@ -245,7 +252,7 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 	}
 
 	if (opts->threshold) {
-		snprintf(buf, sizeof(buf), "%"PRIu64, opts->threshold);
+		snprintf(buf, sizeof(buf), "%" PRIu64, opts->threshold);
 		setenv("UFTRACE_THRESHOLD", buf, 1);
 	}
 
@@ -302,8 +309,7 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 	snprintf(buf, sizeof(buf), "%d", demangler);
 	setenv("UFTRACE_DEMANGLE", buf, 1);
 
-	if ((opts->kernel || has_kernel_event(opts->event)) &&
-	    check_kernel_pid_filter())
+	if ((opts->kernel || has_kernel_event(opts->event)) && check_kernel_pid_filter())
 		setenv("UFTRACE_KERNEL_PID_UPDATE", "1", 1);
 
 	if (opts->script_file)
@@ -318,6 +324,18 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 	if (opts->srcline)
 		setenv("UFTRACE_SRCLINE", "1", 1);
 
+	if (opts->estimate_return)
+		setenv("UFTRACE_ESTIMATE_RETURN", "1", 1);
+
+	if (opts->clock)
+		setenv("UFTRACE_CLOCK", opts->clock, 1);
+
+	if (opts->with_syms)
+		setenv("UFTRACE_SYMBOL_DIR", opts->with_syms, 1);
+
+	if (opts->agent)
+		setenv("UFTRACE_AGENT", "1", 1);
+
 	if (argc > 0) {
 		char *args = NULL;
 		int i;
@@ -335,7 +353,7 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 
 	libpath = get_libmcount_path(opts);
 	if (libpath == NULL)
-		pr_err_ns("cannot found libmcount.so\n");
+		pr_err_ns("uftrace could not find libmcount.so for record-tracing\n");
 
 	pr_dbg("using %s library for tracing\n", libpath);
 
@@ -356,7 +374,7 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 	setenv("GLIBC_TUNABLES", "glibc.cpu.hwcaps=-IBT,-SHSTK", 1);
 }
 
-static uint64_t calc_feat_mask(struct opts *opts)
+static uint64_t calc_feat_mask(struct uftrace_opts *opts)
 {
 	uint64_t features = 0;
 	char *buf = NULL;
@@ -392,6 +410,9 @@ static uint64_t calc_feat_mask(struct opts *opts)
 	if (opts->event)
 		features |= EVENT;
 
+	if (opts->estimate_return)
+		features |= ESTIMATE_RETURN;
+
 	xasprintf(&buf, "%s/*.dbg", opts->dirname);
 	if (glob(buf, GLOB_NOSORT, NULL, &g) != GLOB_NOMATCH)
 		features |= DEBUG_INFO;
@@ -402,8 +423,8 @@ static uint64_t calc_feat_mask(struct opts *opts)
 	return features;
 }
 
-static int fill_file_header(struct opts *opts, int status, struct rusage *rusage,
-			    char *elapsed_time)
+int fill_file_header(struct uftrace_opts *opts, int status, struct rusage *rusage,
+		     char *elapsed_time)
 {
 	int fd, efd;
 	int ret = -1;
@@ -414,7 +435,7 @@ static int fill_file_header(struct opts *opts, int status, struct rusage *rusage
 	xasprintf(&filename, "%s/info", opts->dirname);
 	pr_dbg3("fill header (metadata) info in %s\n", filename);
 
-	fd = open(filename, O_WRONLY | O_CREAT| O_TRUNC, 0644);
+	fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644);
 	if (fd < 0)
 		pr_err("cannot open info file");
 
@@ -429,7 +450,7 @@ static int fill_file_header(struct opts *opts, int status, struct rusage *rusage
 	hdr.version = UFTRACE_FILE_VERSION;
 	hdr.header_size = sizeof(hdr);
 	hdr.endian = elf_ident[EI_DATA];
-	hdr.class = elf_ident[EI_CLASS];
+	hdr.elf_class = elf_ident[EI_CLASS];
 	hdr.feat_mask = calc_feat_mask(opts);
 	hdr.info_mask = 0;
 	hdr.max_stack = opts->max_stack;
@@ -439,8 +460,7 @@ static int fill_file_header(struct opts *opts, int status, struct rusage *rusage
 	if (write(fd, &hdr, sizeof(hdr)) != (int)sizeof(hdr))
 		pr_err("writing header info failed");
 
-	fill_uftrace_info(&hdr.info_mask, fd, opts, status,
-			  rusage, elapsed_time);
+	fill_uftrace_info(&hdr.info_mask, fd, opts, status, rusage, elapsed_time);
 
 try_write:
 	ret = pwrite(fd, &hdr, sizeof(hdr), 0);
@@ -466,7 +486,7 @@ close_fd:
 }
 
 /* size including NUL at the end */
-#define MSG_ID_SIZE  36
+#define MSG_ID_SIZE 36
 
 static void parse_msg_id(char *id, uint64_t *sid, int *tid, int *seq)
 {
@@ -477,7 +497,7 @@ static void parse_msg_id(char *id, uint64_t *sid, int *tid, int *seq)
 	/*
 	 * parse message id of "/uftrace-SESSION-TID-SEQ".
 	 */
-	if (sscanf(id, "/uftrace-%016"SCNx64"-%u-%03u", &_sid, &_tid, &_seq) != 3)
+	if (sscanf(id, "/uftrace-%016" SCNx64 "-%u-%03u", &_sid, &_tid, &_seq) != 3)
 		pr_err("parse msg id failed");
 
 	if (sid)
@@ -515,7 +535,7 @@ static void write_buffer_file(const char *dirname, struct buf_list *buf)
 	free(filename);
 }
 
-static void write_buffer(struct buf_list *buf, struct opts *opts, int sock)
+static void write_buffer(struct buf_list *buf, struct uftrace_opts *opts, int sock)
 {
 	struct mcount_shmem_buffer *shmbuf = buf->shmem_buf;
 
@@ -528,19 +548,19 @@ static void write_buffer(struct buf_list *buf, struct opts *opts, int sock)
 }
 
 struct writer_arg {
-	struct list_head		list;
-	struct list_head		bufs;
-	struct opts			*opts;
-	struct uftrace_kernel_writer	*kern;
-	struct uftrace_perf_writer	*perf;
-	int				sock;
-	int				idx;
-	int				tid;
-	int				nr_cpu;
-	int				cpus[];
+	struct list_head list;
+	struct list_head bufs;
+	struct uftrace_opts *opts;
+	struct uftrace_kernel_writer *kern;
+	struct uftrace_perf_writer *perf;
+	int sock;
+	int idx;
+	int tid;
+	int nr_cpu;
+	int cpus[];
 };
 
-static void write_buf_list(struct list_head *buf_head, struct opts *opts,
+static void write_buf_list(struct list_head *buf_head, struct uftrace_opts *opts,
 			   struct writer_arg *warg)
 {
 	struct buf_list *buf;
@@ -570,8 +590,8 @@ static void write_buf_list(struct list_head *buf_head, struct opts *opts,
 	pthread_mutex_unlock(&free_list_lock);
 }
 
-static int setup_pollfd(struct pollfd **pollfd, struct writer_arg *warg,
-			bool setup_perf, bool setup_kernel)
+static int setup_pollfd(struct pollfd **pollfd, struct writer_arg *warg, bool setup_perf,
+			bool setup_kernel)
 {
 	int nr_poll = 1;
 	struct pollfd *p;
@@ -608,9 +628,8 @@ static int setup_pollfd(struct pollfd **pollfd, struct writer_arg *warg,
 	return nr_poll;
 }
 
-static bool handle_pollfd(struct pollfd *pollfd, struct writer_arg *warg,
-			  bool trace_task, bool trace_perf, bool trace_kernel,
-			  int timeout)
+static bool handle_pollfd(struct pollfd *pollfd, struct writer_arg *warg, bool trace_task,
+			  bool trace_perf, bool trace_kernel, int timeout)
 {
 	int start = trace_task ? 0 : 1;
 	int nr_poll = trace_task ? 1 : 0;
@@ -632,15 +651,12 @@ static bool handle_pollfd(struct pollfd *pollfd, struct writer_arg *warg,
 		if (i == 0)
 			check_task = true;
 		else if (trace_perf && i < (warg->nr_cpu + 1)) {
-			record_perf_data(warg->perf, warg->cpus[i - 1],
-					 warg->sock);
+			record_perf_data(warg->perf, warg->cpus[i - 1], warg->sock);
 		}
 		else if (trace_kernel) {
 			int idx = i - (nr_poll - warg->nr_cpu);
 
-			record_kernel_trace_pipe(warg->kern,
-						 warg->cpus[idx],
-						 warg->sock);
+			record_kernel_trace_pipe(warg->kern, warg->cpus[idx], warg->sock);
 		}
 	}
 
@@ -656,7 +672,7 @@ void *writer_thread(void *arg)
 {
 	struct buf_list *buf, *pos;
 	struct writer_arg *warg = arg;
-	struct opts *opts = warg->opts;
+	struct uftrace_opts *opts = warg->opts;
 	struct pollfd *pollfd;
 	int i, dummy;
 	sigset_t sigset;
@@ -682,8 +698,7 @@ void *writer_thread(void *arg)
 		LIST_HEAD(head);
 		bool check_list = false;
 
-		check_list = handle_pollfd(pollfd, warg, true, has_perf_event,
-					   opts->kernel, 1000);
+		check_list = handle_pollfd(pollfd, warg, true, has_perf_event, opts->kernel, 1000);
 		if (!check_list)
 			continue;
 
@@ -698,8 +713,7 @@ void *writer_thread(void *arg)
 
 		if (!list_empty(&buf_write_list)) {
 			/* pick first unhandled buf  */
-			buf = list_first_entry(&buf_write_list,
-					       struct buf_list, list);
+			buf = list_first_entry(&buf_write_list, struct buf_list, list);
 			list_move(&buf->list, &head);
 
 			warg->tid = buf->tid;
@@ -731,8 +745,7 @@ void *writer_thread(void *arg)
 			if (!has_perf_event && !opts->kernel)
 				continue;
 
-			handle_pollfd(pollfd, warg, false, has_perf_event,
-				      opts->kernel, 0);
+			handle_pollfd(pollfd, warg, false, has_perf_event, opts->kernel, 0);
 		}
 	}
 	pr_dbg2("stop writer thread %d\n", warg->idx);
@@ -816,8 +829,7 @@ static void record_mmap_file(const char *dirname, char *sess_id, int bufsize)
 		return;
 	}
 
-	shmem_buf = mmap(NULL, bufsize, PROT_READ | PROT_WRITE,
-			 MAP_SHARED, fd, 0);
+	shmem_buf = mmap(NULL, bufsize, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
 	if (shmem_buf == MAP_FAILED)
 		pr_err("mmap shmem buffer");
 
@@ -828,8 +840,7 @@ static void record_mmap_file(const char *dirname, char *sess_id, int bufsize)
 			bool found = false;
 
 			if (!list_empty(&shmem_need_unlink)) {
-				sl = list_last_entry(&shmem_need_unlink,
-						     struct shmem_list, list);
+				sl = list_last_entry(&shmem_need_unlink, struct shmem_list, list);
 
 				/* length of "uftrace-<session id>-" is 25 */
 				if (!strncmp(sl->id, sess_id, 25))
@@ -862,7 +873,7 @@ static void stop_all_writers(void)
 	thread_ctl[1] = -1;
 }
 
-static void record_remaining_buffer(struct opts *opts, int sock)
+static void record_remaining_buffer(struct uftrace_opts *opts, int sock)
 {
 	struct buf_list *buf;
 
@@ -1193,8 +1204,7 @@ static void read_record_mmap(int pfd, const char *dirname, int bufsize)
 			 */
 			list_for_each_entry(tl, &tid_list_head, list) {
 				if (tl->tid == -1) {
-					pr_dbg3("override parent of daemon to %d\n",
-						tl->pid);
+					pr_dbg3("override parent of daemon to %d\n", tl->pid);
 					tmsg.pid = tl->pid;
 					break;
 				}
@@ -1285,8 +1295,7 @@ static int filter_map(const struct dirent *de)
 {
 	size_t len = strlen(de->d_name);
 
-	return !strncmp("sid-", de->d_name, 4) &&
-	       !strncmp(".map", de->d_name + len - 4, 4);
+	return !strncmp("sid-", de->d_name, 4) && !strncmp(".map", de->d_name + len - 4, 4);
 }
 
 static void send_map_files(int sock, const char *dirname)
@@ -1409,10 +1418,10 @@ static void send_log_file(int sock, const char *logfile)
 	if (access(logfile, F_OK) != 0)
 		return;
 
-	send_trace_metadata(sock, NULL, (char*)logfile);
+	send_trace_metadata(sock, NULL, (char *)logfile);
 }
 
-static void load_session_symbols(struct opts *opts)
+static void update_session_maps(struct uftrace_opts *opts)
 {
 	struct dirent **map_list;
 	int i, maps;
@@ -1425,9 +1434,32 @@ static void load_session_symbols(struct opts *opts)
 	}
 
 	for (i = 0; i < maps; i++) {
-		struct symtabs symtabs = {
-			.dirname  = opts->dirname,
-			.flags    = SYMTAB_FL_ADJ_OFFSET,
+		char buf[PATH_MAX];
+
+		snprintf(buf, sizeof(buf), "%s/%s", opts->dirname, map_list[i]->d_name);
+		update_session_map(buf);
+		free(map_list[i]);
+	}
+
+	free(map_list);
+}
+
+static void load_session_symbols(struct uftrace_opts *opts)
+{
+	struct dirent **map_list;
+	int i, maps;
+
+	maps = scandir(opts->dirname, &map_list, filter_map, alphasort);
+	if (maps <= 0) {
+		if (maps == 0)
+			errno = ENOENT;
+		pr_err("cannot find map files");
+	}
+
+	for (i = 0; i < maps; i++) {
+		struct uftrace_sym_info sinfo = {
+			.dirname = opts->dirname,
+			.flags = SYMTAB_FL_ADJ_OFFSET,
 		};
 		char sid[20];
 
@@ -1435,11 +1467,11 @@ static void load_session_symbols(struct opts *opts)
 		free(map_list[i]);
 
 		pr_dbg2("reading symbols for session %s\n", sid);
-		read_session_map(opts->dirname, &symtabs, sid);
+		read_session_map(opts->dirname, &sinfo, sid);
 
-		load_module_symtabs(&symtabs);
+		load_module_symtabs(&sinfo);
 
-		delete_session_map(&symtabs);
+		delete_session_map(&sinfo);
 	}
 
 	free(map_list);
@@ -1447,10 +1479,10 @@ static void load_session_symbols(struct opts *opts)
 
 static char *get_child_time(struct timespec *ts1, struct timespec *ts2)
 {
-#define SEC_TO_NSEC  (1000000000ULL)
+#define SEC_TO_NSEC (1000000000ULL)
 
 	char *elapsed_time = NULL;
-	uint64_t  sec = ts2->tv_sec  - ts1->tv_sec;
+	uint64_t sec = ts2->tv_sec - ts1->tv_sec;
 	uint64_t nsec = ts2->tv_nsec - ts1->tv_nsec;
 
 	if (nsec > SEC_TO_NSEC) {
@@ -1458,7 +1490,7 @@ static char *get_child_time(struct timespec *ts1, struct timespec *ts2)
 		sec--;
 	}
 
-	xasprintf(&elapsed_time, "%"PRIu64".%09"PRIu64" sec", sec, nsec);
+	xasprintf(&elapsed_time, "%" PRIu64 ".%09" PRIu64 " sec", sec, nsec);
 	return elapsed_time;
 }
 
@@ -1469,37 +1501,39 @@ static void print_child_time(char *elapsed_time)
 
 static void print_child_usage(struct rusage *ru)
 {
-	pr_out(" system time: %6lu.%06lu000 sec\n",
-	       ru->ru_stime.tv_sec, ru->ru_stime.tv_usec);
-	pr_out("   user time: %6lu.%06lu000 sec\n",
-	       ru->ru_utime.tv_sec, ru->ru_utime.tv_usec);
+	pr_out(" system time: %6lu.%06lu000 sec\n", ru->ru_stime.tv_sec, ru->ru_stime.tv_usec);
+	pr_out("   user time: %6lu.%06lu000 sec\n", ru->ru_utime.tv_sec, ru->ru_utime.tv_usec);
 }
 
-#define UFTRACE_MSG  "Cannot trace '%s': No such executable file.\n"
+#define UFTRACE_MSG "Cannot trace '%s': No such executable file.\n"
 
-#define MCOUNT_MSG  "Can't find '%s' symbol in the '%s'.\n"		\
-"\tIt seems not to be compiled with -pg or -finstrument-functions flag.\n" 	\
-"\tYou can rebuild your program with it or use -P option for dynamic tracing.\n"
+#define MCOUNT_MSG                                                                                 \
+	"Can't find '%s' symbol in the '%s'.\n"                                                    \
+	"\tIt seems not to be compiled with -pg or -finstrument-functions flag.\n"                 \
+	"\tYou can rebuild your program with it or use -P option for dynamic tracing.\n"
 
-#define UFTRACE_ELF_MSG  "Cannot trace '%s': Invalid file\n"		\
-"\tThis file doesn't look like an executable ELF file.\n"		\
-"\tPlease check whether it's a kind of script or shell functions.\n"
+#define UFTRACE_ELF_MSG                                                                            \
+	"Cannot trace '%s': Invalid file\n"                                                        \
+	"\tThis file doesn't look like an executable ELF file.\n"                                  \
+	"\tPlease check whether it's a kind of script or shell functions.\n"
 
-#define MACHINE_MSG  "Cannot trace '%s': Unsupported machine\n"		\
-"\tThis machine type (%u) is not supported currently.\n"		\
-"\tSorry about that!\n"
+#define MACHINE_MSG                                                                                \
+	"Cannot trace '%s': Unsupported machine\n"                                                 \
+	"\tThis machine type (%u) is not supported currently.\n"                                   \
+	"\tSorry about that!\n"
 
-#define ARGUMENT_MSG  "uftrace: -A or -R might not work for binaries"	\
-" with -finstrument-functions\n"
+#define ARGUMENT_MSG "uftrace: -A or -R might not work for binaries with -finstrument-functions\n"
 
-#define STATIC_MSG  "Cannot trace static binary: %s\n"			\
-"\tIt seems to be compiled with -static, rebuild the binary without it.\n"
+#define STATIC_MSG                                                                                 \
+	"Cannot trace static binary: %s\n"                                                         \
+	"\tIt seems to be compiled with -static, rebuild the binary without it.\n"
 
-#define SCRIPT_MSG  "Cannot trace script file: %s\n"			\
-"\tTo trace binaries run by the script, use --force option.\n"
+#define SCRIPT_MSG                                                                                 \
+	"Cannot trace script file: %s\n"                                                           \
+	"\tTo trace binaries run by the script, use --force option.\n"
 
-#ifndef  EM_AARCH64
-# define EM_AARCH64  183
+#ifndef EM_AARCH64
+#define EM_AARCH64 183
 #endif
 
 static bool is_regular_executable(const char *pathname)
@@ -1545,7 +1579,7 @@ static void find_in_path(char **exename)
 	strv_free(&strv);
 }
 
-static void check_binary(struct opts *opts)
+static void check_binary(struct uftrace_opts *opts)
 {
 	int fd;
 	int chk;
@@ -1553,9 +1587,7 @@ static void check_binary(struct opts *opts)
 	char elf_ident[EI_NIDENT];
 	uint16_t e_type;
 	uint16_t e_machine;
-	uint16_t supported_machines[] = {
-		EM_X86_64, EM_ARM, EM_AARCH64, EM_386
-	};
+	uint16_t supported_machines[] = { EM_X86_64, EM_ARM, EM_AARCH64, EM_386 };
 
 again:
 	/* if it cannot be found in PATH, then fails inside */
@@ -1636,7 +1668,7 @@ again:
 	close(fd);
 }
 
-static void check_perf_event(struct opts *opts)
+static void check_perf_event(struct uftrace_opts *opts)
 {
 	struct strv strv = STRV_INIT;
 	char *evt;
@@ -1646,11 +1678,7 @@ static void check_perf_event(struct opts *opts)
 
 	has_perf_event = has_sched_event = !opts->no_event;
 
-	/*
-	 * caller filter focuses onto a given function,
-	 * displaying sched event with it is annoying.
-	 */
-	if (opts->caller != NULL)
+	if (opts->no_sched)
 		has_sched_event = false;
 
 	if (opts->event == NULL)
@@ -1686,19 +1714,19 @@ static void check_perf_event(struct opts *opts)
 }
 
 struct writer_data {
-	int				pid;
-	int				pipefd;
-	int				sock;
-	int				nr_cpu;
-	int				status;
-	pthread_t			*writers;
-	struct timespec			ts1, ts2;
-	struct rusage			usage;
-	struct uftrace_kernel_writer	kernel;
-	struct uftrace_perf_writer	perf;
+	int pid;
+	int pipefd;
+	int sock;
+	int nr_cpu;
+	int status;
+	pthread_t *writers;
+	struct timespec ts1, ts2;
+	struct rusage usage;
+	struct uftrace_kernel_writer kernel;
+	struct uftrace_perf_writer perf;
 };
 
-static void setup_writers(struct writer_data *wd, struct opts *opts)
+static void setup_writers(struct writer_data *wd, struct uftrace_opts *opts)
 {
 	struct uftrace_kernel_writer *kernel = &wd->kernel;
 	struct uftrace_perf_writer *perf = &wd->perf;
@@ -1742,6 +1770,7 @@ static void setup_writers(struct writer_data *wd, struct opts *opts)
 		kernel->output_dir = opts->dirname;
 		kernel->depth = opts->kernel_depth ?: 1;
 		kernel->bufsize = opts->kernel_bufsize;
+		kernel->clock = opts->clock;
 
 		if (!opts->nr_thread) {
 			if (opts->kernel_depth >= 4)
@@ -1765,8 +1794,7 @@ static void setup_writers(struct writer_data *wd, struct opts *opts)
 				pr_warn("kernel tracing requires root privilege\n");
 			else
 				pr_warn("kernel tracing disabled due to an error\n"
-				        "is CONFIG_FUNCTION_GRAPH_TRACER enabled in the kernel?\n");
-
+					"is CONFIG_FUNCTION_GRAPH_TRACER enabled in the kernel?\n");
 
 			opts->kernel = false;
 		}
@@ -1778,8 +1806,9 @@ static void setup_writers(struct writer_data *wd, struct opts *opts)
 		opts->nr_thread = wd->nr_cpu;
 
 	if (has_perf_event) {
-		if (setup_perf_record(perf, wd->nr_cpu, wd->pid,
-				      opts->dirname, has_sched_event) < 0)
+		setup_clock_id(opts->clock);
+		if (setup_perf_record(perf, wd->nr_cpu, wd->pid, opts->dirname, has_sched_event) <
+		    0)
 			has_perf_event = false;
 	}
 
@@ -1791,7 +1820,7 @@ out:
 		pr_err("cannot create an eventfd for writer thread");
 }
 
-static void start_tracing(struct writer_data *wd, struct opts *opts, int ready_fd)
+static void start_tracing(struct writer_data *wd, struct uftrace_opts *opts, int ready_fd)
 {
 	int i, k;
 	uint64_t go = 1;
@@ -1810,7 +1839,7 @@ static void start_tracing(struct writer_data *wd, struct opts *opts, int ready_f
 
 		warg = xzalloc(sizeof_warg);
 		warg->opts = opts;
-		warg->idx  = i;
+		warg->idx = i;
 		warg->sock = wd->sock;
 		warg->kern = &wd->kernel;
 		warg->perf = &wd->perf;
@@ -1837,7 +1866,7 @@ static void start_tracing(struct writer_data *wd, struct opts *opts, int ready_f
 		pr_err("signal to child failed");
 }
 
-static int stop_tracing(struct writer_data *wd, struct opts *opts)
+static int stop_tracing(struct writer_data *wd, struct uftrace_opts *opts)
 {
 	int status = -1;
 	int ret = UFTRACE_EXIT_SUCCESS;
@@ -1877,8 +1906,7 @@ static int stop_tracing(struct writer_data *wd, struct opts *opts)
 	if (child_exited) {
 		wait4(wd->pid, &status, 0, &wd->usage);
 		if (WIFEXITED(status)) {
-			pr_dbg("child terminated with exit code: %d\n",
-			       WEXITSTATUS(status));
+			pr_dbg("child terminated with exit code: %d\n", WEXITSTATUS(status));
 
 			if (WEXITSTATUS(status))
 				ret = UFTRACE_EXIT_FAILURE;
@@ -1886,13 +1914,12 @@ static int stop_tracing(struct writer_data *wd, struct opts *opts)
 				ret = UFTRACE_EXIT_SUCCESS;
 		}
 		else if (WIFSIGNALED(status)) {
-			pr_warn("child terminated by signal: %d: %s\n",
-				  WTERMSIG(status), strsignal(WTERMSIG(status)));
+			pr_warn("child terminated by signal: %d: %s\n", WTERMSIG(status),
+				strsignal(WTERMSIG(status)));
 			ret = UFTRACE_EXIT_SIGNALED;
 		}
 		else {
-			pr_warn("child terminated with unknown reason: %d\n",
-				  status);
+			pr_warn("child terminated with unknown reason: %d\n", status);
 			memset(&wd->usage, 0, sizeof(wd->usage));
 			ret = UFTRACE_EXIT_UNKNOWN;
 		}
@@ -1912,7 +1939,7 @@ static int stop_tracing(struct writer_data *wd, struct opts *opts)
 	return ret;
 }
 
-static void finish_writers(struct writer_data *wd, struct opts *opts)
+static void finish_writers(struct writer_data *wd, struct uftrace_opts *opts)
 {
 	int i;
 	char *elapsed_time = get_child_time(&wd->ts1, &wd->ts2);
@@ -1951,24 +1978,52 @@ static void finish_writers(struct writer_data *wd, struct opts *opts)
 		finish_perf_record(&wd->perf);
 }
 
-static void write_symbol_files(struct writer_data *wd, struct opts *opts)
+static void copy_data_files(struct uftrace_opts *opts, const char *ext)
+{
+	char path[PATH_MAX];
+	glob_t g;
+	size_t i;
+
+	snprintf(path, sizeof(path), "%s/*%s", opts->with_syms, ext);
+	glob(path, GLOB_NOSORT, NULL, &g);
+
+	for (i = 0; i < g.gl_pathc; i++) {
+		snprintf(path, sizeof(path), "%s/%s", opts->dirname, basename(g.gl_pathv[i]));
+		copy_file(g.gl_pathv[i], path);
+	}
+
+	globfree(&g);
+}
+
+static void write_symbol_files(struct writer_data *wd, struct uftrace_opts *opts)
 {
 	struct dlopen_list *dlib, *tmp;
 
 	if (opts->nop)
 		return;
 
+	/* add build-id info map files */
+	update_session_maps(opts);
+
+	if (opts->with_syms) {
+		copy_data_files(opts, ".sym");
+		copy_data_files(opts, ".dbg");
+		goto after_save;
+	}
+
 	/* main executable and shared libraries */
 	load_session_symbols(opts);
 
 	/* dynamically loaded libraries using dlopen() */
 	list_for_each_entry_safe(dlib, tmp, &dlopen_libs, list) {
-		struct symtabs dlib_symtabs = {
+		struct uftrace_sym_info dlib_sinfo = {
 			.dirname = opts->dirname,
 			.flags = SYMTAB_FL_ADJ_OFFSET,
 		};
+		char build_id[BUILD_ID_STR_SIZE];
 
-		load_module_symtab(&dlib_symtabs, dlib->libname);
+		read_build_id(dlib->libname, build_id, sizeof(build_id));
+		load_module_symtab(&dlib_sinfo, dlib->libname, build_id);
 
 		list_del(&dlib->list);
 
@@ -1979,6 +2034,7 @@ static void write_symbol_files(struct writer_data *wd, struct opts *opts)
 	save_module_symtabs(opts->dirname);
 	unload_module_symtabs();
 
+after_save:
 	if (opts->host) {
 		int sock = wd->sock;
 
@@ -2004,7 +2060,7 @@ static void write_symbol_files(struct writer_data *wd, struct opts *opts)
 		chown_directory(opts->dirname);
 }
 
-int do_main_loop(int ready, struct opts *opts, int pid)
+int do_main_loop(int ready, struct uftrace_opts *opts, int pid)
 {
 	int ret;
 	struct writer_data wd;
@@ -2065,8 +2121,7 @@ int do_main_loop(int ready, struct opts *opts, int pid)
 	return ret;
 }
 
-int do_child_exec(int ready, struct opts *opts,
-		  int argc, char *argv[])
+int do_child_exec(int ready, struct uftrace_opts *opts, int argc, char *argv[])
 {
 	uint64_t dummy;
 	char *shebang;
@@ -2127,7 +2182,7 @@ int do_child_exec(int ready, struct opts *opts,
 	abort();
 }
 
-int command_record(int argc, char *argv[], struct opts *opts)
+int command_record(int argc, char *argv[], struct uftrace_opts *opts)
 {
 	int pid;
 	int ready;
