@@ -1,35 +1,34 @@
-#include <stdbool.h>
-#include <unistd.h>
-#include <signal.h>
-#include <link.h>
-#include <sys/mman.h>
-#include <pthread.h>
-#include <assert.h>
 #include <dlfcn.h>
-#include <fnmatch.h>
 #include <errno.h>
+#include <fnmatch.h>
+#include <link.h>
+#include <pthread.h>
+#include <signal.h>
+#include <stdbool.h>
+#include <sys/mman.h>
+#include <unistd.h>
 
 /* This should be defined before #include "utils.h" */
-#define PR_FMT     "plthook"
-#define PR_DOMAIN  DBG_PLTHOOK
+#define PR_FMT "plthook"
+#define PR_DOMAIN DBG_PLTHOOK
 
-#include "libmcount/mcount.h"
 #include "libmcount/internal.h"
+#include "libmcount/mcount.h"
 #include "mcount-arch.h"
-#include "utils/utils.h"
 #include "utils/filter.h"
 #include "utils/script.h"
 #include "utils/symbol.h"
+#include "utils/utils.h"
 
-#ifndef  PT_GNU_RELRO
-# define PT_GNU_RELRO  0x6474e552  /* Read-only after relocation */
+#ifndef PT_GNU_RELRO
+#define PT_GNU_RELRO 0x6474e552 /* Read-only after relocation */
 #endif
 
 /* global symbol tables for libmcount */
 extern struct symtabs symtabs;
 
 /* address of dynamic linker's resolver routine (copied from GOT[2]) */
-unsigned long plthook_resolver_addr;	/* referenced by arch/.../plthook.S */
+unsigned long plthook_resolver_addr; /* referenced by arch/.../plthook.S */
 
 /* list of plthook_data for each library (module) */
 static LIST_HEAD(plthook_modules);
@@ -43,8 +42,7 @@ static void overwrite_pltgot(struct plthook_data *pd, int idx, void *data)
 	pd->pltgot_ptr[idx] = (unsigned long)data;
 }
 
-unsigned long setup_pltgot(struct plthook_data *pd, int got_idx, int sym_idx,
-			   void *data)
+unsigned long setup_pltgot(struct plthook_data *pd, int got_idx, int sym_idx, void *data)
 {
 	unsigned long real_addr = pd->pltgot_ptr[got_idx];
 
@@ -58,27 +56,31 @@ static void resolve_pltgot(struct plthook_data *pd, int idx)
 {
 	if (pd->resolved_addr[idx] == 0) {
 		unsigned long addr;
-		struct sym *sym;
+		struct uftrace_symbol *sym;
 
 		sym = &pd->dsymtab.sym[idx];
-		addr = (unsigned long) dlsym(RTLD_DEFAULT, sym->name);
+		addr = (unsigned long)dlsym(RTLD_DEFAULT, sym->name);
 
 		/* On ARM dlsym(DEFAULT) returns the address of PLT */
-		if (unlikely(pd->base_addr <= addr &&
-			     addr < sym->addr + sym->size)) {
+		if (unlikely(pd->base_addr <= addr && addr < sym->addr + sym->size)) {
 			void *real_addr = dlsym(RTLD_NEXT, sym->name);
 
 			if (real_addr)
 				addr = (unsigned long)real_addr;
 		}
 
-		pr_dbg2("resolved addr of %s = %#lx\n", sym->name, addr);
+		if (dbg_domain[DBG_PLTHOOK] >= 2) {
+			char *symname = demangle(sym->name);
+
+			pr_dbg2("resolved addr of %s = %#lx\n", symname, addr);
+			free(symname);
+		}
 		pd->resolved_addr[idx] = addr;
 	}
 }
 
 /* use weak reference for non-defined (arch-dependent) symbols */
-#define ALIAS_DECL(_sym)  extern __weak void (*uftrace_##_sym)(void);
+#define ALIAS_DECL(_sym) extern __weak void (*uftrace_##_sym)(void);
 
 ALIAS_DECL(mcount);
 ALIAS_DECL(_mcount);
@@ -87,7 +89,10 @@ ALIAS_DECL(__gnu_mcount_nc);
 ALIAS_DECL(__cyg_profile_func_enter);
 ALIAS_DECL(__cyg_profile_func_exit);
 
-#define SKIP_SYM(func)  { #func, &uftrace_ ## func }
+#define SKIP_SYM(func)                                                                             \
+	{                                                                                          \
+#func, &uftrace_##func                                                             \
+	}
 
 const struct plthook_skip_symbol plt_skip_syms[] = {
 	SKIP_SYM(mcount),
@@ -109,7 +114,7 @@ size_t plt_skip_nr = ARRAY_SIZE(plt_skip_syms);
 static void restore_plt_functions(struct plthook_data *pd)
 {
 	unsigned i, k;
-	struct symtab *dsymtab = &pd->dsymtab;
+	struct uftrace_symtab *dsymtab = &pd->dsymtab;
 
 	for (i = 0; i < dsymtab->nr_sym; i++) {
 		/*
@@ -123,7 +128,7 @@ static void restore_plt_functions(struct plthook_data *pd)
 		bool skipped = false;
 		unsigned long plthook_addr;
 		unsigned long resolved_addr;
-		struct sym *sym = dsymtab->sym_names[i];
+		struct uftrace_symbol *sym = dsymtab->sym_names[i];
 
 		for (k = 0; k < plt_skip_nr; k++) {
 			const struct plthook_skip_symbol *skip_sym;
@@ -133,8 +138,8 @@ static void restore_plt_functions(struct plthook_data *pd)
 				continue;
 
 			overwrite_pltgot(pd, got_idx, skip_sym->addr);
-			pr_dbg2("overwrite GOT[%d] to %p (%s)\n",
-				got_idx, skip_sym->addr, skip_sym->name);
+			pr_dbg2("overwrite GOT[%d] to %p (%s)\n", got_idx, skip_sym->addr,
+				skip_sym->name);
 
 			skipped = true;
 			break;
@@ -145,12 +150,23 @@ static void restore_plt_functions(struct plthook_data *pd)
 		resolved_addr = pd->pltgot_ptr[got_idx];
 		plthook_addr = mcount_arch_plthook_addr(pd, i);
 		if (resolved_addr != plthook_addr) {
+			char *symname;
+
 			/* save already resolved address and hook it */
 			pd->resolved_addr[i] = resolved_addr;
 			overwrite_pltgot(pd, got_idx, (void *)plthook_addr);
-			pr_dbg2("restore GOT[%d] from \"%s\"(%#lx) to PLT(base + %#lx)\n",
-				got_idx, sym->name, resolved_addr,
-				plthook_addr - pd->base_addr);
+
+			if (dbg_domain[DBG_PLTHOOK] < 2)
+				continue;
+
+			symname = demangle(sym->name);
+			pr_dbg2("restore GOT[%d] from \"%s\"(%#lx) to PLT(base + %#lx)\n", got_idx,
+				symname, resolved_addr, plthook_addr - pd->base_addr);
+			free(symname);
+		}
+		else if (mcount_estimate_return) {
+			/* we can't resolve PLT functions at return. do it now */
+			resolve_pltgot(pd, i);
 		}
 	}
 }
@@ -159,22 +175,18 @@ extern void __weak plt_hooker(void);
 extern unsigned long plthook_return(void);
 
 __weak struct plthook_data *mcount_arch_hook_no_plt(struct uftrace_elf_data *elf,
-						    const char *modname,
-						    unsigned long offset)
+						    const char *modname, unsigned long offset)
 {
 	return NULL;
 }
 
-__weak void mcount_arch_plthook_setup(struct plthook_data *pd,
-				      struct uftrace_elf_data *elf)
+__weak void mcount_arch_plthook_setup(struct plthook_data *pd, struct uftrace_elf_data *elf)
 {
 	pd->arch = NULL;
 }
 
-static int find_got(struct uftrace_elf_data *elf,
-		    struct uftrace_elf_iter *iter,
-		    const char *modname,
-		    unsigned long offset)
+static int find_got(struct uftrace_elf_data *elf, struct uftrace_elf_iter *iter,
+		    const char *modname, unsigned long offset)
 {
 	bool plt_found = false;
 	unsigned long pltgot_addr = 0;
@@ -224,21 +236,21 @@ static int find_got(struct uftrace_elf_data *elf,
 	}
 
 	pd = xmalloc(sizeof(*pd));
-	pd->mod_name   = xstrdup(modname);
+	pd->mod_name = xstrdup(modname);
 	pd->pltgot_ptr = (void *)pltgot_addr;
-	pd->module_id  = pd->pltgot_ptr[1];
-	pd->base_addr  = offset;
-	pd->plt_addr   = plt_addr;
+	pd->module_id = pd->pltgot_ptr[1];
+	pd->base_addr = offset;
+	pd->plt_addr = plt_addr;
 
-	pr_dbg2("\"%s\" is loaded at %#lx\n",
-		basename(pd->mod_name), pd->base_addr);
+	pr_dbg2("\"%s\" is loaded at %#lx\n", basename(pd->mod_name), pd->base_addr);
 
 	memset(&pd->dsymtab, 0, sizeof(pd->dsymtab));
-	load_elf_dynsymtab(&pd->dsymtab, elf, pd->base_addr, SYMTAB_FL_DEMANGLE);
+	/* do not demangle symbol names since it might call dlsym() */
+	load_elf_dynsymtab(&pd->dsymtab, elf, pd->base_addr, 0);
 
 	pd->resolved_addr = xcalloc(pd->dsymtab.nr_sym, sizeof(long));
 	pd->special_funcs = NULL;
-	pd->nr_special    = 0;
+	pd->nr_special = 0;
 
 	mcount_arch_plthook_setup(pd, elf);
 	list_add_tail(&pd->list, &plthook_modules);
@@ -258,8 +270,7 @@ static int find_got(struct uftrace_elf_data *elf,
 
 	pr_dbg2("found GOT at %p (base_addr + %#lx)\n", pd->pltgot_ptr,
 		(unsigned long)pd->pltgot_ptr - pd->base_addr);
-	pr_dbg2("module id = %#lx, PLT resolver = %#lx\n",
-		pd->module_id, plthook_resolver_addr);
+	pr_dbg2("module id = %#lx, PLT resolver = %#lx\n", pd->module_id, plthook_resolver_addr);
 
 	restore_plt_functions(pd);
 
@@ -293,20 +304,19 @@ static int hook_pltgot(const char *modname, unsigned long offset)
 
 		if (iter.phdr.p_type == PT_GNU_RELRO) {
 			relro_start = iter.phdr.p_vaddr + offset;
-			relro_size  = iter.phdr.p_memsz;
+			relro_size = iter.phdr.p_memsz;
 
 			page_size = getpagesize();
 
 			relro_start &= ~(page_size - 1);
-			relro_size   = ALIGN(relro_size, page_size);
+			relro_size = ALIGN(relro_size, page_size);
 			relro = true;
 		}
 	}
 
 	if (found_dynamic) {
 		if (relro) {
-			mprotect((void *)relro_start, relro_size,
-				 PROT_READ | PROT_WRITE);
+			mprotect((void *)relro_start, relro_size, PROT_READ | PROT_WRITE);
 		}
 
 		ret = find_got(&elf, &iter, modname, offset);
@@ -321,14 +331,9 @@ static int hook_pltgot(const char *modname, unsigned long offset)
 
 /* functions should skip PLT hooking */
 static const char *skip_syms[] = {
-	"_mcleanup",
-	"__libc_start_main",
-	"__cxa_throw",
-	"__cxa_rethrow",
-	"__cxa_begin_catch",
-	"__cxa_end_catch",
-	"__cxa_finalize",
-	"_Unwind_Resume",
+	"_mcleanup",	  "__libc_start_main",	  "__cxa_throw",
+	"__cxa_rethrow",  "__cxa_begin_catch",	  "__cxa_end_catch",
+	"__cxa_finalize", "__gxx_personality_v0", "_Unwind_Resume",
 };
 
 static const char *setjmp_syms[] = {
@@ -354,10 +359,9 @@ static const char *dlsym_syms[] = {
 };
 
 static const char *flush_syms[] = {
-	"fork", "vfork", "daemon", "exit",
-	"longjmp", "siglongjmp", "__longjmp_chk",
-	"execl", "execlp", "execle", "execv", "execve", "execvp", "execvpe",
-	"fexecve", "posix_spawn", "posix_spawnp",
+	"fork",		 "vfork",   "daemon",  "exit",	      "longjmp",      "siglongjmp",
+	"__longjmp_chk", "execl",   "execlp",  "execle",      "execv",	      "execve",
+	"execvp",	 "execvpe", "fexecve", "posix_spawn", "posix_spawnp",
 };
 
 static const char *except_syms[] = {
@@ -365,8 +369,8 @@ static const char *except_syms[] = {
 };
 
 static const char *resolve_syms[] = {
-	"execl", "execlp", "execle", "execv", "execve", "execvp", "execvpe",
-	"fexecve", "posix_spawn", "posix_spawnp", "pthread_exit",
+	"execl",   "execlp",  "execle",	     "execv",	     "execve",	     "execvp",
+	"execvpe", "fexecve", "posix_spawn", "posix_spawnp", "pthread_exit",
 };
 
 static void add_special_func(struct plthook_data *pd, unsigned idx, unsigned flags)
@@ -383,17 +387,16 @@ static void add_special_func(struct plthook_data *pd, unsigned idx, unsigned fla
 		}
 	}
 
-	pd->special_funcs = xrealloc(pd->special_funcs,
-				     (pd->nr_special + 1) * sizeof(*func));
+	pd->special_funcs = xrealloc(pd->special_funcs, (pd->nr_special + 1) * sizeof(*func));
 
 	func = &pd->special_funcs[pd->nr_special++];
 
-	func->idx     = idx;
-	func->flags   = flags;
+	func->idx = idx;
+	func->flags = flags;
 }
 
-static void build_special_funcs(struct plthook_data *pd, const char *syms[],
-				unsigned nr_sym, unsigned flag)
+static void build_special_funcs(struct plthook_data *pd, const char *syms[], unsigned nr_sym,
+				unsigned flag)
 {
 	unsigned i;
 	struct dynsym_idxlist idxlist;
@@ -418,7 +421,7 @@ static int idxsort(const void *a, const void *b)
 
 static int idxfind(const void *a, const void *b)
 {
-	unsigned idx = (unsigned long) a;
+	unsigned idx = (unsigned long)a;
 	const struct plthook_special_func *func = b;
 
 	if (func->idx == idx)
@@ -429,22 +432,14 @@ static int idxfind(const void *a, const void *b)
 
 void setup_dynsym_indexes(struct plthook_data *pd)
 {
-	build_special_funcs(pd, skip_syms, ARRAY_SIZE(skip_syms),
-			    PLT_FL_SKIP);
-	build_special_funcs(pd, longjmp_syms, ARRAY_SIZE(longjmp_syms),
-			    PLT_FL_LONGJMP);
-	build_special_funcs(pd, setjmp_syms, ARRAY_SIZE(setjmp_syms),
-			    PLT_FL_SETJMP);
-	build_special_funcs(pd, vfork_syms, ARRAY_SIZE(vfork_syms),
-			    PLT_FL_VFORK);
-	build_special_funcs(pd, dlsym_syms, ARRAY_SIZE(dlsym_syms),
-			    PLT_FL_DLSYM);
-	build_special_funcs(pd, flush_syms, ARRAY_SIZE(flush_syms),
-			    PLT_FL_FLUSH);
-	build_special_funcs(pd, except_syms, ARRAY_SIZE(except_syms),
-			    PLT_FL_EXCEPT);
-	build_special_funcs(pd, resolve_syms, ARRAY_SIZE(resolve_syms),
-			    PLT_FL_RESOLVE);
+	build_special_funcs(pd, skip_syms, ARRAY_SIZE(skip_syms), PLT_FL_SKIP);
+	build_special_funcs(pd, longjmp_syms, ARRAY_SIZE(longjmp_syms), PLT_FL_LONGJMP);
+	build_special_funcs(pd, setjmp_syms, ARRAY_SIZE(setjmp_syms), PLT_FL_SETJMP);
+	build_special_funcs(pd, vfork_syms, ARRAY_SIZE(vfork_syms), PLT_FL_VFORK);
+	build_special_funcs(pd, dlsym_syms, ARRAY_SIZE(dlsym_syms), PLT_FL_DLSYM);
+	build_special_funcs(pd, flush_syms, ARRAY_SIZE(flush_syms), PLT_FL_FLUSH);
+	build_special_funcs(pd, except_syms, ARRAY_SIZE(except_syms), PLT_FL_EXCEPT);
+	build_special_funcs(pd, resolve_syms, ARRAY_SIZE(resolve_syms), PLT_FL_RESOLVE);
 
 	/* built all table, now sorting */
 	qsort(pd->special_funcs, pd->nr_special, sizeof(*pd->special_funcs), idxsort);
@@ -467,7 +462,7 @@ static int setup_mod_plthook_data(struct dl_phdr_info *info, size_t sz, void *ar
 {
 	const char *exename = info->dlpi_name;
 	unsigned long offset = info->dlpi_addr;
-	static const char * const skip_libs[] = {
+	static const char *const skip_libs[] = {
 		/* uftrace internal libraries */
 		"libmcount.so",
 		"libmcount-fast.so",
@@ -476,6 +471,8 @@ static int setup_mod_plthook_data(struct dl_phdr_info *info, size_t sz, void *ar
 		/* system base libraries */
 		"libc.so.6",
 		"libc-2.*.so",
+		"libm.so.6",
+		"libm-2.*.so",
 		"libgcc_s.so.1",
 		"libpthread.so.0",
 		"libpthread-2.*.so",
@@ -524,8 +521,7 @@ void mcount_setup_plthook(char *exename, bool nest_libcall)
 {
 	struct plthook_data *pd;
 
-	pr_dbg("setup %sPLT hooking \"%s\"\n", nest_libcall ? "nested " : "",
-		exename);
+	pr_dbg("setup %sPLT hooking \"%s\"\n", nest_libcall ? "nested " : "", exename);
 
 	if (!nest_libcall)
 		dl_iterate_phdr(setup_exe_plthook_data, exename);
@@ -546,8 +542,7 @@ struct mcount_jmpbuf_rstack {
 
 static LIST_HEAD(jmpbuf_list);
 
-static void setup_jmpbuf_rstack(struct mcount_thread_data *mtdp,
-				unsigned long addr)
+static void setup_jmpbuf_rstack(struct mcount_thread_data *mtdp, unsigned long addr)
 {
 	int i;
 	struct mcount_jmpbuf_rstack *jbstack;
@@ -566,15 +561,14 @@ static void setup_jmpbuf_rstack(struct mcount_thread_data *mtdp,
 	pr_dbg2("setup jmpbuf rstack at %lx (%d entries)\n", addr, mtdp->idx);
 
 	/* currently, only saves a single jmpbuf */
-	jbstack->count      = mtdp->idx;
+	jbstack->count = mtdp->idx;
 	jbstack->record_idx = mtdp->record_idx;
 
 	for (i = 0; i < jbstack->count; i++)
 		jbstack->rstack[i] = mtdp->rstack[i];
 }
 
-static void restore_jmpbuf_rstack(struct mcount_thread_data *mtdp,
-				  unsigned long addr)
+static void restore_jmpbuf_rstack(struct mcount_thread_data *mtdp, unsigned long addr)
 {
 	int i;
 	struct mcount_jmpbuf_rstack *jbstack;
@@ -583,11 +577,11 @@ static void restore_jmpbuf_rstack(struct mcount_thread_data *mtdp,
 		if (jbstack->addr == addr)
 			break;
 	}
-	assert(!list_no_entry(jbstack, &jmpbuf_list, list));
+	ASSERT(!list_no_entry(jbstack, &jmpbuf_list, list));
 
 	pr_dbg2("restore jmpbuf rstack at %lx (%d entries)\n", addr, jbstack->count);
 
-	mtdp->idx        = jbstack->count;
+	mtdp->idx = jbstack->count;
 	mtdp->record_idx = jbstack->record_idx;
 
 	for (i = 0; i < jbstack->count; i++) {
@@ -605,8 +599,7 @@ static int vfork_record_idx;
 static struct mcount_ret_stack vfork_rstack;
 static struct mcount_shmem vfork_shmem;
 
-static void prepare_vfork(struct mcount_thread_data *mtdp,
-			  struct mcount_ret_stack *rstack)
+static void prepare_vfork(struct mcount_thread_data *mtdp, struct mcount_ret_stack *rstack)
 {
 	/* save original parent info */
 	vfork_parent = getpid();
@@ -643,8 +636,8 @@ static void setup_vfork(struct mcount_thread_data *mtdp)
 }
 
 /* this function detects whether child is finished */
-static struct mcount_ret_stack * restore_vfork(struct mcount_thread_data *mtdp,
-					       struct mcount_ret_stack *rstack)
+static struct mcount_ret_stack *restore_vfork(struct mcount_thread_data *mtdp,
+					      struct mcount_ret_stack *rstack)
 {
 	/*
 	 * On vfork, parent sleeps until child is exec'ed or exited.
@@ -688,14 +681,13 @@ static struct mcount_ret_stack * restore_vfork(struct mcount_thread_data *mtdp,
  */
 __weak unsigned long mcount_arch_plthook_addr(struct plthook_data *pd, int idx)
 {
-	struct sym *sym;
+	struct uftrace_symbol *sym;
 
 	sym = &pd->dsymtab.sym[idx];
 	return sym->addr + ARCH_PLTHOOK_ADDR_OFFSET;
 }
 
-static void update_pltgot(struct mcount_thread_data *mtdp,
-			  struct plthook_data *pd, int dyn_idx)
+static void update_pltgot(struct mcount_thread_data *mtdp, struct plthook_data *pd, int dyn_idx)
 {
 	if (unlikely(plthook_no_pltbind))
 		return;
@@ -710,7 +702,7 @@ static void update_pltgot(struct mcount_thread_data *mtdp,
 		if (!pd->resolved_addr[dyn_idx]) {
 			int got_idx = 3 + dyn_idx;
 			plthook_addr = mcount_arch_plthook_addr(pd, dyn_idx);
-			setup_pltgot(pd, got_idx, dyn_idx, (void*)plthook_addr);
+			setup_pltgot(pd, got_idx, dyn_idx, (void *)plthook_addr);
 		}
 
 #ifndef SINGLE_THREAD
@@ -724,17 +716,12 @@ __weak unsigned long mcount_arch_child_idx(unsigned long child_idx)
 	return child_idx;
 }
 
-static unsigned long __plthook_entry(unsigned long *ret_addr,
-				     unsigned long child_idx,
-				     unsigned long module_id,
-				     struct mcount_regs *regs)
+static unsigned long __plthook_entry(unsigned long *ret_addr, unsigned long child_idx,
+				     unsigned long module_id, struct mcount_regs *regs)
 {
-	struct sym *sym;
+	struct uftrace_symbol *sym;
 	struct mcount_thread_data *mtdp = NULL;
 	struct mcount_ret_stack *rstack;
-	struct uftrace_trigger tr = {
-		.flags = 0,
-	};
 	bool skip = false;
 	bool recursion = true;
 	enum filter_result filtered;
@@ -742,6 +729,9 @@ static unsigned long __plthook_entry(unsigned long *ret_addr,
 	struct plthook_special_func *func;
 	unsigned long special_flag = 0;
 	unsigned long real_addr = 0;
+	struct uftrace_trigger tr;
+
+	mcount_memset4(&tr, 0, sizeof(tr));
 
 	// if necessary, implement it by architecture.
 	child_idx = mcount_arch_child_idx(child_idx);
@@ -769,8 +759,8 @@ static unsigned long __plthook_entry(unsigned long *ret_addr,
 
 	recursion = false;
 
-	func = bsearch((void *)child_idx, pd->special_funcs, pd->nr_special,
-		       sizeof(*func), idxfind);
+	func = bsearch((void *)child_idx, pd->special_funcs, pd->nr_special, sizeof(*func),
+		       idxfind);
 	if (func)
 		special_flag |= func->flags;
 
@@ -779,13 +769,20 @@ static unsigned long __plthook_entry(unsigned long *ret_addr,
 
 	if (likely(child_idx < pd->dsymtab.nr_sym)) {
 		sym = &pd->dsymtab.sym[child_idx];
-		pr_dbg3("[idx: %4d] enter %"PRIx64": %s@plt (mod: %lx)\n",
-			(int)child_idx, sym->addr, sym->name, module_id);
+
+		if (dbg_domain[DBG_PLTHOOK] >= 3) {
+			char *symname = demangle(sym->name);
+
+			pr_dbg3("[idx: %4d] enter %" PRIx64 ": %s@plt (mod: %lx)\n", (int)child_idx,
+				sym->addr, symname, module_id);
+			free(symname);
+		}
 	}
 	else {
-		sym = NULL;
-		pr_err_ns("invalid function idx found! (idx: %lu/%zu, module: %s)\n",
-			  child_idx, pd->dsymtab.nr_sym, pd->mod_name);
+		pr_dbg("invalid function idx found! (idx: %lu/%zu, module: %s)\n", child_idx,
+		       pd->dsymtab.nr_sym, pd->mod_name);
+		mcount_unguard_recursion(mtdp);
+		return 0;
 	}
 
 	filtered = mcount_entry_filter_check(mtdp, sym->addr, &tr);
@@ -800,28 +797,35 @@ static unsigned long __plthook_entry(unsigned long *ret_addr,
 		/* but if we don't have rstack, just bail out */
 		if (filtered == FILTER_RSTACK)
 			goto out;
+		if (mcount_estimate_return)
+			goto out;
 	}
 
+	if (mcount_estimate_return)
+		mcount_rstack_inject_return(mtdp, ret_addr, sym->addr);
+
 	rstack = &mtdp->rstack[mtdp->idx++];
 
-	rstack->depth      = mtdp->record_idx;
-	rstack->pd         = pd;
-	rstack->dyn_idx    = child_idx;
+	rstack->depth = mtdp->record_idx;
+	rstack->pd = pd;
+	rstack->dyn_idx = child_idx;
 	rstack->parent_loc = ret_addr;
-	rstack->parent_ip  = *ret_addr;
-	rstack->child_ip   = sym->addr;
+	rstack->parent_ip = *ret_addr;
+	rstack->child_ip = sym->addr;
 	rstack->start_time = skip ? 0 : mcount_gettime();
-	rstack->end_time   = 0;
-	rstack->flags      = skip ? MCOUNT_FL_NORECORD : 0;
-	rstack->nr_events  = 0;
-	rstack->event_idx  = ARGBUF_SIZE;
+	rstack->end_time = 0;
+	rstack->flags = skip ? MCOUNT_FL_NORECORD : 0;
+	rstack->nr_events = 0;
+	rstack->event_idx = ARGBUF_SIZE;
 
-	/* hijack the return address of child */
-	*ret_addr = (unsigned long)plthook_return;
+	if (!mcount_estimate_return) {
+		/* hijack the return address of child */
+		*ret_addr = (unsigned long)plthook_return;
 
-	/* restore return address of parent */
-	if (mcount_auto_recover)
-		mcount_auto_restore(mtdp);
+		/* restore return address of parent */
+		if (mcount_auto_recover)
+			mcount_auto_restore(mtdp);
+	}
 
 	mcount_entry_filter_record(mtdp, rstack, &tr, regs);
 
@@ -882,8 +886,7 @@ static unsigned long __plthook_entry(unsigned long *ret_addr,
 	}
 
 out:
-	if (likely(pd && child_idx < pd->dsymtab.nr_sym) &&
-	    pd->resolved_addr[child_idx] != 0)
+	if (likely(pd && child_idx < pd->dsymtab.nr_sym) && pd->resolved_addr[child_idx] != 0)
 		real_addr = pd->resolved_addr[child_idx];
 
 	if (!recursion)
@@ -903,7 +906,6 @@ unsigned long plthook_entry(unsigned long *ret_addr, unsigned long child_idx,
 	return ret;
 }
 
-
 void mtd_dtor(void *arg);
 
 static unsigned long __plthook_exit(long *retval)
@@ -915,7 +917,7 @@ static unsigned long __plthook_exit(long *retval)
 	unsigned long ret_addr = 0;
 
 	mtdp = get_thread_data();
-	assert(!check_thread_data(mtdp));
+	ASSERT(!check_thread_data(mtdp));
 
 	/*
 	 * it's only called when mcount_entry() was succeeded and
@@ -928,7 +930,7 @@ again:
 	if (likely(mtdp->idx > 0))
 		rstack = &mtdp->rstack[mtdp->idx - 1];
 	else
-		rstack = restore_vfork(mtdp, NULL);  /* FIXME! */
+		rstack = restore_vfork(mtdp, NULL); /* FIXME! */
 
 	if (unlikely(rstack->flags & (MCOUNT_FL_LONGJMP | MCOUNT_FL_VFORK))) {
 		if (rstack->flags & MCOUNT_FL_LONGJMP) {
@@ -946,8 +948,7 @@ again:
 		rstack = restore_vfork(mtdp, rstack);
 
 	dyn_idx = rstack->dyn_idx;
-	if (unlikely(dyn_idx == MCOUNT_INVALID_DYNIDX ||
-		     dyn_idx >= rstack->pd->dsymtab.nr_sym))
+	if (unlikely(dyn_idx == MCOUNT_INVALID_DYNIDX || dyn_idx >= rstack->pd->dsymtab.nr_sym))
 		pr_err_ns("<%d> invalid dynsym idx: %d\n", mtdp->idx, dyn_idx);
 
 	if (!ARCH_CAN_RESTORE_PLTHOOK && unlikely(mtdp->dead)) {
@@ -978,12 +979,11 @@ again:
 	 */
 	update_pltgot(mtdp, rstack->pd, dyn_idx);
 
-	ret_loc  = rstack->parent_loc;
+	ret_loc = rstack->parent_loc;
 	ret_addr = rstack->parent_ip;
 
-	pr_dbg3("[idx: %4d] exit  %lx: %s     (resolved addr: %lx)\n",
-		dyn_idx, ret_addr, rstack->pd->dsymtab.sym[dyn_idx].name,
-		rstack->pd->resolved_addr[dyn_idx]);
+	pr_dbg3("[idx: %4d] exit  %lx: %s     (resolved addr: %lx)\n", dyn_idx, ret_addr,
+		rstack->pd->dsymtab.sym[dyn_idx].name, rstack->pd->resolved_addr[dyn_idx]);
 
 	/* re-hijack return address of parent */
 	if (mcount_auto_recover)
