@@ -1,29 +1,29 @@
-#include <string.h>
-#include <pthread.h>
-#include <unistd.h>
 #include <dlfcn.h>
 #include <link.h>
-#include <sys/uio.h>
+#include <pthread.h>
 #include <spawn.h>
+#include <string.h>
+#include <sys/uio.h>
+#include <unistd.h>
 
 /* This should be defined before #include "utils.h" */
-#define PR_FMT     "mcount"
-#define PR_DOMAIN  DBG_MCOUNT
+#define PR_FMT "wrap"
+#define PR_DOMAIN DBG_WRAP
 
-#include "libmcount/mcount.h"
+#include "libmcount/dynamic.h"
 #include "libmcount/internal.h"
-#include "utils/utils.h"
+#include "libmcount/mcount.h"
 #include "utils/compiler.h"
+#include "utils/utils.h"
 
-extern struct symtabs symtabs;
+extern struct uftrace_sym_info mcount_sym_info;
 
 struct dlopen_base_data {
 	struct mcount_thread_data *mtdp;
 	uint64_t timestamp;
 };
 
-
-static const char *simple_basename(const char *pathname)
+const char *uftrace_basename(const char *pathname)
 {
 	const char *p = strrchr(pathname, '/');
 
@@ -31,8 +31,7 @@ static const char *simple_basename(const char *pathname)
 }
 
 static void send_dlopen_msg(struct mcount_thread_data *mtdp, const char *sess_id,
-			    uint64_t timestamp,  uint64_t base_addr,
-			    const char *libname)
+			    uint64_t timestamp, uint64_t base_addr, const char *libname)
 {
 	struct uftrace_msg_dlopen dlop = {
 		.task = {
@@ -49,9 +48,18 @@ static void send_dlopen_msg(struct mcount_thread_data *mtdp, const char *sess_id
 		.len = sizeof(dlop) + dlop.namelen,
 	};
 	struct iovec iov[3] = {
-		{ .iov_base = &msg, .iov_len = sizeof(msg), },
-		{ .iov_base = &dlop, .iov_len = sizeof(dlop), },
-		{ .iov_base = (void *)libname, .iov_len = dlop.namelen, },
+		{
+			.iov_base = &msg,
+			.iov_len = sizeof(msg),
+		},
+		{
+			.iov_base = &dlop,
+			.iov_len = sizeof(dlop),
+		},
+		{
+			.iov_base = (void *)libname,
+			.iov_len = dlop.namelen,
+		},
 	};
 	int len = sizeof(msg) + msg.len;
 
@@ -66,8 +74,7 @@ static void send_dlopen_msg(struct mcount_thread_data *mtdp, const char *sess_id
 	}
 }
 
-static int dlopen_base_callback(struct dl_phdr_info *info,
-				size_t size, void *arg)
+static int dlopen_base_callback(struct dl_phdr_info *info, size_t size, void *arg)
 {
 	struct dlopen_base_data *data = arg;
 	char buf[PATH_MAX];
@@ -82,17 +89,18 @@ static int dlopen_base_callback(struct dl_phdr_info *info,
 	if (p == NULL)
 		p = buf;
 
-	if (find_map_by_name(&symtabs, simple_basename(p)))
+	if (find_map_by_name(&mcount_sym_info, uftrace_basename(p)))
 		return 0;
 
 	/* report a library not found in the session maps */
-	send_dlopen_msg(data->mtdp, mcount_session_name(), data->timestamp,
-			info->dlpi_addr, info->dlpi_name);
+	send_dlopen_msg(data->mtdp, mcount_session_name(), data->timestamp, info->dlpi_addr,
+			info->dlpi_name);
+
+	mcount_dynamic_dlopen(&mcount_sym_info, info, p);
 	return 0;
 }
 
-void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp,
-				   unsigned long frame_addr)
+void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp, unsigned long frame_addr)
 {
 	int idx;
 	struct mcount_ret_stack *rstack;
@@ -101,7 +109,7 @@ void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp,
 	for (idx = mtdp->idx - 1; idx >= 0; idx--) {
 		rstack = &mtdp->rstack[idx];
 
-		pr_dbg2("[%d] parent at %p\n", idx, rstack->parent_loc);
+		pr_dbg3("%s: [%d] parent at %p\n", __func__, idx, rstack->parent_loc);
 		if (rstack->parent_loc == &mtdp->cygprof_dummy)
 			break;
 
@@ -124,7 +132,7 @@ void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp,
 
 				idx--;
 				rstack = tail_call;
-				pr_dbg2("exception in tail call at [%d]\n", idx + 1);
+				pr_dbg3("%s: exception in tail call at [%d]\n", __func__, idx + 1);
 			}
 			idx = orig_idx;
 
@@ -142,29 +150,48 @@ void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp,
 
 	/* we're in ENTER state, so add 1 to the index */
 	mtdp->idx = idx + 1;
-	pr_dbg2("exception returned to [%d]\n", mtdp->idx);
+	pr_dbg3("%s: exception returned to [%d]\n", __func__, mtdp->idx);
 
 	mcount_rstack_reset(mtdp);
 }
 
-static char ** collect_uftrace_envp(void)
+static char **collect_uftrace_envp(void)
 {
 	size_t n = 0;
 	size_t i, k;
 	char **envp;
 
-#define ENV(_name)  "UFTRACE_" #_name
+#define ENV(_name) "UFTRACE_" #_name
 
 	const char *const uftrace_env[] = {
-		ENV(FILTER), ENV(TRIGGER), ENV(ARGUMENT), ENV(RETVAL),
-		ENV(AUTO_ARGS), ENV(DEPTH), ENV(DISABLED), ENV(PIPE),
-		ENV(LOGFD), ENV(DEBUG), ENV(BUFFER), ENV(MAX_STACK),
-		ENV(COLOR), ENV(THRESHOLD), ENV(DEMANGLE), ENV(PLTHOOK),
-		ENV(PATCH), ENV(EVENT), ENV(SCRIPT), ENV(NEST_LIBCALL),
-		ENV(DEBUG_DOMAIN), ENV(LIST_EVENT), ENV(DIR),
-		ENV(KERNEL_PID_UPDATE), ENV(PATTERN),
+		ENV(FILTER),
+		ENV(TRIGGER),
+		ENV(ARGUMENT),
+		ENV(RETVAL),
+		ENV(AUTO_ARGS),
+		ENV(DEPTH),
+		ENV(DISABLED),
+		ENV(PIPE),
+		ENV(LOGFD),
+		ENV(DEBUG),
+		ENV(BUFFER),
+		ENV(MAX_STACK),
+		ENV(COLOR),
+		ENV(THRESHOLD),
+		ENV(DEMANGLE),
+		ENV(PLTHOOK),
+		ENV(PATCH),
+		ENV(EVENT),
+		ENV(SCRIPT),
+		ENV(NEST_LIBCALL),
+		ENV(DEBUG_DOMAIN),
+		ENV(LIST_EVENT),
+		ENV(DIR),
+		ENV(KERNEL_PID_UPDATE),
+		ENV(PATTERN),
 		/* not uftrace-specific, but necessary to run */
-		"LD_PRELOAD", "LD_LIBRARY_PATH",
+		"LD_PRELOAD",
+		"LD_LIBRARY_PATH",
 	};
 
 #undef ENV
@@ -191,16 +218,23 @@ static char ** collect_uftrace_envp(void)
 	return envp;
 }
 
-static char ** merge_envp(char *const *env1, char **env2)
+static int count_envp(char *const *env)
+{
+	int i, n = 0;
+
+	for (i = 0; env && env[i]; i++)
+		n++;
+
+	return n;
+}
+
+static char **merge_envp(char *const *env1, char **env2)
 {
 	int i, n = 0;
 	char **envp;
 
-	for (i = 0; env1 && env1[i]; i++)
-		n++;
-	for (i = 0; env2 && env2[i]; i++)
-		n++;
-
+	n += count_envp(env1);
+	n += count_envp(env2);
 
 	envp = xcalloc(sizeof(*envp), n + 1);
 
@@ -219,41 +253,39 @@ static char ** merge_envp(char *const *env1, char **env2)
 static int (*real_backtrace)(void **buffer, int sz);
 static void (*real_cxa_throw)(void *exc, void *type, void *dest);
 static void (*real_cxa_rethrow)(void);
-static void * (*real_cxa_begin_catch)(void *exc);
+static void *(*real_cxa_begin_catch)(void *exc);
 static void (*real_cxa_end_catch)(void);
-static void * (*real_dlopen)(const char *filename, int flags);
+static void *(*real_dlopen)(const char *filename, int flags);
 static __noreturn void (*real_pthread_exit)(void *retval);
 static void (*real_unwind_resume)(void *exc);
 static int (*real_posix_spawn)(pid_t *pid, const char *path,
 			       const posix_spawn_file_actions_t *actions,
-			       const posix_spawnattr_t *attr,
-			       char *const argv[], char *const envp[]);
+			       const posix_spawnattr_t *attr, char *const argv[],
+			       char *const envp[]);
 static int (*real_posix_spawnp)(pid_t *pid, const char *file,
 				const posix_spawn_file_actions_t *actions,
-				const posix_spawnattr_t *attr,
-				char *const argv[], char *const envp[]);
+				const posix_spawnattr_t *attr, char *const argv[],
+				char *const envp[]);
 /* TODO: support execle() */
-static int (*real_execve)(const char *path, char *const argv[],
-			  char *const envp[]);
-static int (*real_execvpe)(const char *file, char *const argv[],
-			   char *const envp[]);
+static int (*real_execve)(const char *path, char *const argv[], char *const envp[]);
+static int (*real_execvpe)(const char *file, char *const argv[], char *const envp[]);
 static int (*real_fexecve)(int fd, char *const argv[], char *const envp[]);
 
 void mcount_hook_functions(void)
 {
-	real_backtrace		= dlsym(RTLD_NEXT, "backtrace");
-	real_cxa_throw		= dlsym(RTLD_NEXT, "__cxa_throw");
-	real_cxa_rethrow	= dlsym(RTLD_NEXT, "__cxa_rethrow");
-	real_cxa_begin_catch	= dlsym(RTLD_NEXT, "__cxa_begin_catch");
-	real_cxa_end_catch	= dlsym(RTLD_NEXT, "__cxa_end_catch");
-	real_dlopen		= dlsym(RTLD_NEXT, "dlopen");
-	real_pthread_exit	= dlsym(RTLD_NEXT, "pthread_exit");
-	real_unwind_resume	= dlsym(RTLD_NEXT, "_Unwind_Resume");
-	real_posix_spawn	= dlsym(RTLD_NEXT, "posix_spawn");
-	real_posix_spawnp	= dlsym(RTLD_NEXT, "posix_spawnp");
-	real_execve		= dlsym(RTLD_NEXT, "execve");
-	real_execvpe		= dlsym(RTLD_NEXT, "execvpe");
-	real_fexecve		= dlsym(RTLD_NEXT, "fexecve");
+	real_backtrace = dlsym(RTLD_NEXT, "backtrace");
+	real_cxa_throw = dlsym(RTLD_NEXT, "__cxa_throw");
+	real_cxa_rethrow = dlsym(RTLD_NEXT, "__cxa_rethrow");
+	real_cxa_begin_catch = dlsym(RTLD_NEXT, "__cxa_begin_catch");
+	real_cxa_end_catch = dlsym(RTLD_NEXT, "__cxa_end_catch");
+	real_dlopen = dlsym(RTLD_NEXT, "dlopen");
+	real_pthread_exit = dlsym(RTLD_NEXT, "pthread_exit");
+	real_unwind_resume = dlsym(RTLD_NEXT, "_Unwind_Resume");
+	real_posix_spawn = dlsym(RTLD_NEXT, "posix_spawn");
+	real_posix_spawnp = dlsym(RTLD_NEXT, "posix_spawnp");
+	real_execve = dlsym(RTLD_NEXT, "execve");
+	real_execvpe = dlsym(RTLD_NEXT, "execvpe");
+	real_fexecve = dlsym(RTLD_NEXT, "fexecve");
 }
 
 __visible_default int backtrace(void **buffer, int sz)
@@ -265,8 +297,10 @@ __visible_default int backtrace(void **buffer, int sz)
 		mcount_hook_functions();
 
 	mtdp = get_thread_data();
-	if (!check_thread_data(mtdp))
+	if (!check_thread_data(mtdp)) {
 		mcount_rstack_restore(mtdp);
+		pr_dbg("%s is called from [%d]\n", __func__, mtdp->idx);
+	}
 
 	ret = real_backtrace(buffer, sz);
 
@@ -285,7 +319,7 @@ __visible_default void __cxa_throw(void *exception, void *type, void *dest)
 
 	mtdp = get_thread_data();
 	if (!check_thread_data(mtdp)) {
-		pr_dbg("exception thrown from [%d]\n", mtdp->idx);
+		pr_dbg2("%s: exception thrown from [%d]\n", __func__, mtdp->idx);
 
 		mtdp->in_exception = true;
 
@@ -309,7 +343,7 @@ __visible_default void __cxa_rethrow(void)
 
 	mtdp = get_thread_data();
 	if (!check_thread_data(mtdp)) {
-		pr_dbg("exception rethrown from [%d]\n", mtdp->idx);
+		pr_dbg2("%s: exception rethrown from [%d]\n", __func__, mtdp->idx);
 
 		mtdp->in_exception = true;
 
@@ -333,7 +367,7 @@ __visible_default void _Unwind_Resume(void *exception)
 
 	mtdp = get_thread_data();
 	if (!check_thread_data(mtdp)) {
-		pr_dbg2("exception resumed on [%d]\n", mtdp->idx);
+		pr_dbg2("%s: exception resumed on [%d]\n", __func__, mtdp->idx);
 
 		mtdp->in_exception = true;
 
@@ -348,7 +382,7 @@ __visible_default void _Unwind_Resume(void *exception)
 	real_unwind_resume(exception);
 }
 
-__visible_default void * __cxa_begin_catch(void *exception)
+__visible_default void *__cxa_begin_catch(void *exception)
 {
 	struct mcount_thread_data *mtdp;
 	void *obj;
@@ -359,12 +393,12 @@ __visible_default void * __cxa_begin_catch(void *exception)
 	obj = real_cxa_begin_catch(exception);
 
 	mtdp = get_thread_data();
-	if (!check_thread_data(mtdp) && unlikely(mtdp->in_exception)) {
+	if (!mcount_estimate_return && !check_thread_data(mtdp) && unlikely(mtdp->in_exception)) {
 		unsigned long *frame_ptr;
 		unsigned long frame_addr;
 
 		frame_ptr = __builtin_frame_address(0);
-		frame_addr = *frame_ptr;  /* XXX: probably dangerous */
+		frame_addr = *frame_ptr; /* XXX: probably dangerous */
 
 		/* basic sanity check */
 		if (frame_addr < (unsigned long)frame_ptr)
@@ -372,6 +406,7 @@ __visible_default void * __cxa_begin_catch(void *exception)
 
 		mcount_rstack_reset_exception(mtdp, frame_addr);
 		mtdp->in_exception = false;
+		pr_dbg2("%s: exception caught begin on [%d]\n", __func__, mtdp->idx);
 	}
 
 	return obj;
@@ -382,10 +417,11 @@ __visible_default void __cxa_end_catch(void)
 	if (unlikely(real_cxa_end_catch == NULL))
 		mcount_hook_functions();
 
+	pr_dbg2("%s: exception caught end\n", __func__);
 	real_cxa_end_catch();
 }
 
-__visible_default void * dlopen(const char *filename, int flags)
+__visible_default void *dlopen(const char *filename, int flags)
 {
 	struct mcount_thread_data *mtdp;
 	struct dlopen_base_data data = {
@@ -402,6 +438,7 @@ __visible_default void * dlopen(const char *filename, int flags)
 	if (unlikely(real_dlopen == NULL))
 		mcount_hook_functions();
 
+	pr_dbg("%s is called for '%s'\n", __func__, filename);
 	ret = real_dlopen(filename, flags);
 
 	if (filename == NULL)
@@ -434,7 +471,7 @@ __visible_default __noreturn void pthread_exit(void *retval)
 		mcount_hook_functions();
 
 	mtdp = get_thread_data();
-	if (!check_thread_data(mtdp)) {
+	if (!mcount_estimate_return && !check_thread_data(mtdp)) {
 		rstack = &mtdp->rstack[mtdp->idx - 1];
 		/* record the final call */
 		mcount_exit_filter_record(mtdp, rstack, NULL);
@@ -448,13 +485,15 @@ __visible_default __noreturn void pthread_exit(void *retval)
 		mcount_rstack_restore(mtdp);
 	}
 
+	if (!check_thread_data(mtdp))
+		pr_dbg("%s: pthread exited on [%d]\n", __func__, mtdp->idx);
 	real_pthread_exit(retval);
 }
 
 __visible_default int posix_spawn(pid_t *pid, const char *path,
 				  const posix_spawn_file_actions_t *actions,
-				  const posix_spawnattr_t *attr,
-				  char *const argv[], char *const envp[])
+				  const posix_spawnattr_t *attr, char *const argv[],
+				  char *const envp[])
 {
 	char **uftrace_envp;
 	char **new_envp;
@@ -465,13 +504,14 @@ __visible_default int posix_spawn(pid_t *pid, const char *path,
 	uftrace_envp = collect_uftrace_envp();
 	new_envp = merge_envp(envp, uftrace_envp);
 
+	pr_dbg("%s is called for '%s'\n", __func__, path);
 	return real_posix_spawn(pid, path, actions, attr, argv, new_envp);
 }
 
 __visible_default int posix_spawnp(pid_t *pid, const char *file,
 				   const posix_spawn_file_actions_t *actions,
-				   const posix_spawnattr_t *attr,
-				   char *const argv[], char *const envp[])
+				   const posix_spawnattr_t *attr, char *const argv[],
+				   char *const envp[])
 {
 	char **uftrace_envp;
 	char **new_envp;
@@ -482,11 +522,11 @@ __visible_default int posix_spawnp(pid_t *pid, const char *file,
 	uftrace_envp = collect_uftrace_envp();
 	new_envp = merge_envp(envp, uftrace_envp);
 
+	pr_dbg("%s is called for '%s'\n", __func__, file);
 	return real_posix_spawnp(pid, file, actions, attr, argv, new_envp);
 }
 
-__visible_default int execve(const char *path, char *const argv[],
-			     char *const envp[])
+__visible_default int execve(const char *path, char *const argv[], char *const envp[])
 {
 	char **uftrace_envp;
 	char **new_envp;
@@ -497,11 +537,11 @@ __visible_default int execve(const char *path, char *const argv[],
 	uftrace_envp = collect_uftrace_envp();
 	new_envp = merge_envp(envp, uftrace_envp);
 
+	pr_dbg("%s is called for '%s'\n", __func__, path);
 	return real_execve(path, argv, new_envp);
 }
 
-__visible_default int execvpe(const char *file, char *const argv[],
-			      char *const envp[])
+__visible_default int execvpe(const char *file, char *const argv[], char *const envp[])
 {
 	char **uftrace_envp;
 	char **new_envp;
@@ -512,6 +552,7 @@ __visible_default int execvpe(const char *file, char *const argv[],
 	uftrace_envp = collect_uftrace_envp();
 	new_envp = merge_envp(envp, uftrace_envp);
 
+	pr_dbg("%s is called for '%s'\n", __func__, file);
 	return real_execvpe(file, argv, new_envp);
 }
 
@@ -526,6 +567,7 @@ __visible_default int fexecve(int fd, char *const argv[], char *const envp[])
 	uftrace_envp = collect_uftrace_envp();
 	new_envp = merge_envp(envp, uftrace_envp);
 
+	pr_dbg("%s is called for fd %d\n", __func__, fd);
 	return real_fexecve(fd, argv, new_envp);
 }
 
@@ -535,9 +577,13 @@ TEST_CASE(mcount_wrap_dlopen)
 {
 	void *handle;
 
-	TEST_EQ(real_dlopen, NULL);
+	/* In some environment, dlopen() is called already */
+	if (unlikely(real_dlopen != NULL))
+		real_dlopen = NULL;
 
-	handle= dlopen(NULL, RTLD_LAZY);
+	pr_dbg("calling %s (%s) should init all the wrappers\n", "dlopen",
+	       "or other wrapped function");
+	handle = dlopen(NULL, RTLD_LAZY);
 
 	TEST_NE(handle, NULL);
 	TEST_NE(real_dlopen, NULL);
@@ -545,4 +591,30 @@ TEST_CASE(mcount_wrap_dlopen)
 	return TEST_OK;
 }
 
+TEST_CASE(mcount_env_check)
+{
+	char **uftrace_envp;
+	char **new_envp;
+	int old1_cnt, old2_cnt, new_cnt;
+	int i;
+
+	pr_dbg("collecting environ related to uftrace\n");
+	uftrace_envp = collect_uftrace_envp();
+	old1_cnt = count_envp(uftrace_envp);
+	old2_cnt = count_envp(environ);
+
+	pr_dbg("merging uftrace envp to the existing one\n");
+	new_envp = merge_envp(environ, uftrace_envp);
+	new_cnt = count_envp(new_envp);
+
+	TEST_EQ(old1_cnt + old2_cnt, new_cnt);
+
+	for (i = 0; i < old1_cnt; i++)
+		free(uftrace_envp[i]);
+	free(uftrace_envp);
+	free(new_envp);
+
+	return TEST_OK;
+}
+
 #endif /* UNIT_TEST */
