@@ -10,6 +10,8 @@
 // contain more function prototypes as of now.
 //
 
+// clang-format off
+
 #include <stdlib.h>
 int atoi(const char *str);
 long atol(const char *str);
@@ -345,6 +347,12 @@ int pthread_mutex_trylock(pthread_mutex_t *mutex);
 int pthread_mutex_unlock(pthread_mutex_t *mutex);
 int pthread_mutex_destroy(pthread_mutex_t *mutex);
 int pthread_mutex_init(pthread_mutex_t *restrict mutex, const pthread_mutexattr_t *restrict attr);
+
+int pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex);
+int pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex, const struct timespec *abstime);
+
+int pthread_cond_signal(pthread_cond_t *cond);
+int pthread_cond_broadcast(pthread_cond_t *cond);
 ////////////////////////////////////////////////////////////////////////////////
 
 
@@ -526,6 +534,9 @@ int fchown(int fd, uid_t uid, uid_t gid);
 int dup(int oldfd);
 int dup2(int oldfd, int newfd);
 
+unsigned sleep(unsigned seconds);
+int usleep(unsigned usec);
+
 #include <time.h>
 enum uft_clockid_t {
 	CLOCK_REALTIME = 0,
@@ -544,3 +555,5 @@ int clock_getres(enum uft_clockid_t clk_id, struct timespec *res);
 int clock_gettime(enum uft_clockid_t clk_id, struct timespec *tp);
 int clock_settime(enum uft_clockid_t clk_id, const struct timespec *tp);
 ////////////////////////////////////////////////////////////////////////////////
+
+// clang-format on
