@@ -1,13 +1,72 @@
 #!/usr/bin/env python
 
-import random
-import os, sys
-import tempfile
-import glob, re
-import subprocess as sp
+import glob
 import multiprocessing
+import os
+import random
+import re
+import subprocess as sp
+import sys
+import tempfile
 import time
 
+class Elf:
+    EI_NIDENT = 16
+
+    @staticmethod
+    def is_32bit(filename):
+        # e_ident[] indexes
+        EI_CLASS      = 4
+
+        # EI_CLASS
+        ELFCLASSNONE  = 0
+        ELFCLASS32    = 1
+        ELFCLASS64    = 2
+        ELFCLASSNUM   = 3
+
+        try:
+            with open(filename, 'rb') as f:
+                elf_ident = list(f.read(Elf.EI_NIDENT))
+                ei_class = ord(elf_ident[EI_CLASS])
+
+            if ei_class == ELFCLASS32:
+                return True
+        except:
+            pass
+
+        return False
+
+    @staticmethod
+    def get_elf_machine(filename):
+        # e_machine (architecture)
+        EM_386        = 3       # Intel 80386
+        EM_ARM        = 40      # ARM
+        EM_X86_64     = 62      # AMD x86-64 architecture
+        EM_AARCH64    = 183     # ARM AARCH64
+
+        machine = {
+            EM_386: 'i386',
+            EM_ARM: 'arm',
+            EM_X86_64: 'x86_64',
+            EM_AARCH64: 'aarch64',
+        }
+
+        try:
+            with open(filename, 'rb') as f:
+                # consume elf_ident and e_type
+                f.read(Elf.EI_NIDENT + 2)
+
+                # read e_machine
+                e_machine = f.read(2)[0]
+                if type(e_machine) is str:
+                    e_machine = ord(e_machine)
+
+            return machine[e_machine]
+        except:
+            pass
+
+        return None
+
 class TestBase:
     supported_lang = {
         'C':   { 'cc': 'gcc', 'flags': 'CFLAGS',   'ext': '.c' },
@@ -24,15 +83,19 @@ class TestBase:
     TEST_SKIP = -7
     TEST_SUCCESS_FIXED = -8
 
-    basedir = os.path.dirname(os.getcwd())
+    origdir = os.getcwd()
+    basedir = os.path.dirname(origdir)
     objdir = 'objdir' in os.environ and os.environ['objdir'] or basedir
-    uftrace_cmd = objdir + '/uftrace --no-pager --no-event -L' + objdir
+    uftrace_cmd = objdir + '/uftrace'
+    default_opt = '--no-pager --no-event --libmcount-path=' + objdir
 
     default_cflags = ['-fno-inline', '-fno-builtin', '-fno-ipa-cp',
                       '-fno-omit-frame-pointer', '-D_FORTIFY_SOURCE=0']
+    feature = set()
 
     def __init__(self, name, result, lang='C', cflags='', ldflags='', sort='task', serial=False):
         _tmp = tempfile.mkdtemp(prefix='test_%s_' % name)
+        self.keep = False
         os.chdir(_tmp)
         self.test_dir = _tmp
         self.name = name
@@ -42,6 +105,23 @@ class TestBase:
         self.lang = lang
         self.sort_method = sort
         self.serial = serial
+        self.subcmd = 'live'
+        self.option = ''
+        self.exearg = 't-' + name
+        self.test_feature()
+        if Elf.get_elf_machine(self.uftrace_cmd) == 'i386':
+            self.default_cflags.append('-m32')
+
+    def set_compiler(self, compiler):
+        if compiler == 'gcc':
+            self.supported_lang['C']['cc'] = 'gcc'
+            self.supported_lang['C++']['cc'] = 'g++'
+        elif compiler == 'clang':
+            self.supported_lang['C']['cc'] = 'clang'
+            self.supported_lang['C++']['cc'] = 'clang++'
+        else:
+            # ignore invalid compiler argument
+            pass
 
     def set_debug(self, dbg):
         self.debug = dbg
@@ -50,9 +130,23 @@ class TestBase:
         if self.debug:
             print(msg)
 
+    def set_keep(self, keep):
+        self.keep = keep
+
     def gen_port(self):
         self.port = random.randint(40000, 50000)
 
+    def test_feature(self):
+        try:
+            p = sp.Popen(self.uftrace_cmd + ' --version', shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
+            uftrace_version = p.communicate()[0].decode(errors='ignore')
+            s = uftrace_version.split()
+            for i in range(3, len(s) - 1):
+                self.feature.add(s[i])
+            return True
+        except:
+            return False
+
     def convert_abs_path(self, build_cmd):
         cmd = build_cmd.split()
         src_idx = [i for i, _cmd in enumerate(cmd) if _cmd.startswith('s-')][0]
@@ -77,7 +171,7 @@ class TestBase:
 
     def build(self, name, cflags='', ldflags=''):
         if self.lang not in TestBase.supported_lang:
-            pr_debug("%s: unsupported language: %s" % (name, self.lang))
+            self.pr_debug("%s: unsupported language: %s" % (name, self.lang))
             return TestBase.TEST_UNSUPP_LANG
 
         lang = TestBase.supported_lang[self.lang]
@@ -167,10 +261,19 @@ class TestBase:
         self.pr_debug("build command for executable: %s" % build_cmd)
         return self.build_it(build_cmd)
 
+    def prepare(self):
+        """ This function returns command line need to be run before"""
+        return ''
+
+    def setup(self):
+        """ This function sets up options to be passed to runcmd"""
+        pass
+
     def runcmd(self):
         """ This function returns (shell) command that runs the test.
             A test case can extend this to setup a complex configuration.  """
-        return '%s %s' % (TestBase.uftrace_cmd, 't-' + self.name)
+        return '%s %s %s %s %s' % (TestBase.uftrace_cmd, self.subcmd, \
+                                   TestBase.default_opt, self.option, self.exearg)
 
     def task_sort(self, output, ignore_children=False):
         """ This function post-processes output of the test to be compared .
@@ -247,7 +350,7 @@ class TestBase:
                     continue
             except:
                 pass
-            result.append('%s %s' % (line[4], line[5]))
+            result.append('%s %s' % (line[-2], line[-1]))
 
         return '\n'.join(result)
 
@@ -282,17 +385,26 @@ class TestBase:
     def dump_sort(self, output, ignored):
         """ This function post-processes output of the test to be compared .
             It ignores blank and comment (#) lines and remaining functions.  """
+        mode = 1
+        result = []
 
         # A (raw) dump result consists of following data
         # <timestamp> <tid>: [<type>] <func>(<addr>) depth: <N>
-        mode = 1
         patt = re.compile(r'[^[]*(?P<type>\[(entry|exit )\]) (?P<func>[_a-z0-9]*)\([0-9a-f]+\) (?P<depth>.*)')
-        result = []
+
+        # A (raw) dump argument patter
+        arg_patt = re.compile(r'^  args')
+
         for ln in output.split('\n'):
             if ln.startswith('uftrace'):
                 #result.append(ln)
                 pass
             else:
+                m = arg_patt.match(ln)
+                if m is not None:
+                    result.append(ln)
+                    continue
+
                 m = patt.match(ln)
                 if m is None:
                     continue
@@ -330,6 +442,30 @@ class TestBase:
                 result.append("%s %s" % (ln['ph'], ln['name']))
         return '\n'.join(result)
 
+    def mermaid_sort(self, output, ignore_children=False):
+        """ This function post-processes output of the test to be compared .
+            It ignores blank and comment (#) lines and remaining functions.  """
+        result = []
+        start_mermaid = False
+
+        for ln in output.split('\n'):
+            if ln.find('<div class=\"mermaid\"') >= 0:
+                start_mermaid = True
+                continue
+            if start_mermaid == False:
+                continue
+            if ln.find('</div>') >= 0:
+                break
+
+            m = re.match( r'\s+(?P<start_depth>\d+)_(?P<start_id>\d+)\[\"(?P<start_name>\S+)\"\]\s+'
+                + '-->\|(?P<call_num>\d+)\|\s+(?P<end_depth>\d+)_(?P<end_id>\d+)\[\"(?P<end_name>\S+)\"\];', ln)
+            if m:
+                result.append("%s_%s_%s %s> %s_%s_%s" % (m.group('start_depth'), m.group('start_id'), m.group('start_name'),
+                m.group('call_num'), m.group('end_depth'), m.group('end_id'), m.group('end_name')))
+            else:
+                continue
+        return '\n'.join(result)
+
     def sort(self, output, ignore_children=False):
         if not hasattr(TestBase, self.sort_method + '_sort'):
             print('cannot find the sort function: %s' % self.sort_method)
@@ -340,11 +476,7 @@ class TestBase:
         else:
             return ''  # this leads to a failure with 'NG'
 
-    def pre(self):
-        """This function is called before running a testcase"""
-        return TestBase.TEST_SUCCESS
-
-    def post(self, result):
+    def postrun(self, result):
         """This function is called after running a testcase"""
         return result
 
@@ -359,6 +491,8 @@ class TestBase:
         return os.path.exists('%s/check-deps/' % self.basedir + item)
 
     def check_perf_paranoid(self):
+        if not 'perf' in TestBase.feature:
+            return False
         try:
             f = open('/proc/sys/kernel/perf_event_paranoid')
             v = int(f.readline())
@@ -372,66 +506,42 @@ class TestBase:
         return True
 
     def is_32bit(self):
-        EI_NIDENT = 16
+        return Elf.is_32bit('t-' + self.name)
 
-        # e_ident[] indexes
-        EI_CLASS      = 4
-
-        # EI_CLASS
-        ELFCLASSNONE  = 0
-        ELFCLASS32    = 1
-        ELFCLASS64    = 2
-        ELFCLASSNUM   = 3
-
-        try:
-            elfname = 't-' + self.name
-            f = open(elfname, 'rb')
-            elf_ident = list(f.read(EI_NIDENT))
-            ei_class = ord(elf_ident[EI_CLASS])
-            f.close()
-
-            if ei_class == ELFCLASS32:
-                return True
-        except:
-            pass
-
-        return False
+    def get_machine(self):
+        return os.uname()[4]
 
     def get_elf_machine(self):
-        EI_NIDENT = 16
+        return Elf.get_elf_machine('t-' + self.name)
 
-        # e_machine (architecture)
-        EM_386        = 3       # Intel 80386
-        EM_ARM        = 40      # ARM
-        EM_X86_64     = 62      # AMD x86-64 architecture
-        EM_AARCH64    = 183     # ARM AARCH64
+    def check_arch_full_dynamic_support(self):
+        elf_machine = TestBase.get_elf_machine(self)
+        if elf_machine == 'x86_64' or elf_machine == 'aarch64':
+            return True
+        return False
 
-        machine = {
-            EM_386: 'i386',
-            EM_ARM: 'arm',
-            EM_X86_64: 'x86_64',
-            EM_AARCH64: 'aarch64'
-        }
+    def check_arch_mfentry_mnop_mcount_support(self):
+        machine = TestBase.get_machine(self)
+        if machine == 'x86_64' or machine == 'i386':
+            return True
+        return False
 
-        try:
-            elfname = 't-' + self.name
-            f = open(elfname, 'rb')
-
-            # consume elf_ident and e_type
-            f.read(EI_NIDENT + 2)
-
-            # read e_machine
-            e_machine = ord(f.read(2)[0])
-            f.close()
-
-            return machine[e_machine]
-        except:
-            pass
-
-        return None
+    def check_arch_sdt_support(self):
+        machine = TestBase.get_machine(self)
+        if machine == 'x86_64':
+            return True
+        return False
 
     def prerun(self, timeout):
-        ret = TestBase.TEST_SUCCESS
+        self.subcmd = 'live'
+        self.option = ''
+        self.exearg = 't-' + self.name
+
+        cmd = self.prepare()
+        if cmd == '':
+            return TestBase.TEST_SUCCESS
+
+        self.pr_debug("prerun command: " + cmd)
 
         class Timeout(Exception):
             pass
@@ -439,12 +549,13 @@ class TestBase:
         def timeout_handler(sig, frame):
             raise Timeout
 
+        ret = TestBase.TEST_SUCCESS
         import signal
         signal.signal(signal.SIGALRM, timeout_handler)
-        signal.alarm(timeout)
 
         try:
-            ret = self.pre()
+            signal.alarm(timeout)
+            sp.call(cmd.split())
         except Timeout:
             ret = TestBase.TEST_TIME_OUT
         signal.alarm(0)
@@ -455,10 +566,16 @@ class TestBase:
         ret = TestBase.TEST_SUCCESS
         dif = ''
 
+        self.setup()
         test_cmd = self.runcmd()
         self.pr_debug("test command: %s" % test_cmd)
 
-        p = sp.Popen(test_cmd, shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
+        if self.debug:
+            # In verbose mode, stderr is printed as is without redirection.
+            # This will inform error messages to users when something goes wrong.
+            p = sp.Popen(test_cmd, shell=True, stdout=sp.PIPE)
+        else:
+            p = sp.Popen(test_cmd, shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
 
         class Timeout(Exception):
             pass
@@ -471,10 +588,10 @@ class TestBase:
 
         import signal
         signal.signal(signal.SIGALRM, timeout_handler)
-        signal.alarm(timeout)
 
         timed_out = False
         try:
+            signal.alarm(timeout)
             result_origin = p.communicate()[0].decode(errors='ignore')
         except Timeout:
             result_origin = ''
@@ -484,6 +601,10 @@ class TestBase:
         result_expect = self.sort(self.result)
         result_tested = self.sort(result_origin)  # for python3
 
+        # strip trailing whitespace for each line.
+        result_expect = '\n'.join([line.rstrip() for line in result_expect.split('\n')])
+        result_tested = '\n'.join([line.rstrip() for line in result_tested.split('\n')])
+
         ret = p.wait()
         if ret < 0:
             if timed_out:
@@ -531,7 +652,10 @@ class TestBase:
         return ret, ''
 
     def __del__(self):
-        sp.call(['rm', '-rf', self.test_dir])
+        if self.keep:
+            sp.call(['mv', self.test_dir, TestBase.origdir])
+        else:
+            sp.call(['rm', '-rf', self.test_dir])
 
 RED     = '\033[1;31m'
 GREEN   = '\033[1;32m'
@@ -583,7 +707,7 @@ def check_serial_case(case):
     return tc.serial
 
 
-def run_single_case(case, flags, opts, arg):
+def run_single_case(case, flags, opts, arg, compilers):
     result = []
 
     # for python3
@@ -591,19 +715,22 @@ def run_single_case(case, flags, opts, arg):
     exec("import %s; tc = %s.TestCase()" % (case, case), globals(), _locals)
     tc = _locals['tc']
     tc.set_debug(arg.debug)
+    tc.set_keep(arg.keep)
     timeout = int(arg.timeout)
 
-    for flag in flags:
-        for opt in opts:
-            cflags = ' '.join(["-" + flag, "-" + opt])
-            dif = ''
-            ret = tc.build(tc.name, cflags)
-            if ret == TestBase.TEST_SUCCESS:
-                ret = tc.prerun(timeout)
+    for compiler in compilers:
+        tc.set_compiler(compiler)
+        for flag in flags:
+            for opt in opts:
+                cflags = ' '.join(["-" + flag, "-" + opt])
+                dif = ''
+                ret = tc.build(tc.name, cflags)
                 if ret == TestBase.TEST_SUCCESS:
-                    ret, dif = tc.run(case, cflags, arg.diff, timeout)
-                    ret = tc.post(ret)
-            result.append((ret, dif))
+                    ret = tc.prerun(timeout)
+                    if ret == TestBase.TEST_SUCCESS:
+                        ret, dif = tc.run(case, cflags, arg.diff, timeout)
+                        ret = tc.postrun(ret)
+                result.append((ret, dif))
 
     return result
 
@@ -618,7 +745,7 @@ def save_test_result(result, case, shared):
         shared.total += 1
 
 
-def print_test_result(case, result, diffs, color, ftests):
+def print_test_result(case, result, diffs, color, ftests, nr_compilers):
     plain_result = [text_result[r] for r in result]
 
     if color:
@@ -631,37 +758,58 @@ def print_test_result(case, result, diffs, color, ftests):
             sys.stdout.write(dif + '\n')
 
     output = case[1:4]
-    output += ' %-20s' % case[5:] + ': ' + ' '.join(result_list) + '\n'
-
-    sys.stdout.write(output)
+    output += ' %-20s' % case[5:] + ':'
+    result_per_compiler = len(result_list) // nr_compilers
+    for i in range(nr_compilers):
+        output += ' '
+        if i != 0:
+            output += ' '
+        begin = result_per_compiler * i
+        end = result_per_compiler * (i + 1)
+        output += ' '.join(result_list[begin:end])
+    output += '\n'
 
     # write abnormal test result to failed-tests.txt
     normal = [TestBase.TEST_SUCCESS, TestBase.TEST_SUCCESS_FIXED, TestBase.TEST_SKIP]
     for r in result:
         if r not in normal:
-            output = case[1:4]
-            output += ' %-20s' % case[5:] + ': ' + ' '.join(plain_result) + '\n'
             ftests.write(output)
             ftests.flush()
             break
 
+    if arg.quiet:
+        for r in result:
+            if r not in normal:
+                sys.stdout.write(output)
+                break
+    else:
+        sys.stdout.write(output)
 
-def print_test_header(opts, flags, ftests):
+
+def print_test_header(opts, flags, ftests, compilers):
     optslen = len(opts)
+    header1 = '%-24s ' % 'Compiler'
+    header2 = '%-24s ' % 'Test case'
+    header3 = '-' * 24 + ':'
+    empty = '                             '
 
-    header1 = '%-24s ' % 'Test case'
-    header2 = '-' * 24 + ':'
-    empty = '                      '
-
-    for flag in flags:
-        # align with optimization flags
-        header1 += ' ' + flag[:optslen] + empty[len(flag):optslen]
-        header2 += ' ' + opts
+    for i, compiler in enumerate(compilers):
+        if i != 0:
+            header1 += ' '
+            header2 += ' '
+            header3 += ' '
+        for flag in flags:
+            # align with optimization flags
+            header2 += ' ' + flag[:optslen] + empty[len(flag):optslen]
+            header3 += ' ' + opts
+        header1 += ' ' + compiler[:optslen*len(flags)+len(flags)-1] + empty[len(compiler):len(header3)-len(header1)-1]
 
     print(header1)
     print(header2)
+    print(header3)
     ftests.write(header1 + '\n')
     ftests.write(header2 + '\n')
+    ftests.write(header3 + '\n')
     ftests.flush()
 
 
@@ -700,12 +848,18 @@ def parse_argument():
                         help="show diff result if not matched")
     parser.add_argument("-v", "--verbose", dest='debug', action='store_true',
                         help="show internal command and result for debugging")
-    parser.add_argument("-n", "--no-color", dest='color', action='store_false',
-                        help="suppress color in the output")
+    parser.add_argument("-l", "--color", dest='color', default='auto',
+                        help="set color in the output. 'auto', 'on' or 'off'")
     parser.add_argument("-t", "--timeout", dest='timeout', default=5,
                         help="fail test if it runs more than TIMEOUT seconds")
     parser.add_argument("-j", "--worker", dest='worker', type=int, default=multiprocessing.cpu_count(),
                         help="Parallel worker count; using all core for default")
+    parser.add_argument("-c", "--compiler", dest='compiler', default="all",
+                        help="Select compiler gcc or clang. (use both by default)")
+    parser.add_argument("-k", "--keep", dest='keep', action='store_true',
+                        help="keep the test directories with compiled binaries")
+    parser.add_argument("-q", "--quiet", dest='quiet', action='store_true',
+                        help="Hide normal results and print only abnormal results.")
 
     return parser.parse_args()
 
@@ -727,6 +881,9 @@ if __name__ == "__main__":
                 print("cannot find testcase for : %s" % arg.case)
                 sys.exit(0)
 
+    # Use multiprocessing pool if the number of workers is greater than 1.
+    use_pool = arg.worker > 1
+
     opts = ' '.join(sorted(['O' + o for o in arg.opts]))
 
     if arg.pg_flag:
@@ -736,10 +893,32 @@ if __name__ == "__main__":
     else:
         flags = arg.flags.split()
 
-    from functools import partial
+    def has_compiler(compiler):
+        installed = os.system('command -v %s > /dev/null' % compiler) == 0
+        return installed
 
-    manager = multiprocessing.Manager()
-    shared = manager.dict()
+    compilers = []
+    if arg.compiler == 'all':
+        for compiler in ['gcc', 'clang']:
+            if has_compiler(compiler):
+                compilers.append(compiler)
+    else:
+        if has_compiler(arg.compiler):
+            compilers.append(arg.compiler)
+    nr_compilers = len(compilers)
+    if nr_compilers == 0:
+        print('no compilers available for testing\n')
+        sys.exit(-1)
+
+    from functools import partial
+    class dotdict(dict):
+        """dot.notation access to dictionary attributes"""
+        __getattr__ = dict.get
+        __setattr__ = dict.__setitem__
+        __delattr__ = dict.__delitem__
+
+    manager = multiprocessing.Manager() if use_pool else None
+    shared = manager.dict() if use_pool else dotdict()
 
     shared.tests_count = len(testcases)
     shared.progress = 0
@@ -757,45 +936,68 @@ if __name__ == "__main__":
     res.append(TestBase.TEST_UNSUPP_LANG)
     res.append(TestBase.TEST_SKIP)
 
-    ftests = open("failed-tests.txt", "w")
+    failed_tests = "failed-tests.txt"
+    if os.path.exists(failed_tests):
+        os.rename(failed_tests, failed_tests + ".old")
+    ftests = open(failed_tests, "w")
 
     shared.stats = dict.fromkeys(res, 0)
-    pool = multiprocessing.Pool(arg.worker)
-    serial_pool = multiprocessing.Pool(1)
+    pool = multiprocessing.Pool(arg.worker) if use_pool else None
+    serial_pool = multiprocessing.Pool(1) if use_pool else None
 
-    for tc in sorted(testcases):
-        _pool = pool
-        name = tc.split('.')[0]  # remove '.py'
-        clbk = partial(save_test_result, case=name, shared=shared)
-        if check_serial_case(name):
-            _pool = serial_pool
+    if use_pool:
+        print("Start %s tests with %d worker" % (shared.tests_count, arg.worker))
+    else:
+        print("Start %s tests without worker pool" % shared.tests_count)
+    print_test_header(opts, flags, ftests, compilers)
 
-        _pool.apply_async(run_single_case, callback=clbk,
-                         args=[name, flags, opts.split(), arg])
-
-    print("Start %s tests with %d worker" % (shared.tests_count, arg.worker))
-    print_test_header(opts, flags, ftests)
-
-    color = arg.color
-    if not sys.stdout.isatty():
-        color = False
-    if 'TERM' in os.environ and os.environ['TERM'] == 'dumb':
+    color = True
+    if arg.color == 'auto':
+        if not sys.stdout.isatty():
+            color = False
+        if 'TERM' in os.environ and os.environ['TERM'] == 'dumb':
+            color = False
+    elif arg.color == 'on':
+        color = True
+    elif arg.color == 'off':
         color = False
+    else:
+        print("unknown color: %s" % arg.color)
+        sys.exit(-1)
 
     for tc in sorted(testcases):
         name = tc.split('.')[0]  # remove '.py'
+        if use_pool:
+            _pool = serial_pool if check_serial_case(name) else pool
+            clbk = partial(save_test_result, case=name, shared=shared)
 
-        while name not in shared.results:
-            time.sleep(1)
+            _pool.apply_async(run_single_case, callback=clbk,
+                            args=[name, flags, opts.split(), arg, compilers])
+        else:
+            results = run_single_case(name, flags, opts.split(), arg, compilers)
+            save_test_result(results, case=name, shared=shared)
 
-        print_test_result(name, shared.results[name], shared.diffs[name], color, ftests)
+            # Print sequentially when executing in serial
+            print_test_result(name, shared.results[name], shared.diffs[name], color, ftests, nr_compilers)
 
-    pool.close()
-    pool.join()
+    if use_pool:
+        # Print via polling when using multiprocessing pool
+        for tc in sorted(testcases):
+            name = tc.split('.')[0]  # remove '.py'
+
+            while name not in shared.results:
+                time.sleep(1)
+
+            print_test_result(name, shared.results[name], shared.diffs[name], color, ftests, nr_compilers)
+
+    if use_pool:
+        pool.close()
+        pool.join()
 
     ftests.close()
 
     sys.stdout.write("\n")
     sys.stdout.flush()
 
-    print_test_report(color, shared)
+    if shared.total >= 30:
+        print_test_report(color, shared)
