@@ -1,18 +1,21 @@
 /*
  * This test calls posix_spawn() to run the t-abc and t-openclose executables.
  */
-#include <stdio.h>
 #include <spawn.h>
-#include <unistd.h>
+#include <stdio.h>
 #include <sys/wait.h>
+#include <unistd.h>
 
-#define TEST_PROG1  "t-abc"
-#define TEST_PROG2  "t-openclose"
+#define TEST_PROG1 "t-abc"
+#define TEST_PROG2 "t-openclose"
 
 int main(int argc, char *argv[])
 {
 	int pid;
-	char *args[] = { NULL, NULL, };
+	char *args[] = {
+		NULL,
+		NULL,
+	};
 	char *envp[] = { "PATH=.", "HOME=/home/user", NULL };
 
 	args[0] = TEST_PROG1;
