@@ -58,5 +58,5 @@ class TestCase(TestBase):
  658.759 us [ 1429] | } /* main */
 """)
 
-    def runcmd(self):
-        return '%s --no-merge %s' % (TestBase.uftrace_cmd, 't-' + self.name)
+    def setup(self):
+        self.option = '--no-merge'
