@@ -15,5 +15,5 @@ class TestCase(TestBase):
    2.405 us [28141] | } /* a */
 """, sort='simple')
 
-    def runcmd(self):
-        return '%s -F a %s' % (TestBase.uftrace_cmd, 't-abc')
+    def setup(self):
+        self.option = '-F a'
