@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import os
 
+from runtest import TestBase
+
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'getids', serial=True, result="""
@@ -23,26 +24,22 @@ class TestCase(TestBase):
   81.933 us [20769] | } /* main */
 """)
 
-    def pre(self):
+    def prerun(self, timeout):
         if os.geteuid() != 0:
             return TestBase.TEST_SKIP
         if os.path.exists('/.dockerenv'):
             return TestBase.TEST_SKIP
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        program = 't-' + self.name
-        options = "-k -F sys_gete.*@kernel"
-
-        return '%s %s %s' % (uftrace, options, program)
+    def setup(self):
+        self.option = "-k -F sys_gete.*@kernel"
 
     def fixup(self, cflags, result):
         uname = os.uname()
         # Linux v4.17 (x86_64) changed syscall routines
         major, minor, release = uname[2].split('.')
         if uname[0] == 'Linux' and uname[4] == 'x86_64' and \
-           int(major) >= 4 and int(minor) >= 17:
+           int(major) >= 5 or (int(major) == 4 and int(minor) >= 17):
             result = result.replace('sys_gete', '__x64_sys_gete')
 
         return result
