@@ -1,35 +1,28 @@
 #!/usr/bin/env python
 
 from runtest import TestBase
-import subprocess as sp
-
-TDIR='xxx'
 
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'sort', """
-    Avg self    Min self    Max self  Function
-  ==========  ==========  ==========  ====================================
-    1.078 ms    1.078 ms    1.078 ms  usleep
-   71.683 us   71.683 us   71.683 us  main
-   70.176 us   70.176 us   70.176 us  __monstartup   # ignore this
-    1.813 us    1.813 us    1.813 us  bar
-    1.051 us    0.868 us    1.912 us  loop
-    1.002 us    1.002 us    1.002 us  __cxa_atexit   # and this too
-    0.509 us    0.359 us    0.660 us  foo
+    Self avg    Self min    Self max  Function
+  ==========  ==========  ==========  ====================
+   10.288 ms   10.288 ms   10.288 ms  usleep
+  598.518 us  598.518 us  598.518 us  main
+  249.854 us  249.854 us  249.854 us  bar
+   39.967 us   39.801 us   40.275 us  loop
+    1.044 us    0.884 us    1.205 us  foo
+    0.701 us    0.701 us    0.701 us  __monstartup   # ignore this
+    0.270 us    0.270 us    0.270 us  __cxa_atexit   # and this too
 """)
 
-    def pre(self):
-        record_cmd = '%s record -d %s %s' % (TestBase.uftrace_cmd, TDIR, 't-sort')
-        sp.call(record_cmd.split())
-        return TestBase.TEST_SUCCESS
+    def prepare(self):
+        self.subcmd = 'record'
+        return self.runcmd()
 
-    def runcmd(self):
-        return '%s report --avg-self -d %s' % (TestBase.uftrace_cmd, TDIR)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+    def setup(self):
+        self.subcmd = 'report'
+        self.option = '--avg-self'
 
     def sort(self, output):
         """ This function post-processes output of the test to be compared .
@@ -39,7 +32,7 @@ class TestCase(TestBase):
             if ln.strip() == '':
                 continue
             line = ln.split()
-            if line[0] == 'Avg':
+            if line[1] == 'avg':
                 continue
             if line[0].startswith('='):
                 continue
