@@ -15,11 +15,6 @@ class TestCase(TestBase):
    4.097 us [12703] | } /* main */
 """)
 
-    def fixup(self, cflags, result):
-        if TestBase.get_elf_machine(self) == 'arm':
-            return result.replace('memset', """memset();
-   1.440 us [12703] |     memcpy""");
-
-        return result.replace("""return_large() {
-   1.440 us [12703] |     memset();
-   2.533 us [12703] |   } /* return_large */""", 'return_large();')
+    def setup(self):
+        # to avoid unexpected memcpy in aarch64
+        self.option = '-N memcpy '
