@@ -13,5 +13,5 @@ class TestCase(TestBase):
    2.083 ms [18270] | } /* main */
 """)
 
-    def runcmd(self):
-        return '%s -t 1ms -D3 %s' % (TestBase.uftrace_cmd, 't-' + self.name)
+    def setup(self):
+        self.option = '-t 1ms -D3'
