@@ -21,12 +21,10 @@ class TestCase(TestBase):
 
         return TestBase.build(self, name, cflags, ldflags)
 
-    def runcmd(self):
-        argopt = '-A "int_mul@arg2/x" -A "^int_@arg1,arg2" -A "int_add@arg1/i32"'
+    def setup(self):
+        self.option = '-A "int_mul@arg2/x" -A "^int_@arg1,arg2" -A "int_add@arg1/i32"'
 
         if TestBase.is_32bit(self):
             # int_mul@arg1 is a 'long long', so we should skip arg2
-            argopt  = '-A "int_mul@arg3/x" -A "^int_@arg1" '
-            argopt += '-A "int_(add|sub|div)@arg2" -A "int_mul@arg1/i64" '
-
-        return '%s %s %s' % (TestBase.uftrace_cmd, argopt, 't-' + self.name)
+            self.option  = '-A "int_mul@arg3/x" -A "^int_@arg1" '
+            self.option += '-A "int_(add|sub|div)@arg2" -A "int_mul@arg1/i64" '
