@@ -23,5 +23,5 @@ class TestCase(TestBase):
         return TestBase.build_libmain(self, name, 's-libmain.c',
                                       ['libabc_test_lib.so'])
 
-    def runcmd(self):
-        return '%s --force --no-libcall -A ^lib@libabc_test,arg1 %s' % (TestBase.uftrace_cmd, 't-' + self.name)
+    def setup(self):
+        self.option = '--force --no-libcall -A ^lib@libabc_test,arg1'
