@@ -1,10 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
-import subprocess as sp
 import os
+import subprocess as sp
 
-TDIR='xxx'
+from runtest import TestBase
 
 class TestCase(TestBase):
     def __init__(self):
@@ -28,26 +27,23 @@ class TestCase(TestBase):
  849.948 us [ 1661] | } /* main */
 """)
 
-    def pre(self):
+    def prerun(self, timeout):
         if os.geteuid() != 0:
             return TestBase.TEST_SKIP
         if os.path.exists('/.dockerenv'):
             return TestBase.TEST_SKIP
 
-        uftrace = TestBase.uftrace_cmd
-        program = 't-' + self.name
+        self.subcmd  = 'record'
+        self.option  = '-k --match glob '
+        self.option += '-N *page_fault@kernel'
 
-        argument  = '-k --match glob -d %s' % TDIR
-        argument += ' -N *page_fault@kernel'
-
-        record_cmd = '%s record %s %s' % (uftrace, argument, program)
+        record_cmd = self.runcmd()
         sp.call(record_cmd.split())
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        import os.path
+    def setup(self):
         t = 0
-        for ln in open(os.path.join(TDIR, 'task.txt')):
+        for ln in open(os.path.join('uftrace.data', 'task.txt')):
             if not ln.startswith('TASK'):
                 continue
             try:
@@ -55,12 +51,11 @@ class TestCase(TestBase):
             except:
                 pass
         if t == 0:
-            return 'FAILED TO FIND TID'
-        return '%s replay -k -d %s --tid %d' % (TestBase.uftrace_cmd, TDIR, t)
+            self.subcmd = 'FAILED TO FIND TID'
+            return
 
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+        self.subcmd = 'replay'
+        self.option = '-k --tid %d' % t
 
     def fixup(self, cflags, result):
         result = result.replace("            [ 1661] |   fork() {",
@@ -79,7 +74,7 @@ class TestCase(TestBase):
         # Linux v4.17 (x86_64) changed syscall routines
         major, minor, release = uname[2].split('.')
         if uname[0] == 'Linux' and uname[4] == 'x86_64' and \
-           int(major) >= 4 and int(minor) >= 17:
+           int(major) >= 5 or (int(major) == 4 and int(minor) >= 17):
             result = result.replace('sys_', '__x64_sys_')
 
         return result
