@@ -25,9 +25,6 @@ class TestCase(TestBase):
  199.848 us [ 3336] | } /* main */
 """)
 
-    def runcmd(self):
-        return '%s %s' % (TestBase.uftrace_cmd, 't-' + self.name)
-
     def fixup(self, cflags, result):
         return result.replace('tsd_dtor();', """tsd_dtor() {
    0.347 us [ 3336] |     free();
