@@ -26,5 +26,5 @@ class TestCase(TestBase):
             [22304] | } /* main */
 """)
 
-    def runcmd(self):
-        return '%s --no-merge %s' % (TestBase.uftrace_cmd, 't-' + self.name)
+    def setup(self):
+        self.option = '--no-merge'
