@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import subprocess as sp
 
-TDIR='xxx'
+from runtest import TestBase
+
 START=0
 
 class TestCase(TestBase):
@@ -18,14 +18,20 @@ class TestCase(TestBase):
    5.626 us | } /* main */
 """, sort='simple')
 
-    def pre(self):
+    def prerun(self, timeout):
         global START
 
-        record_cmd = '%s record -d %s %s' % (TestBase.uftrace_cmd, TDIR, 't-' + self.name)
+        self.subcmd = 'record'
+        record_cmd = self.runcmd()
+        self.pr_debug("prerun command: " + record_cmd)
         sp.call(record_cmd.split())
 
         # find timestamp of function 'c'
-        replay_cmd = '%s replay -d %s -f elapsed -F main' % (TestBase.uftrace_cmd, TDIR)
+        self.subcmd = 'replay'
+        self.option = '-f elapsed -F main'
+        replay_cmd = self.runcmd()
+        self.pr_debug("prerun command: " + replay_cmd)
+
         p = sp.Popen(replay_cmd, shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
         r = p.communicate()[0].decode(errors='ignore')
         START, unit = r.split('\n')[4].split()[0:2] # skip header, main, a and b (= 4)
@@ -34,9 +40,6 @@ class TestCase(TestBase):
 
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        return '%s replay -f elapsed -r %s~ -d %s' % (TestBase.uftrace_cmd, START, TDIR)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+    def setup(self):
+        self.subcmd = 'replay'
+        self.option = '-f elapsed -r %s~' % START
