@@ -19,6 +19,6 @@ class TestCase(TestBase):
 
         return TestBase.build(self, name, cflags, ldflags)
 
-    def runcmd(self):
-        return '%s -A printf@arg1/s,arg2/s %s %s' % \
-            (TestBase.uftrace_cmd, 't-' + self.name, "0123456789" * 10)
+    def setup(self):
+        self.option  = '-A printf@arg1/s,arg2/s'
+        self.exearg += ' ' + "0123456789" * 10
