@@ -12,18 +12,16 @@ class TestCase(TestBase):
    3.005 us [28141] | } /* main */
 """)
 
-    def pre(self):
-        if TestBase.get_elf_machine(self) == 'arm':
-            return TestBase.TEST_SKIP
-        return TestBase.TEST_SUCCESS
-
     def build(self, name, cflags='', ldflags=''):
+        if not TestBase.check_arch_mfentry_mnop_mcount_support(self):
+            return TestBase.TEST_SKIP
+        if cflags.find('-finstrument-functions') >= 0:
+             return TestBase.TEST_SKIP
+        if self.supported_lang['C']['cc'] == 'clang':
+            return TestBase.TEST_SKIP
         cflags += ' -mfentry -mnop-mcount'
         cflags += ' -fno-pie -fno-plt'  # workaround of build failure
         return TestBase.build(self, name, cflags, ldflags)
 
-    def runcmd(self):
-        uftrace  = TestBase.uftrace_cmd
-        argument = '-P %s --no-libcall' % 'a.?'
-        program  = 't-' + self.name
-        return '%s %s %s' % (uftrace, argument, program)
+    def setup(self):
+        self.option = '-P "a.?" --no-libcall'
