@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import os
 
+from runtest import TestBase
+
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'openclose', serial=True, result="""
@@ -15,7 +16,7 @@ class TestCase(TestBase):
   37.051 us [ 9875] | } /* main */
 """)
 
-    def pre(self):
+    def prerun(self, timeout):
         if os.geteuid() != 0:
             return TestBase.TEST_SKIP
         if os.path.exists('/.dockerenv'):
@@ -24,9 +25,8 @@ class TestCase(TestBase):
         return TestBase.TEST_SUCCESS
 
     # check syscall name would corrected (for SyS_ prefix)
-    def runcmd(self):
-        return '%s -k -P %s %s openclose' % \
-            (TestBase.uftrace_cmd, '_*sys_open@kernel', 't-' + self.name)
+    def setup(self):
+        self.option = "-k -P '_*sys_open@kernel'"
 
     def fixup(self, cflags, result):
         uname = os.uname()
@@ -34,7 +34,7 @@ class TestCase(TestBase):
         # Linux v4.17 (x86_64) changed syscall routines
         major, minor, release = uname[2].split('.')
         if uname[0] == 'Linux' and uname[4] == 'x86_64' and \
-           int(major) >= 4 and int(minor) >= 17:
+           int(major) >= 5 or (int(major) == 4 and int(minor) >= 17):
             return result.replace(' sys_open', ' __x64_sys_openat')
         else:
             return result.replace(' sys_open', ' sys_openat')
