@@ -25,11 +25,8 @@ task: 6565
 [0] main
 """, lang='C++')
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '-F main -T ns::ns1::foo::bar3@finish'
-        program = 't-' + self.name
-        return '%s %s %s' % (uftrace, options, program)
+    def setup(self):
+        self.option = '-F main -T ns::ns1::foo::bar3@finish'
 
     def fixup(self, cflags, result):
         return result.replace("""ns::ns1::foo::bar3() {
