@@ -1,30 +1,26 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import subprocess as sp
 
-TDIR='xxx'
+from runtest import TestBase
 
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'abc', '5')
 
-    def pre(self):
-        script_cmd = '%s script -S %s/scripts/count.py -d %s --record %s' % \
-                     (TestBase.uftrace_cmd, self.basedir, TDIR, 't-abc')
+    def prerun(self, timeout):
+        self.subcmd = 'script'
+        self.option = '-S %s/scripts/count.py --record' % self.basedir
+        script_cmd = self.runcmd()
+        self.pr_debug('prerun command: ' + script_cmd)
+
         p = sp.Popen(script_cmd.split(), stdout=sp.PIPE, stderr=sp.PIPE)
         if p.communicate()[1].decode(errors='ignore').startswith('WARN:'):
             return TestBase.TEST_SKIP
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '-F main -S %s/scripts/count.py' % self.basedir
-        return '%s script -d %s %s' % (uftrace, TDIR, options)
+    def setup(self):
+        self.option = '-F main -S %s/scripts/count.py' % self.basedir
 
     def sort(self, output):
         return output.strip()
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
