@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import subprocess as sp
 
+from runtest import TestBase
+
 XDIR='xxx'
 YDIR='yyy'
 
@@ -14,30 +15,35 @@ class TestCase(TestBase):
 #  [0] base: xxx   (from uftrace record -d xxx -F main tests/t-diff 0 )
 #  [1] diff: yyy   (from uftrace record -d yyy -F main tests/t-diff 1 )
 #
-  Total time    Self time        Calls   Function
-  ==========   ==========   ==========   ====================
-      -5.40%       -5.40%           +0   atoi
-    +999.99%     +875.84%           +0   bar
-    +999.99%     +498.85%           +0   foo
-    +999.99%      +10.47%           +0   main
-    +999.99%     +999.99%           +3   usleep
+    Total time     Self time         Calls   Function
+   ===========   ===========   ===========   ====================
+        +2.99%        +2.99%            +0   atoi
+      +195.87%      +104.40%            +2   bar
+       +95.74%       +16.14%            +1   foo
+      +113.47%       +88.88%            +0   main
+      +113.96%      +113.96%            +4   usleep
 """)
 
-    def pre(self):
-        record_cmd = '%s record -d %s -F main %s 0' % (TestBase.uftrace_cmd, XDIR, 't-' + self.name)
+    def prerun(self, timeout):
+        self.subcmd = 'record'
+        self.option = '-d %s -F main' % XDIR
+        self.exearg = 't-' + self.name + ' 0'
+        record_cmd = self.runcmd()
+        self.pr_debug('prerun command: ' + record_cmd)
         sp.call(record_cmd.split())
-        record_cmd = '%s record -d %s -F main %s 1' % (TestBase.uftrace_cmd, YDIR, 't-' + self.name)
+
+        self.option = '-d %s -F main' % YDIR
+        self.exearg = 't-' + self.name + ' 1'
+        record_cmd = self.runcmd()
+        self.pr_debug('prerun command: %s' + record_cmd)
         sp.call(record_cmd.split())
+
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '--diff-policy percent -s func'
-        return '%s report -d %s --diff %s %s' % (uftrace, XDIR, YDIR, options)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', XDIR, YDIR])
-        return ret
+    def setup(self):
+        self.subcmd = 'report'
+        self.option = '--diff-policy percent -s func'
+        self.exearg = '-d %s --diff %s' % (XDIR, YDIR)
 
     def sort(self, output):
         """ This function post-processes output of the test to be compared .
