@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import subprocess as sp
 
-TDIR='xxx'
+from runtest import TestBase
+
 FUNC='main'
 
 class TestCase(TestBase):
@@ -18,26 +18,32 @@ class TestCase(TestBase):
   10.329 ms : (1) main
   53.100 us :  +-(2) foo
   50.745 us :  | (6) loop
-            :  | 
+            :  |
   10.150 ms :  +-(1) bar
   10.102 ms :    (1) usleep
   10.088 ms :    (1) linux:schedule
 """, sort='graph')
 
-    def pre(self):
+    def prerun(self, timeout):
         if not TestBase.check_dependency(self, 'perf_context_switch'):
             return TestBase.TEST_SKIP
         if not TestBase.check_perf_paranoid(self):
             return TestBase.TEST_SKIP
 
-        options = '-d %s -E %s' % (TDIR, 'linux:schedule')
-        record_cmd = '%s record %s %s' % (TestBase.uftrace_cmd, options, 't-' + self.name)
+        self.subcmd = 'record'
+        self.option = '-E linux:schedule'
+        self.exearg = 't-' + self.name
+
+        record_cmd = TestBase.runcmd(self)
+        self.pr_debug('prerun command: ' + record_cmd)
         sp.call(record_cmd.split())
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        return '%s graph -d %s %s' % (TestBase.uftrace_cmd.split()[0], TDIR, FUNC)
+    def setup(self):
+        self.subcmd = 'graph'
+        self.option = ''
+        self.exearg = 'main'
 
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+    def runcmd(self):
+        cmd = TestBase.runcmd(self)
+        return cmd.replace('--no-event', '')
