@@ -1,9 +1,8 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import subprocess as sp
 
-TDIR='xxx'
+from runtest import TestBase
 
 class TestCase(TestBase):
     def __init__(self):
@@ -39,20 +38,23 @@ class TestCase(TestBase):
 } }
 """, sort='chrome')
 
-    def pre(self):
+    def prerun(self, timeout):
         if not TestBase.check_dependency(self, 'perf_context_switch'):
             return TestBase.TEST_SKIP
         if not TestBase.check_perf_paranoid(self):
             return TestBase.TEST_SKIP
 
-        options = '-d %s -E %s' % (TDIR, 'linux:schedule')
-        record_cmd = '%s record %s %s' % (TestBase.uftrace_cmd, options, 't-' + self.name)
+        self.subcmd = 'record'
+        self.option = '-E  linux:schedule'
+        record_cmd = TestBase.runcmd(self)
+        self.pr_debug('prerun command: ' + record_cmd)
         sp.call(record_cmd.split())
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        return '%s dump -d %s -F main --chrome' % (TestBase.uftrace_cmd.split()[0], TDIR)
+    def setup(self):
+        self.subcmd = 'dump'
+        self.option = '-F main --chrome'
 
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+    def runcmd(self):
+        cmd = TestBase.runcmd(self)
+        return cmd.replace('--no-event', '')
