@@ -1,10 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
-import subprocess as sp
 import os
+import subprocess as sp
 
-TDIR='xxx'
+from runtest import TestBase
 
 class TestCase(TestBase):
     def __init__(self):
@@ -18,33 +17,24 @@ class TestCase(TestBase):
  128.387 us [18343] | } /* main */
 """)
 
-    def pre(self):
+    def prerun(self, timeout):
         if os.geteuid() != 0:
             return TestBase.TEST_SKIP
         if os.path.exists('/.dockerenv'):
             return TestBase.TEST_SKIP
 
-        uftrace  = TestBase.uftrace_cmd
-        options  = '-k -d ' + TDIR
-        program  = 't-' + self.name
+        self.subcmd = 'record'
+        self.option = '-k'
 
-        record_cmd = '%s record %s %s' % (uftrace, options, program)
+        record_cmd = self.runcmd()
+        self.pr_debug('prerun command: ' + record_cmd)
         sp.call(record_cmd.split())
 
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        uname = os.uname()
-
-        kfunc = 'sys_open*@kernel'
-
-        argument = '-F main -D2 -F %s -d %s' % (kfunc, TDIR)
-        return '%s replay %s' % (uftrace, argument)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+    def setup(self):
+        self.subcmd = 'replay'
+        self.option = '-F main -D2 -F sys_open*@kernel'
 
     def fixup(self, cflags, result):
         uname = os.uname()
@@ -52,7 +42,7 @@ class TestCase(TestBase):
         # Linux v4.17 (x86_64) changed syscall routines
         major, minor, release = uname[2].split('.')
         if uname[0] == 'Linux' and uname[4] == 'x86_64' and \
-           int(major) >= 4 and int(minor) >= 17:
+           int(major) >= 5 or (int(major) == 4 and int(minor) >= 17):
             result = result.replace('sys_', '__x64_sys_')
 
         return result.replace(' sys_open', ' sys_openat')
