@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import subprocess as sp
 
+from runtest import TestBase
+
 XDIR='xxx'
 YDIR='yyy'
 
@@ -14,30 +15,37 @@ class TestCase(TestBase):
 #  [0] base: xxx   (from uftrace record -d xxx -F main tests/t-diff 0 )
 #  [1] diff: yyy   (from uftrace record -d yyy -F main tests/t-diff 1 )
 #
-   Total time     Self time         Calls   Function
-  ===========   ===========   ===========   ====================
-    +0.027 us     +0.027 us            +0   atoi
-  +158.853 us     +1.319 us            +0   bar
-    +1.235 ms     +2.749 us            +0   foo
-    +1.305 ms     +0.319 us            +0   main
-    +1.300 ms     +1.300 ms            +3   usleep
+    Total time     Self time         Calls   Function
+   ===========   ===========   ===========   ====================
+     -0.092 us     -0.092 us            +0   atoi
+   -315.716 us     -2.259 us            +2   bar
+     -1.318 ms     -3.505 us            +1   foo
+     -1.548 ms     -1.632 us            +0   main
+     -1.540 ms     -1.540 ms            +4   usleep
 """)
 
-    def pre(self):
-        record_cmd = '%s record -d %s -F main %s 0' % (TestBase.uftrace_cmd, XDIR, 't-' + self.name)
+    def prerun(self, timeout):
+        self.subcmd = 'record'
+        self.option = '-d %s -F main' % XDIR
+        self.exearg = 't-' + self.name + ' 0'
+
+        record_cmd = self.runcmd()
+        self.pr_debug('prerun command: ' + record_cmd)
         sp.call(record_cmd.split())
-        record_cmd = '%s record -d %s -F main %s 1' % (TestBase.uftrace_cmd, YDIR, 't-' + self.name)
+
+        self.option = '-d %s -F main' % YDIR
+        self.exearg = 't-' + self.name + ' 1'
+
+        record_cmd = self.runcmd()
+        self.pr_debug('prerun command: ' + record_cmd)
         sp.call(record_cmd.split())
+
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '--diff-policy compact -s func'
-        return '%s report -d %s --diff %s %s' % (uftrace, XDIR, YDIR, options)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', XDIR, YDIR])
-        return ret
+    def setup(self):
+        self.subcmd = 'report'
+        self.option = '--diff-policy compact -s func'
+        self.exearg = '-d %s --diff %s' % (XDIR, YDIR)
 
     def sort(self, output):
         """ This function post-processes output of the test to be compared .
