@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import re
 
+from runtest import TestBase
+
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'autoargs', result="""hello
@@ -16,8 +17,9 @@ class TestCase(TestBase):
   18.574 us [16523] | } /* main */
 """)
 
-    def runcmd(self):
-        return '%s -F main --auto-args %s hello' % (TestBase.uftrace_cmd, 't-' + self.name)
+    def setup(self):
+        self.option  = '-F main --auto-args'
+        self.exearg += ' hello'
 
     def sort(self, output):
         result = []
