@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import re
 
+from runtest import TestBase
+
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'autoargs', result="""hello
@@ -17,9 +18,9 @@ class TestCase(TestBase):
 """)
 
     # override calloc() to save 2nd argument only
-    def runcmd(self):
-        return '%s -F main -A calloc@arg2 --auto-args %s hello' % \
-            (TestBase.uftrace_cmd, 't-' + self.name)
+    def setup(self):
+        self.option  = '-F main -A calloc@arg2 --auto-args'
+        self.exearg += ' hello'
 
     def sort(self, output):
         result = []
