@@ -1,10 +1,6 @@
 #!/usr/bin/env python
 
 from runtest import TestBase
-import subprocess as sp
-
-TDIR='xxx'
-FUNC='main'
 
 class TestCase(TestBase):
     def __init__(self):
@@ -19,19 +15,16 @@ class TestCase(TestBase):
    10.148 ms   37.889 us  560e956bd610 : (1) main
    15.991 us    0.765 us  560e956bd7ce :  +-(2) foo
    15.226 us   15.226 us  560e956bd7a0 :  | (6) loop
-                                       :  | 
+                                       :  |
    10.094 ms   13.365 us  560e956bd802 :  +-(1) bar
    10.081 ms   10.081 ms  560e956bd608 :    (1) usleep
 """, sort='graph')
 
-    def pre(self):
-        record_cmd = '%s record -d %s %s' % (TestBase.uftrace_cmd, TDIR, 't-' + self.name)
-        sp.call(record_cmd.split())
-        return TestBase.TEST_SUCCESS
+    def prepare(self):
+        self.subcmd = 'record'
+        return self.runcmd()
 
-    def runcmd(self):
-        return '%s graph -f +self,addr -d %s %s' % (TestBase.uftrace_cmd, TDIR, FUNC)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+    def setup(self):
+        self.subcmd = 'graph'
+        self.option = '-f +self,addr'
+        self.exearg = 'main'
