@@ -1,9 +1,8 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import subprocess as sp
 
-TDIR='xxx'
+from runtest import TestBase
 
 class TestCase(TestBase):
     def __init__(self):
@@ -13,26 +12,29 @@ v0.8.3-10/gfbfac3
 ('foo', 'bar')
 """)
 
-    def pre(self):
-        script_cmd = '%s script' % (TestBase.uftrace_cmd)
+    def prerun(self, timeout):
+        self.subcmd = 'script'
+        self.option = ''
+        self.exearg = ''
+
+        script_cmd = self.runcmd()
         p = sp.Popen(script_cmd.split(), stdout=sp.PIPE, stderr=sp.PIPE)
         if p.communicate()[1].decode(errors='ignore').startswith('WARN:'):
             return TestBase.TEST_SKIP
 
-        record_cmd = '%s record -d %s %s' % (TestBase.uftrace_cmd, TDIR, 't-abc')
+        self.subcmd = 'record'
+        self.exearg = 't-' + self.name
+        record_cmd = self.runcmd()
+        self.pr_debug('prerun command: ' + record_cmd)
         sp.call(record_cmd.split())
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '-F main -S %s/scripts/info.py foo bar' % self.basedir
-        return '%s script -d %s %s' % (uftrace, TDIR, options)
+    def setup(self):
+        self.subcmd = 'script'
+        self.option = '-F main -S %s/scripts/info.py' % self.basedir
+        self.exearg = 'foo bar'
 
     def sort(self, output):
         result = output.strip().split('\n')
         result[1] = 'uftrace version'  # overwrite the version number
         return '\n'.join(result)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
