@@ -1,5 +1,7 @@
 #!/usr/bin/env python
 
+import os
+
 from runtest import TestBase
 
 class TestCase(TestBase):
@@ -20,6 +22,7 @@ class TestCase(TestBase):
   48.519 us [ 29510] |   dlclose();
  465.432 us [ 29510] | } /* main */
 """)
+        os.environ['LD_LIBRARY_PATH'] = "."
 
     def build(self, name, cflags='', ldflags=''):
         if TestBase.build_libfoo(self, 'bar', cflags, ldflags) != 0:
@@ -29,5 +32,5 @@ class TestCase(TestBase):
         return TestBase.build_libmain(self, name, 's-dlopen2.cpp', ['libdl.so'],
                                       cflags, ldflags)
 
-    def runcmd(self):
-        return 'LD_LIBRARY_PATH=. %s -F a %s' % (TestBase.uftrace_cmd, 't-dlopen2')
+    def setup(self):
+        self.option = '-F a'
