@@ -18,12 +18,12 @@ class TestCase(TestBase):
 """, cflags='-g')
 
     def build(self, name, cflags='', ldflags=''):
+        if not 'dwarf' in self.feature:
+            return TestBase.TEST_SKIP
         if cflags.find('-finstrument-functions') >= 0:
             return TestBase.TEST_SKIP
 
         return TestBase.build(self, name, cflags, ldflags)
 
-    def runcmd(self):
-        return '%s %s %s' % (TestBase.uftrace_cmd,
-                             '-A . -R. -F main',
-                             't-' + self.name)
+    def setup(self):
+        self.option = '-A . -R. -F main'
