@@ -14,15 +14,15 @@ class TestCase(TestBase):
 """, lang='C++', cflags='-g')
 
     def build(self, name, cflags='', ldflags=''):
+        if not 'dwarf' in self.feature:
+            return TestBase.TEST_SKIP
         if cflags.find('-finstrument-functions') >= 0:
             return TestBase.TEST_SKIP
 
         return TestBase.build(self, name, cflags, ldflags)
 
-    def runcmd(self):
-        return '%s %s %s' % (TestBase.uftrace_cmd,
-                             '-A . -R . -F main -N personality_v.',
-                             't-' + self.name)
+    def setup(self):
+        self.option = '-A . -R . -F main -N personality_v.'
 
     def sort(self, output):
         import re
