@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
 import re
 
+from runtest import TestBase
+
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'enum', result="""
@@ -15,17 +16,16 @@ class TestCase(TestBase):
 """, cflags='-g')
 
     def build(self, name, cflags='', ldflags=''):
+        if not "dwarf" in self.feature:
+            return TestBase.TEST_SKIP
         # cygprof doesn't support arguments now
         if cflags.find('-finstrument-functions') >= 0:
             return TestBase.TEST_SKIP
 
         return TestBase.build(self, name, cflags, ldflags)
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '-F main -A kill -A foo'
-        program = 't-' + self.name
-        return '%s %s %s' % (uftrace, options, program)
+    def setup(self):
+        self.option = '-F main -A kill -A foo'
 
     def sort(self, output):
         result = []
