@@ -2,8 +2,6 @@
 
 from runtest import TestBase
 
-TDIR='xxx'
-
 class TestCase(TestBase):
     def __init__(self):
         TestBase.__init__(self, 'taskname', ldflags='-pthread', serial=True, result="""
@@ -11,26 +9,22 @@ class TestCase(TestBase):
         taskname | main() {
         taskname |   task_name1() {
         taskname |     prctl() {
-             foo |       /* linux:task-name (name=foo) */
+             foo |       /* linux:task-name (comm="foo") */
              foo |     } /* prctl */
              foo |   } /* task_name1 */
              foo |   task_name2() {
              foo |     pthread_self();
              foo |     pthread_setname_np() {
-             bar |       /* linux:task-name (name=bar) */
+             bar |       /* linux:task-name (comm="bar") */
              bar |     } /* pthread_setname_np */
              bar |   } /* task_name2 */
              bar | } /* main */
 """, sort='simple')
 
-    def pre(self):
+    def prerun(self, timeout):
         if not TestBase.check_perf_paranoid(self):
             return TestBase.TEST_SKIP
         return TestBase.TEST_SUCCESS
 
-    def runcmd(self):
-        uftrace  = TestBase.uftrace_cmd
-        options = '-F main -E linux:task-name -t 1'
-        program  = 't-' + self.name
-
-        return '%s %s %s' % (uftrace, options, program)
+    def setup(self):
+        self.option = '-F main -E linux:task-name -t 1'
