@@ -20,6 +20,11 @@ class TestCase(TestBase):
 7.607 us [ 26661] | } /* main */
 """)
 
+    def prerun(self, timeout):
+        if not TestBase.check_arch_full_dynamic_support(self):
+            return TestBase.TEST_SKIP
+        return TestBase.TEST_SUCCESS
+
     def build(self, name, cflags='', ldflags=''):
         if TestBase.build_notrace_lib(self, 'dyn1', 'libdyn1', cflags, ldflags) != 0:
             return TestBase.TEST_BUILD_FAIL
@@ -30,8 +35,5 @@ class TestCase(TestBase):
                                       ['libdyn1.so', 'libdyn2.so'],
                                       cflags, ldflags, instrument=False)
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '-Pmain -P.@libdyn1.so -P.@libdyn2.so --no-libcall'
-        program = 't-' + self.name
-        return '%s %s %s' % (uftrace, options, program)
+    def setup(self):
+        self.option = '-Pmain -P.@libdyn1.so -P.@libdyn2.so --no-libcall'
