@@ -12,14 +12,16 @@ class TestCase(TestBase):
 
 """)
 
+    def prerun(self, timeout):
+        if not TestBase.check_arch_full_dynamic_support(self):
+            return TestBase.TEST_SKIP
+        return TestBase.TEST_SUCCESS
+
     def build(self, name, cflags='', ldflags=''):
         cflags = cflags.replace('-pg', '')
         cflags = cflags.replace('-finstrument-functions', '')
         cflags += ' -funroll-loops'
         return TestBase.build(self, name, cflags, ldflags)
 
-    def runcmd(self):
-        uftrace = TestBase.uftrace_cmd
-        options = '-P. -Z 100'
-        program = 't-' + self.name
-        return '%s %s %s' % (uftrace, options, program)
+    def setup(self):
+        self.option = '-P. -Z 100'
