@@ -1,11 +1,8 @@
 #!/usr/bin/env python
 
-from runtest import TestBase
-import subprocess as sp
 import re
 
-TDIR='xxx'
-FUNC='main'
+from runtest import TestBase
 
 class TestCase(TestBase):
     def __init__(self):
@@ -13,21 +10,18 @@ class TestCase(TestBase):
 ========== TASK GRAPH ==========
 # TOTAL TIME   SELF TIME     TID     TASK NAME
    16.172 ms  356.762 us  [129306] : t-fork
-                                   :  |    
+                                   :  |
    11.015 us   11.015 us  [129317] :  +----t-fork
 """)
 
-    def pre(self):
-        record_cmd = '%s record -d %s %s' % (TestBase.uftrace_cmd, TDIR, 't-' + self.name)
-        sp.call(record_cmd.split())
-        return TestBase.TEST_SUCCESS
+    def prepare(self):
+        self.subcmd = 'record'
+        return self.runcmd()
 
-    def runcmd(self):
-        return '%s graph --task -d %s %s' % (TestBase.uftrace_cmd, TDIR, FUNC)
-
-    def post(self, ret):
-        sp.call(['rm', '-rf', TDIR])
-        return ret
+    def setup(self):
+        self.subcmd = 'graph'
+        self.option = '--task'
+        self.exearg = ''
 
     def sort(self, output, ignored=False):
         """ This function post-processes output of the test to be compared.
