@@ -1,15 +1,14 @@
-#include <stdio.h>
 #include <fcntl.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <unistd.h>
-#include <sys/wait.h>
 #include <link.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/wait.h>
+#include <unistd.h>
 
-#include "utils/utils.h"
-#include "utils/symbol.h"
 #include "tests/unittest.h"
-
+#include "utils/symbol.h"
+#include "utils/utils.h"
 
 static bool color = true;
 
@@ -18,6 +17,7 @@ TEST_CASE(unittest_framework)
 {
 	static const char hello[] = "Hello";
 
+	pr_dbg("check basic test macro\n");
 	TEST_EQ(1 + 1, 2);
 	TEST_NE(true, false);
 	TEST_GT(1 * 2, 0 * 2);
@@ -25,6 +25,7 @@ TEST_CASE(unittest_framework)
 	TEST_LT(0 * 2, 1);
 	TEST_LE(0.0, 0);
 
+	pr_dbg("check string test macro\n");
 	TEST_STREQ("Hello", hello);
 	TEST_MEMEQ("Hello", hello, sizeof(hello));
 
@@ -32,32 +33,25 @@ TEST_CASE(unittest_framework)
 }
 
 static const char *retcodes[] = {
-	TERM_COLOR_GREEN  "PASS" TERM_COLOR_RESET,
-	TERM_COLOR_RED    "FAIL" TERM_COLOR_RESET,
-	TERM_COLOR_YELLOW "SKIP" TERM_COLOR_RESET,
-	TERM_COLOR_RED    "SIG " TERM_COLOR_RESET,
-	TERM_COLOR_RED    "BAD " TERM_COLOR_RESET,
+	TERM_COLOR_GREEN "PASS" TERM_COLOR_RESET,  TERM_COLOR_RED "FAIL" TERM_COLOR_RESET,
+	TERM_COLOR_YELLOW "SKIP" TERM_COLOR_RESET, TERM_COLOR_RED "SIG " TERM_COLOR_RESET,
+	TERM_COLOR_RED "BAD " TERM_COLOR_RESET,
 };
 
 static const char *retcodes_nocolor[] = {
-	"PASS",
-	"FAIL",
-	"SKIP",
-	"SIG ",
-	"BAD ",
+	"PASS", "FAIL", "SKIP", "SIG ", "BAD ",
 };
 
 static const char *messages[] = {
-	"ran successfully",
-	"failed",
-	"skipped",
-	"signal caught",
-	"unknown result",
+	"ran successfully", "failed", "skipped", "signal caught", "unknown result",
 };
 
 static void set_debug_domain(struct uftrace_unit_test *test)
 {
-#define DOMAIN(x)  { #x, DBG_##x }
+#define DOMAIN(x)                                                                                  \
+	{                                                                                          \
+#x, DBG_##x                                                                        \
+	}
 
 	struct {
 		char *name;
@@ -75,9 +69,9 @@ static void set_debug_domain(struct uftrace_unit_test *test)
 		DOMAIN(SCRIPT),
 		DOMAIN(DWARF),
 		/* some fixup domains */
-		{ "task",       DBG_SESSION },
-		{ "argspec",    DBG_FILTER },
-		{ "trigger",    DBG_FILTER },
+		{ "task", DBG_SESSION },
+		{ "argspec", DBG_FILTER },
+		{ "trigger", DBG_FILTER },
 	};
 	unsigned int i;
 	int count = 0;
@@ -93,12 +87,15 @@ static void set_debug_domain(struct uftrace_unit_test *test)
 		dbg_domain[DBG_UFTRACE] = debug;
 }
 
-static void run_unit_test(struct uftrace_unit_test *test, int *test_stats)
+static void run_unit_test(struct uftrace_unit_test *test, int *test_stats, char *filter)
 {
 	static int count;
 	int status;
 	int ret = TEST_BAD;
 
+	if (filter && !strstr(test->name, filter))
+		return;
+
 	if (debug) {
 		printf("Testing %s...\n", test->name);
 		fflush(stdout);
@@ -113,7 +110,7 @@ static void run_unit_test(struct uftrace_unit_test *test, int *test_stats)
 	if (WIFSIGNALED(status))
 		ret = TEST_SIG;
 	else if (WIFEXITED(status))
-		ret = WEXITSTATUS(status);  /* OK or NG */
+		ret = WEXITSTATUS(status); /* OK or NG */
 
 	if (ret < 0 || ret >= TEST_MAX)
 		ret = TEST_BAD;
@@ -128,8 +125,7 @@ static void run_unit_test(struct uftrace_unit_test *test, int *test_stats)
 
 static unsigned long load_base;
 
-static int find_load_base(struct dl_phdr_info *info,
-			  size_t size, void *arg)
+static int find_load_base(struct dl_phdr_info *info, size_t size, void *arg)
 {
 	unsigned i;
 
@@ -149,7 +145,21 @@ static int find_load_base(struct dl_phdr_info *info,
 	return 1;
 }
 
-static int setup_unit_test(struct uftrace_unit_test **test_cases, size_t *test_num)
+static int sort_tests(const void *tc1, const void *tc2)
+{
+	const struct uftrace_unit_test *test1 = tc1;
+	const struct uftrace_unit_test *test2 = tc2;
+
+	/* keep unittest_framework first */
+	if (!strcmp(test1->name, "unittest_framework"))
+		return -1;
+	if (!strcmp(test2->name, "unittest_framework"))
+		return 1;
+
+	return strcmp(test1->name, test2->name);
+}
+
+static int setup_unit_test(struct uftrace_unit_test **test_cases, size_t *test_num, char *filter)
 {
 	char *exename;
 	struct uftrace_elf_data elf;
@@ -157,7 +167,7 @@ static int setup_unit_test(struct uftrace_unit_test **test_cases, size_t *test_n
 	struct uftrace_unit_test *tcases;
 	bool found_unittest = false;
 	size_t sec_size;
-	unsigned i, num;
+	unsigned i, num, actual;
 	int ret = -1;
 
 	exename = read_exename();
@@ -186,26 +196,38 @@ static int setup_unit_test(struct uftrace_unit_test **test_cases, size_t *test_n
 	dl_iterate_phdr(find_load_base, NULL);
 
 	tcases = xmalloc(sec_size);
-	num    = sec_size / sizeof(*tcases);
+	num = sec_size / sizeof(*tcases);
 
 	elf_get_secdata(&elf, &iter);
 	elf_read_secdata(&elf, &iter, 0, tcases, sec_size);
 
 	/* relocate section symbols in case of PIE */
-	for (i = 0; i < num && load_base; i++) {
+	for (i = 0, actual = 0; i < num && (load_base || filter); i++) {
 		struct uftrace_unit_test *tc = &tcases[i];
 		unsigned long faddr = (unsigned long)tc->func;
 		unsigned long naddr = (unsigned long)tc->name;
 
-	       faddr += load_base;
-	       naddr += load_base;
+		if (load_base) {
+			faddr += load_base;
+			naddr += load_base;
 
-	       tc->func = (void *)faddr;
-	       tc->name = (void *)naddr;
+			tc->func = (void *)faddr;
+			tc->name = (void *)naddr;
+		}
+
+		if (filter && strstr(tc->name, filter))
+			actual++;
 	}
 
+	if (!filter)
+		actual = num;
+
+	printf("Running %u test cases\n======================\n", actual);
+
+	qsort(tcases, num, sizeof(*tcases), sort_tests);
+
 	*test_cases = tcases;
-	*test_num   = num;
+	*test_num = num;
 
 	ret = 0;
 
@@ -225,10 +247,9 @@ static int finish_unit_test(struct uftrace_unit_test *test_cases, int *test_stat
 
 	printf("\n");
 	free(test_cases);
-	return test_stats[TEST_NG]
-		+ test_stats[TEST_BAD]
-		+ test_stats[TEST_SIG]
-		> 0 ? EXIT_FAILURE : EXIT_SUCCESS;
+	return test_stats[TEST_NG] + test_stats[TEST_BAD] + test_stats[TEST_SIG] > 0 ?
+		       EXIT_FAILURE :
+		       EXIT_SUCCESS;
 }
 
 int __attribute__((weak)) arch_fill_cpuinfo_model(int fd)
@@ -236,30 +257,38 @@ int __attribute__((weak)) arch_fill_cpuinfo_model(int fd)
 	return 0;
 }
 
-void mcount_return(void) {}
-void plthook_return(void) {}
-void dynamic_return(void) {}
-void __fentry__(void) {}
-void __dentry__(void) {}
-void __xray_entry(void) {}
-void __xray_exit(void) {}
+void mcount_return(void)
+{
+}
+void plthook_return(void)
+{
+}
+void dynamic_return(void)
+{
+}
+void __fentry__(void)
+{
+}
+void __dentry__(void)
+{
+}
+void __xray_entry(void)
+{
+}
+void __xray_exit(void)
+{
+}
 
 #undef main
 int main(int argc, char *argv[])
 {
 	struct uftrace_unit_test *test_cases = NULL;
-	int test_stats[TEST_MAX] = { };
+	int test_stats[TEST_MAX] = {};
 	size_t i, test_num = 0;
+	char *filter = NULL;
 	char *term;
 	int c;
 
-	if (setup_unit_test(&test_cases, &test_num) < 0) {
-		printf("Cannot run unit tests - failed to load test cases\n");
-		return -1;
-	}
-
-	printf("Running %zd test cases\n======================\n", test_num);
-
 	while ((c = getopt(argc, argv, "dvnpiO:f:")) != -1) {
 		switch (c) {
 		case 'd':
@@ -274,8 +303,16 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (optind < argc)
+		filter = argv[optind];
+
 	outfp = logfp = stdout;
 
+	if (setup_unit_test(&test_cases, &test_num, filter) < 0) {
+		printf("Cannot run unit tests - failed to load test cases\n");
+		return -1;
+	}
+
 	term = getenv("TERM");
 	if (term && !strcmp(term, "dumb"))
 		color = false;
@@ -283,7 +320,7 @@ int main(int argc, char *argv[])
 		color = false;
 
 	for (i = 0; i < test_num; i++)
-		run_unit_test(&test_cases[i], test_stats);
+		run_unit_test(&test_cases[i], test_stats, filter);
 
 	return finish_unit_test(test_cases, test_stats);
 }
