@@ -2,17 +2,18 @@
 #include <stdlib.h>
 
 /* This should be defined before #include "utils.h" */
-#define PR_FMT     "filter"
-#define PR_DOMAIN  DBG_FILTER
+#define PR_FMT "filter"
+#define PR_DOMAIN DBG_FILTER
 
 #include "uftrace.h"
-#include "utils/filter.h"
-#include "utils/utils.h"
-#include "utils/symbol.h"
-#include "utils/rbtree.h"
-#include "utils/list.h"
+#include "utils/argspec.h"
 #include "utils/auto-args.h"
 #include "utils/dwarf.h"
+#include "utils/filter.h"
+#include "utils/list.h"
+#include "utils/rbtree.h"
+#include "utils/symbol.h"
+#include "utils/utils.h"
 
 /* RB-tree maintaining automatic arguments and return value */
 static struct rb_root auto_argspec = RB_ROOT;
@@ -21,9 +22,8 @@ static struct rb_root auto_enum = RB_ROOT;
 
 extern void add_trigger(struct uftrace_filter *filter, struct uftrace_trigger *tr,
 			bool exact_match);
-extern int setup_trigger_action(char *str, struct uftrace_trigger *tr,
-				char **module, unsigned long orig_flags,
-				struct uftrace_filter_setting *setting);
+extern int setup_trigger_action(char *str, struct uftrace_trigger *tr, char **module,
+				unsigned long orig_flags, struct uftrace_filter_setting *setting);
 
 static void add_auto_args(struct rb_root *root, struct uftrace_filter *entry,
 			  struct uftrace_trigger *tr)
@@ -63,8 +63,7 @@ static void add_auto_args(struct rb_root *root, struct uftrace_filter *entry,
 	rb_insert_color(&new->node, root);
 }
 
-static void build_auto_args(const char *args_str, struct rb_root *root,
-			    unsigned long flag,
+static void build_auto_args(const char *args_str, struct rb_root *root, unsigned long flag,
 			    struct uftrace_filter_setting *setting)
 {
 	struct strv specs = STRV_INIT;
@@ -118,7 +117,7 @@ next:
 	strv_free(&specs);
 }
 
-static struct uftrace_filter * find_auto_args(struct rb_root *root, char *name)
+static struct uftrace_filter *find_auto_args(struct rb_root *root, char *name)
 {
 	struct rb_node *parent = NULL;
 	struct rb_node **p = &root->rb_node;
@@ -144,10 +143,9 @@ static struct uftrace_filter * find_auto_args(struct rb_root *root, char *name)
 
 static struct uftrace_filter *dwarf_argspec_list;
 
-static struct uftrace_filter * find_dwarf_argspec(struct uftrace_filter *filter,
-						  struct debug_info *dinfo,
-						  bool is_retval,
-						  struct uftrace_filter_setting *setting)
+static struct uftrace_filter *find_dwarf_argspec(struct uftrace_filter *filter,
+						 struct uftrace_dbg_info *dinfo, bool is_retval,
+						 struct uftrace_filter_setting *setting)
 {
 	LIST_HEAD(dwarf_argspec);
 	struct uftrace_filter *dwarf_filter;
@@ -187,10 +185,9 @@ static struct uftrace_filter * find_dwarf_argspec(struct uftrace_filter *filter,
 	return dwarf_filter;
 }
 
-struct uftrace_filter * find_auto_argspec(struct uftrace_filter *filter,
-					  struct uftrace_trigger *tr,
-					  struct debug_info *dinfo,
-					  struct uftrace_filter_setting *setting)
+struct uftrace_filter *find_auto_argspec(struct uftrace_filter *filter, struct uftrace_trigger *tr,
+					 struct uftrace_dbg_info *dinfo,
+					 struct uftrace_filter_setting *setting)
 {
 	struct uftrace_filter *auto_arg = NULL;
 
@@ -203,10 +200,9 @@ struct uftrace_filter * find_auto_argspec(struct uftrace_filter *filter,
 	return auto_arg;
 }
 
-struct uftrace_filter * find_auto_retspec(struct uftrace_filter *filter,
-					  struct uftrace_trigger *tr,
-					  struct debug_info *dinfo,
-					  struct uftrace_filter_setting *setting)
+struct uftrace_filter *find_auto_retspec(struct uftrace_filter *filter, struct uftrace_trigger *tr,
+					 struct uftrace_dbg_info *dinfo,
+					 struct uftrace_filter_setting *setting)
 {
 	struct uftrace_filter *auto_ret = NULL;
 
@@ -231,12 +227,9 @@ char *get_auto_retspec_str(void)
 
 void setup_auto_args(struct uftrace_filter_setting *setting)
 {
-
 	parse_enum_string(auto_enum_list, &auto_enum);
-	build_auto_args(auto_args_list, &auto_argspec, TRIGGER_FL_ARGUMENT,
-			setting);
-	build_auto_args(auto_retvals_list, &auto_retspec, TRIGGER_FL_RETVAL,
-			setting);
+	build_auto_args(auto_args_list, &auto_argspec, TRIGGER_FL_ARGUMENT, setting);
+	build_auto_args(auto_retvals_list, &auto_retspec, TRIGGER_FL_RETVAL, setting);
 }
 
 void setup_auto_args_str(char *args, char *rets, char *enums,
@@ -283,8 +276,8 @@ void finish_auto_args(void)
 		tmp = (void *)dwarf_argspec_list->name;
 
 		while (!list_empty(dwarf_argspec_list->trigger.pargs)) {
-			spec = list_first_entry(dwarf_argspec_list->trigger.pargs,
-						typeof(*spec), list);
+			spec = list_first_entry(dwarf_argspec_list->trigger.pargs, typeof(*spec),
+						list);
 			list_del(&spec->list);
 			free_arg_spec(spec);
 		}
@@ -333,8 +326,7 @@ int extract_trigger_args(char **pargs, char **prets, char *trigger)
 			*act++ = '\0';
 
 			while ((pos = strsep(&act, ",")) != NULL) {
-				if (!strncasecmp(pos, "arg", 3) ||
-				    !strncasecmp(pos, "fparg", 5))
+				if (!strncasecmp(pos, "arg", 3) || !strncasecmp(pos, "fparg", 5))
 					args = strjoin(args, pos, ",");
 				if (!strncasecmp(pos, "retval", 6))
 					rval = "retval";
@@ -488,7 +480,7 @@ static void add_enum_tree(struct rb_root *root, struct enum_def *e_def)
 	rb_insert_color(&e_def->node, root);
 }
 
-struct enum_def * find_enum_def(struct rb_root *root, char *name)
+struct enum_def *find_enum_def(struct rb_root *root, char *name)
 {
 	struct rb_node *parent = NULL;
 	struct rb_node **p = &root->rb_node;
@@ -511,7 +503,7 @@ struct enum_def * find_enum_def(struct rb_root *root, char *name)
 	return NULL;
 }
 
-char * convert_enum_val(struct enum_def *e_def, long val)
+char *convert_enum_val(struct enum_def *e_def, int val)
 {
 	struct enum_val *e_val;
 	char *str = NULL;
@@ -537,22 +529,22 @@ char * convert_enum_val(struct enum_def *e_def, long val)
 	if (str && val) {
 		char *tmp;
 
-		xasprintf(&tmp, "%s+%#lx", str, val);
+		xasprintf(&tmp, "%s+%#x", str, val);
 		free(str);
 		str = tmp;
 	}
 	else if (unlikely(str == NULL)) {
 		if (labs(val) > 100000)
-			xasprintf(&str, "%#lx", val);
+			xasprintf(&str, "%#x", val);
 		else
-			xasprintf(&str, "%ld", val);
+			xasprintf(&str, "%d", val);
 	}
 
 	return str;
 }
 
 /* caller should free the return value */
-char *get_enum_string(struct rb_root *root, char *name, long val)
+char *get_enum_string(struct rb_root *root, char *name, int val)
 {
 	struct enum_def *e_def;
 	char *ret;
@@ -562,7 +554,7 @@ char *get_enum_string(struct rb_root *root, char *name, long val)
 		e_def = find_enum_def(&auto_enum, name);
 
 	if (e_def == NULL)
-		xasprintf(&ret, "%ld", val);
+		xasprintf(&ret, "%d", val);
 	else
 		ret = convert_enum_val(e_def, val);
 
@@ -576,7 +568,7 @@ char *get_enum_string(struct rb_root *root, char *name, long val)
  * This function parses @enum_str and add it to @root so that it can be
  * used for argument/return later.  The syntax of enum is same as C
  * (except for the 'enum' keyword) but it only accepts a simple integer
- * contant or other enum constant in RHS.
+ * constant or other enum constant in RHS.
  *
  * For example, following string should be accepted:
  *
@@ -770,9 +762,10 @@ TEST_CASE(argspec_auto_args)
 	};
 	int idx = 1;
 
-	build_auto_args(test_auto_args, &auto_argspec, TRIGGER_FL_ARGUMENT,
-			&setting);
+	pr_dbg("build auto args from: %s\n", test_auto_args);
+	build_auto_args(test_auto_args, &auto_argspec, TRIGGER_FL_ARGUMENT, &setting);
 
+	pr_dbg("'foo' should have two arguments\n");
 	key.name = "foo";
 	entry = find_auto_argspec(&key, NULL, NULL, &setting);
 	TEST_NE(entry, NULL);
@@ -784,6 +777,7 @@ TEST_CASE(argspec_auto_args)
 		idx++;
 	}
 
+	pr_dbg("'foo' should have one FP argument\n");
 	key.name = "bar";
 	entry = find_auto_argspec(&key, NULL, NULL, &setting);
 	TEST_NE(entry, NULL);
@@ -793,12 +787,14 @@ TEST_CASE(argspec_auto_args)
 	TEST_EQ(spec->fmt, ARG_FMT_FLOAT);
 	TEST_EQ(spec->idx, 1);
 
+	pr_dbg("'xxx' should not have arguments\n");
 	key.name = "xxx";
 	entry = find_auto_argspec(&key, NULL, NULL, &setting);
 	TEST_EQ(entry, NULL);
 
 	release_auto_args(&auto_argspec);
 
+	pr_dbg("'foo' should not have arguments after released\n");
 	key.name = "foo";
 	entry = find_auto_argspec(&key, NULL, NULL, &setting);
 	TEST_EQ(entry, NULL);
@@ -813,6 +809,7 @@ TEST_CASE(argspec_extract)
 	char test_trigger_str3[] = "foo@libabc,arg3/i32%rax,backtrace";
 	char *args, *rets;
 
+	pr_dbg("extracting args/rets from %s\n", test_trigger_str1);
 	args = rets = NULL;
 	extract_trigger_args(&args, &rets, test_trigger_str1);
 
@@ -822,6 +819,7 @@ TEST_CASE(argspec_extract)
 	free(args);
 	free(rets);
 
+	pr_dbg("extracting args/rets from %s\n", test_trigger_str2);
 	args = rets = NULL;
 	extract_trigger_args(&args, &rets, test_trigger_str2);
 
@@ -831,6 +829,7 @@ TEST_CASE(argspec_extract)
 	free(args);
 	free(rets);
 
+	pr_dbg("extracting args/rets from %s\n", test_trigger_str3);
 	args = rets = NULL;
 	extract_trigger_args(&args, &rets, test_trigger_str3);
 
@@ -854,19 +853,22 @@ TEST_CASE(argspec_parse_enum)
 	struct enum_val *e_val, *e_next;
 	char *str;
 
+	pr_dbg("parse enum string: %s\n", test_enum_str1);
 	TEST_EQ(parse_enum_string(test_enum_str1, &enum_tree), 0);
+	pr_dbg("parse enum string: %s\n", test_enum_str2);
 	TEST_EQ(parse_enum_string(test_enum_str2, &enum_tree), 0);
+	pr_dbg("parse enum string: %s\n", test_enum_str3);
 	TEST_EQ(parse_enum_string(test_enum_str3, &enum_tree), 0);
 
 	node = rb_first(&enum_tree);
 	while (node) {
 		e_def = rb_entry(node, struct enum_def, node);
 
-		e_val  = list_first_entry(&e_def->vals, struct enum_val, list);
+		e_val = list_first_entry(&e_def->vals, struct enum_val, list);
 		e_next = list_next_entry(e_val, list);
 		TEST_GE(e_val->val, e_next->val);
 
-		e_val  = list_next_entry(e_val, list);
+		e_val = list_next_entry(e_val, list);
 		e_next = list_next_entry(e_next, list);
 		TEST_GE(e_val->val, e_next->val);
 
@@ -876,19 +878,23 @@ TEST_CASE(argspec_parse_enum)
 	e_def = find_enum_def(&enum_tree, "xxx");
 	TEST_NE(e_def, NULL);
 
+	pr_dbg("first enum item should have 0 value\n");
 	e_val = list_last_entry(&e_def->vals, struct enum_val, list);
 	TEST_STREQ(e_val->str, "ZERO");
 	TEST_EQ(e_val->val, 0L);
 
+	pr_dbg("check value increments from previous value\n");
 	e_val = list_first_entry(&e_def->vals, struct enum_val, list);
 	TEST_STREQ(e_val->str, "TWO");
 	TEST_EQ(e_val->val, 112L);
 
+	pr_dbg("check value match to multiple bitmask\n");
 	e_def = find_enum_def(&enum_tree, "a");
 	str = convert_enum_val(e_def, 3);
 	TEST_STREQ(str, "CCC|BBB");
 	free(str);
 
+	pr_dbg("check value increments correctly for negative numbers\n");
 	e_def = find_enum_def(&enum_tree, "uftrace");
 	str = convert_enum_val(e_def, -22);
 	TEST_STREQ(str, "report");
@@ -896,6 +902,7 @@ TEST_CASE(argspec_parse_enum)
 
 	release_enum_def(&enum_tree);
 
+	pr_dbg("after release, it should not find any definition\n");
 	TEST_EQ(find_enum_def(&enum_tree, "xxx"), NULL);
 
 	return TEST_OK;
