@@ -1,23 +1,22 @@
-#include <stdio.h>
-#include <stdint.h>
-#include <inttypes.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <assert.h>
 #include <byteswap.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <glob.h>
+#include <inttypes.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
+#include "libmcount/mcount.h"
 #include "uftrace.h"
-#include "utils/utils.h"
-#include "utils/fstack.h"
+#include "utils/event.h"
 #include "utils/filter.h"
-#include "utils/symbol.h"
+#include "utils/fstack.h"
 #include "utils/kernel.h"
 #include "utils/perf.h"
-#include "libmcount/mcount.h"
-
+#include "utils/symbol.h"
+#include "utils/utils.h"
 
 /**
  * read_task_file - read 'task' file from data directory
@@ -34,8 +33,8 @@
  *
  * It returns 0 for success, -1 for error.
  */
-int read_task_file(struct uftrace_session_link *sess, char *dirname,
-		   bool needs_symtab, bool sym_rel_addr, bool needs_srcline)
+int read_task_file(struct uftrace_session_link *sess, char *dirname, bool needs_symtab,
+		   bool sym_rel_addr, bool needs_srcline)
 {
 	int fd;
 	char pad[8];
@@ -61,13 +60,11 @@ int read_task_file(struct uftrace_session_link *sess, char *dirname,
 				goto out;
 			if (read_all(fd, buf, smsg.namelen) < 0)
 				goto out;
-			if (smsg.namelen % 8 &&
-			    read_all(fd, pad, 8 - (smsg.namelen % 8)) < 0)
+			if (smsg.namelen % 8 && read_all(fd, pad, 8 - (smsg.namelen % 8)) < 0)
 				goto out;
 
-			create_session(sess, &smsg, dirname, buf,
-				       sym_rel_addr, needs_symtab,
-				       needs_srcline);
+			create_session(sess, &smsg, dirname, dirname, buf, sym_rel_addr,
+				       needs_symtab, needs_srcline);
 			break;
 
 		case UFTRACE_MSG_TASK_START:
@@ -109,7 +106,7 @@ out:
  *
  * It returns 0 for success, -1 for error.
  */
-int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
+int read_task_txt_file(struct uftrace_session_link *sess, char *dirname, char *symdir,
 		       bool needs_symtab, bool sym_rel_addr, bool needs_srcline)
 {
 	FILE *fp;
@@ -135,8 +132,8 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 	pr_dbg("reading %s file\n", fname);
 	while (getline(&line, &sz, fp) >= 0) {
 		if (!strncmp(line, "TASK", 4)) {
-			num = sscanf(line + 5, "timestamp=%lu.%lu tid=%d pid=%d",
-				     &sec, &nsec, &tmsg.tid, &tmsg.pid);
+			num = sscanf(line + 5, "timestamp=%lu.%lu tid=%d pid=%d", &sec, &nsec,
+				     &tmsg.tid, &tmsg.pid);
 			if (num != 4)
 				goto out;
 
@@ -144,8 +141,8 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 			create_task(sess, &tmsg, false);
 		}
 		else if (!strncmp(line, "FORK", 4)) {
-			num = sscanf(line + 5, "timestamp=%lu.%lu pid=%d ppid=%d",
-				     &sec, &nsec, &tmsg.tid, &tmsg.pid);
+			num = sscanf(line + 5, "timestamp=%lu.%lu pid=%d ppid=%d", &sec, &nsec,
+				     &tmsg.tid, &tmsg.pid);
 			if (num != 4)
 				goto out;
 
@@ -153,8 +150,8 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 			create_task(sess, &tmsg, true);
 		}
 		else if (!strncmp(line, "SESS", 4)) {
-			num = sscanf(line + 5, "timestamp=%lu.%lu %*[^i]id=%d sid=%s",
-				     &sec, &nsec, &smsg.task.pid, (char *)&smsg.sid);
+			num = sscanf(line + 5, "timestamp=%lu.%lu %*[^i]id=%d sid=%s", &sec, &nsec,
+				     &smsg.task.pid, (char *)&smsg.sid);
 			if (num != 4)
 				goto out;
 
@@ -163,7 +160,7 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 			if (pos == NULL)
 				goto out;
 
-			exename = pos + 8 + 1;  // skip double-quote
+			exename = pos + 8 + 1; // skip double-quote
 			pos = strrchr(exename, '\"');
 			if (pos)
 				*pos = '\0';
@@ -172,9 +169,8 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 			smsg.task.time = (uint64_t)sec * NSEC_PER_SEC + nsec;
 			smsg.namelen = strlen(exename);
 
-			create_session(sess, &smsg, dirname, exename,
-				       sym_rel_addr, needs_symtab,
-				       needs_srcline);
+			create_session(sess, &smsg, dirname, symdir, exename, sym_rel_addr,
+				       needs_symtab, needs_srcline);
 		}
 		else if (!strncmp(line, "DLOP", 4)) {
 			struct uftrace_session *s;
@@ -182,7 +178,7 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 			if (!needs_symtab)
 				continue;
 
-			num = sscanf(line + 5, "timestamp=%lu.%lu tid=%d sid=%s base=%"PRIx64,
+			num = sscanf(line + 5, "timestamp=%lu.%lu tid=%d sid=%s base=%" PRIx64,
 				     &sec, &nsec, &dlop.task.tid, (char *)&dlop.sid,
 				     &dlop.base_addr);
 			if (num != 5)
@@ -192,7 +188,7 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 			if (pos == NULL)
 				goto out;
 
-			exename = pos + 8 + 1;  // skip double-quote
+			exename = pos + 8 + 1; // skip double-quote
 			pos = strrchr(exename, '\"');
 			if (pos)
 				*pos = '\0';
@@ -202,9 +198,8 @@ int read_task_txt_file(struct uftrace_session_link *sess, char *dirname,
 			dlop.namelen = strlen(exename);
 
 			s = get_session_from_sid(sess, dlop.sid);
-			assert(s);
-			session_add_dlopen(s, dlop.task.time,
-					   dlop.base_addr, exename);
+			ASSERT(s);
+			session_add_dlopen(s, dlop.task.time, dlop.base_addr, exename);
 		}
 	}
 	ret = 0;
@@ -220,63 +215,9 @@ out:
 	return ret;
 }
 
-/**
- * read_events_file - read 'events.txt' file from data directory
- * @dirname: name of the data directory
- *
- * This function read the events file in the @dirname and build event
- * information (for userspace).
- *
- * It returns 0 for success, -1 for error.
- */
-int read_events_file(struct uftrace_data *handle)
-{
-	FILE *fp;
-	char *fname = NULL;
-	char *line = NULL;
-	size_t sz = 0;
-
-	xasprintf(&fname, "%s/%s", handle->dirname, "events.txt");
-
-	fp = fopen(fname, "r");
-	if (fp == NULL) {
-		/* it might hit no events, so no file is ok */
-		if (errno == ENOENT)
-			errno = 0;
-
-		free(fname);
-		return -errno;
-	}
-
-	pr_dbg("reading %s file\n", fname);
-	while (getline(&line, &sz, fp) >= 0) {
-		char provider[512];
-		char event[512];
-		unsigned evt_id;
-		struct uftrace_event *ev;
-
-		if (!strncmp(line, "EVENT", 5)) {
-			sscanf(line + 7, "%u %[^:]:%s",
-			       &evt_id, provider, event);
-
-			ev = xmalloc(sizeof(*ev));
-			ev->id = evt_id;
-			ev->provider = xstrdup(provider);
-			ev->event = xstrdup(event);
-
-			list_add_tail(&ev->list, &handle->events);
-		}
-	}
-
-	free(line);
-	fclose(fp);
-	free(fname);
-	return 0;
-}
-
 static void snprint_timestamp(char *buf, size_t sz, uint64_t timestamp)
 {
-	snprintf(buf, sz, "%"PRIu64".%09"PRIu64,  // sec.nsec
+	snprintf(buf, sz, "%" PRIu64 ".%09" PRIu64, // sec.nsec
 		 timestamp / NSEC_PER_SEC, timestamp % NSEC_PER_SEC);
 }
 
@@ -293,8 +234,7 @@ void write_task_info(const char *dirname, struct uftrace_msg_task *tmsg)
 		pr_err("cannot open %s", fname);
 
 	snprint_timestamp(ts, sizeof(ts), tmsg->time);
-	fprintf(fp, "TASK timestamp=%s tid=%d pid=%d\n",
-		ts, tmsg->tid, tmsg->pid);
+	fprintf(fp, "TASK timestamp=%s tid=%d pid=%d\n", ts, tmsg->tid, tmsg->pid);
 
 	fclose(fp);
 	free(fname);
@@ -313,15 +253,13 @@ void write_fork_info(const char *dirname, struct uftrace_msg_task *tmsg)
 		pr_err("cannot open %s", fname);
 
 	snprint_timestamp(ts, sizeof(ts), tmsg->time);
-	fprintf(fp, "FORK timestamp=%s pid=%d ppid=%d\n",
-		ts, tmsg->tid, tmsg->pid);
+	fprintf(fp, "FORK timestamp=%s pid=%d ppid=%d\n", ts, tmsg->tid, tmsg->pid);
 
 	fclose(fp);
 	free(fname);
 }
 
-void write_session_info(const char *dirname, struct uftrace_msg_sess *smsg,
-			const char *exename)
+void write_session_info(const char *dirname, struct uftrace_msg_sess *smsg, const char *exename)
 {
 	FILE *fp;
 	char *fname = NULL;
@@ -334,15 +272,14 @@ void write_session_info(const char *dirname, struct uftrace_msg_sess *smsg,
 		pr_err("cannot open %s", fname);
 
 	snprint_timestamp(ts, sizeof(ts), smsg->task.time);
-	fprintf(fp, "SESS timestamp=%s pid=%d sid=%s exename=\"%s\"\n",
-		ts, smsg->task.pid, smsg->sid, exename);
+	fprintf(fp, "SESS timestamp=%s pid=%d sid=%.*s exename=\"%s\"\n", ts, smsg->task.pid,
+		SESSION_ID_LEN, smsg->sid, exename);
 
 	fclose(fp);
 	free(fname);
 }
 
-void write_dlopen_info(const char *dirname, struct uftrace_msg_dlopen *dmsg,
-		       const char *libname)
+void write_dlopen_info(const char *dirname, struct uftrace_msg_dlopen *dmsg, const char *libname)
 {
 	FILE *fp;
 	char *fname = NULL;
@@ -355,8 +292,8 @@ void write_dlopen_info(const char *dirname, struct uftrace_msg_dlopen *dmsg,
 		pr_err("cannot open %s", fname);
 
 	snprint_timestamp(ts, sizeof(ts), dmsg->task.time);
-	fprintf(fp, "DLOP timestamp=%s tid=%d sid=%s base=%"PRIx64" libname=\"%s\"\n",
-		ts, dmsg->task.tid, dmsg->sid, dmsg->base_addr, libname);
+	fprintf(fp, "DLOP timestamp=%s tid=%d sid=%.*s base=%" PRIx64 " libname=\"%s\"\n", ts,
+		dmsg->task.tid, SESSION_ID_LEN, dmsg->sid, dmsg->base_addr, libname);
 
 	fclose(fp);
 	free(fname);
@@ -381,8 +318,7 @@ static void check_data_order(struct uftrace_data *handle)
 		pr_dbg("bitfield order is different!\n");
 }
 
-static bool check_data_file(struct uftrace_data *handle,
-			    const char *pattern)
+static bool check_data_file(struct uftrace_data *handle, const char *pattern)
 {
 	glob_t g;
 	size_t i;
@@ -408,10 +344,10 @@ static bool check_data_file(struct uftrace_data *handle,
 
 bool data_is_lp64(struct uftrace_data *handle)
 {
-	return handle->hdr.class == ELFCLASS64;
+	return handle->hdr.elf_class == ELFCLASS64;
 }
 
-int open_info_file(struct opts *opts, struct uftrace_data *handle)
+int open_info_file(struct uftrace_opts *opts, struct uftrace_data *handle)
 {
 	FILE *fp;
 	char buf[PATH_MAX];
@@ -464,8 +400,9 @@ ok:
 	handle->dirname = opts->dirname;
 	handle->depth = opts->depth;
 	handle->time_filter = opts->threshold;
+	handle->size_filter = opts->size_filter;
 	handle->time_range = opts->range;
-	handle->sessions.root  = RB_ROOT;
+	handle->sessions.root = RB_ROOT;
 	handle->sessions.tasks = RB_ROOT;
 	handle->last_perf_idx = -1;
 	INIT_LIST_HEAD(&handle->events);
@@ -479,7 +416,7 @@ ok:
 	check_data_order(handle);
 
 	if (handle->needs_byte_swap) {
-		handle->hdr.version   = bswap_32(handle->hdr.version);
+		handle->hdr.version = bswap_32(handle->hdr.version);
 		handle->hdr.feat_mask = bswap_64(handle->hdr.feat_mask);
 		handle->hdr.info_mask = bswap_64(handle->hdr.info_mask);
 		handle->hdr.max_stack = bswap_16(handle->hdr.max_stack);
@@ -499,7 +436,7 @@ ok:
 	return 0;
 }
 
-int open_data_file(struct opts *opts, struct uftrace_data *handle)
+int open_data_file(struct uftrace_opts *opts, struct uftrace_data *handle)
 {
 	int ret;
 	char buf[PATH_MAX];
@@ -525,10 +462,10 @@ int open_data_file(struct opts *opts, struct uftrace_data *handle)
 			sym_rel = true;
 
 		/* read old task file first and then try task.txt file */
-		if (read_task_file(sessions, opts->dirname, true, sym_rel,
-				   opts->srcline) < 0 &&
-		    read_task_txt_file(sessions, opts->dirname, true, sym_rel,
-				       opts->srcline) < 0) {
+		if (read_task_file(sessions, opts->dirname, true, sym_rel, opts->srcline) < 0 &&
+		    read_task_txt_file(sessions, opts->dirname, opts->with_syms ?: opts->dirname,
+				       true, sym_rel,
+				       opts->srcline | (opts->loc_filter != NULL)) < 0) {
 			if (errno == ENOENT)
 				saved_errno = ENODATA;
 			else
@@ -557,22 +494,19 @@ int open_data_file(struct opts *opts, struct uftrace_data *handle)
 
 	if (handle->hdr.info_mask & ARG_SPEC) {
 		struct uftrace_filter_setting setting = {
-			.ptype		= handle->info.patt_type,
-			.allow_kernel	= true,
-			.auto_args	= false,
-			.lp64		= data_is_lp64(handle),
-			.arch		= handle->arch,
+			.ptype = handle->info.patt_type,
+			.allow_kernel = true,
+			.auto_args = false,
+			.lp64 = data_is_lp64(handle),
+			.arch = handle->arch,
 		};
 
 		if (handle->hdr.feat_mask & AUTO_ARGS) {
-			setup_auto_args_str(handle->info.autoarg,
-					    handle->info.autoret,
-					    handle->info.autoenum,
-					    &setting);
+			setup_auto_args_str(handle->info.autoarg, handle->info.autoret,
+					    handle->info.autoenum, &setting);
 		}
 
-		setup_fstack_args(handle->info.argspec, handle->info.retspec,
-				  handle, &setting);
+		setup_fstack_args(handle->info.argspec, handle->info.retspec, handle, &setting);
 
 		if (handle->info.auto_args_enabled) {
 			char *autoarg = handle->info.autoarg;
@@ -581,7 +515,7 @@ int open_data_file(struct opts *opts, struct uftrace_data *handle)
 			if (handle->hdr.feat_mask & DEBUG_INFO) {
 				if (handle->info.patt_type == PATT_REGEX)
 					autoarg = autoret = ".";
-				else  /* PATT_GLOB */
+				else /* PATT_GLOB */
 					autoarg = autoret = "*";
 			}
 
@@ -598,8 +532,8 @@ int open_data_file(struct opts *opts, struct uftrace_data *handle)
 
 		kernel = xzalloc(sizeof(*kernel));
 
-		kernel->handle   = handle;
-		kernel->dirname  = opts->dirname;
+		kernel->handle = handle;
+		kernel->dirname = opts->dirname;
 		kernel->skip_out = opts->kernel_skip_out;
 
 		if (setup_kernel_data(kernel) == 0) {
@@ -624,8 +558,7 @@ int open_data_file(struct opts *opts, struct uftrace_data *handle)
 	snprintf(buf, sizeof(buf), "%s/[0-9]*.dat", opts->dirname);
 	if (!check_data_file(handle, buf)) {
 		if (handle->kernel) {
-			snprintf(buf, sizeof(buf), "%s/kernel-*.dat",
-				 opts->dirname);
+			snprintf(buf, sizeof(buf), "%s/kernel-*.dat", opts->dirname);
 
 			if (check_data_file(handle, buf))
 				goto out;
@@ -647,7 +580,7 @@ out:
 	return ret;
 }
 
-void close_data_file(struct opts *opts, struct uftrace_data *handle)
+void __close_data_file(struct uftrace_opts *opts, struct uftrace_data *handle, bool unload_modules)
 {
 	if (opts->exename == handle->info.exename)
 		opts->exename = NULL;
@@ -657,6 +590,8 @@ void close_data_file(struct opts *opts, struct uftrace_data *handle)
 		free(handle->kernel);
 	}
 
+	finish_events_file(handle);
+
 	if (has_perf_data(handle))
 		finish_perf_data(handle);
 
@@ -664,7 +599,9 @@ void close_data_file(struct opts *opts, struct uftrace_data *handle)
 		finish_extern_data(handle);
 
 	delete_sessions(&handle->sessions);
-	unload_module_symtabs();
+
+	if (unload_modules)
+		unload_module_symtabs();
 
 	if (handle->hdr.feat_mask & AUTO_ARGS)
 		finish_auto_args();
@@ -672,3 +609,115 @@ void close_data_file(struct opts *opts, struct uftrace_data *handle)
 	clear_uftrace_info(&handle->info);
 	reset_task_handle(handle);
 }
+
+#ifdef UNIT_TEST
+#define TEST_SESSION_ID "test-session-id"
+#define TEST_EXIT_STATUS 37
+#define TEST_INFO_MASK ((1U << EXE_NAME) | (1U << EXIT_STATUS))
+
+static int create_data(struct uftrace_opts *opts)
+{
+	struct uftrace_msg_sess sess_msg = {
+		.task = { .time = 100, .pid = 10 },
+		.sid = TEST_SESSION_ID,
+	};
+	struct uftrace_msg_task task_msg = {
+		.time = 200,
+		.pid = 10,
+		.tid = 10,
+	};
+	struct uftrace_msg_task fork_msg = {
+		.time = 300,
+		.pid = 10,
+		.tid = 20,
+	};
+	struct uftrace_msg_dlopen dlopen_msg = {
+		.task = { .time = 400, .tid = 20 },
+		.sid = TEST_SESSION_ID,
+		.base_addr = 0x123000,
+	};
+	char *filename = NULL;
+	FILE *fp;
+
+	if (create_directory(opts->dirname) < 0)
+		return -1;
+
+	write_session_info(opts->dirname, &sess_msg, opts->exename);
+	write_task_info(opts->dirname, &task_msg);
+	write_fork_info(opts->dirname, &fork_msg);
+	write_dlopen_info(opts->dirname, &dlopen_msg, "libnothing.so");
+
+	/* create_session() requires a map file even if it's empty */
+	TEST_GE(asprintf(&filename, "%s/sid-%.*s.map", opts->dirname, SESSION_ID_LEN,
+			 TEST_SESSION_ID),
+		0);
+	creat(filename, 0644);
+	free(filename);
+
+	/* open_data_file() requires at least one non-empty data file */
+	TEST_GE(asprintf(&filename, "%s/%d.dat", opts->dirname, task_msg.pid), 0);
+	fp = fopen(filename, "w");
+	fprintf(fp, "%s\n", "empty file");
+	fclose(fp);
+	free(filename);
+
+	fill_file_header(opts, TEST_EXIT_STATUS, NULL, (char *)"1ms");
+
+	return 0;
+}
+
+static int remove_data(struct uftrace_opts *opts)
+{
+	return remove_directory(opts->dirname);
+}
+
+TEST_CASE(data_basic)
+{
+	struct uftrace_opts opts = {
+		.dirname = "data-test",
+		.exename = read_exename(),
+		.max_stack = 10,
+	};
+	struct uftrace_data handle;
+	struct uftrace_session *s;
+	struct uftrace_task *t;
+
+	pr_dbg("create test data\n");
+	if (create_data(&opts) < 0)
+		return TEST_SKIP;
+
+	open_data_file(&opts, &handle);
+
+	pr_dbg("verify header info\n");
+	TEST_NE(handle.hdr.feat_mask & TASK_SESSION, 0);
+	TEST_EQ(handle.hdr.info_mask & TEST_INFO_MASK, TEST_INFO_MASK);
+	TEST_EQ(handle.info.exit_status, TEST_EXIT_STATUS);
+	TEST_STREQ(handle.info.exename, opts.exename);
+
+	pr_dbg("verify session info\n");
+	TEST_NE(handle.sessions.first, NULL);
+	s = handle.sessions.first;
+
+	TEST_EQ(s->start_time, 100);
+	TEST_EQ(s->pid, 10);
+	TEST_STREQ(s->sid, TEST_SESSION_ID);
+	TEST_STREQ(s->exename, opts.exename);
+	TEST_EQ(list_empty(&s->dlopen_libs), false);
+
+	pr_dbg("verify task info\n");
+	TEST_NE(handle.sessions.first_task, NULL);
+	t = handle.sessions.first_task;
+
+	TEST_EQ(t->tid, 10);
+	TEST_EQ(t->tid, t->pid);
+	TEST_STREQ(t->comm, basename(opts.exename));
+
+	close_data_file(&opts, &handle);
+
+	pr_dbg("delete test data\n");
+	if (remove_data(&opts) < 0)
+		pr_dbg("failed to remove data\n");
+
+	return TEST_OK;
+}
+#endif /* UNIT_TEST */
