@@ -38,19 +38,24 @@
  */
 
 #include <stdlib.h>
-#include <assert.h>
 
 /* This should be defined before #include "utils.h" */
-#define PR_FMT     "demangle"
-#define PR_DOMAIN  DBG_DEMANGLE
+#define PR_FMT "demangle"
+#define PR_DOMAIN DBG_DEMANGLE
 
-#include "utils/utils.h"
 #include "utils/symbol.h"
+#include "utils/utils.h"
 
-#define MAX_DEBUG_DEPTH  128
+#define MAX_DEBUG_DEPTH 128
 
 enum symbol_demangler demangler = DEMANGLE_SIMPLE;
 
+struct demangle_debug {
+	const char *func;
+	int level;
+	int pos;
+};
+
 struct demangle_data {
 	char *old;
 	char *new;
@@ -68,7 +73,7 @@ struct demangle_data {
 	bool type_info;
 	bool first_name;
 	bool ignore_disc;
-	const char *debug[MAX_DEBUG_DEPTH];
+	struct demangle_debug debug[MAX_DEBUG_DEPTH];
 };
 
 static char dd_expbuf[2];
@@ -90,10 +95,15 @@ static char dd_curr(struct demangle_data *dd)
 	return dd_peek(dd, 0);
 }
 
-static void __dd_add_debug(struct demangle_data *dd, const char *dbg)
+static void __dd_add_debug(struct demangle_data *dd, const char *func)
 {
-	if (dd->nr_dbg < MAX_DEBUG_DEPTH && dbg)
-		dd->debug[dd->nr_dbg++] = dbg;
+	if (dd->nr_dbg < MAX_DEBUG_DEPTH && func) {
+		struct demangle_debug *dbg = &dd->debug[dd->nr_dbg++];
+
+		dbg->func = func;
+		dbg->level = dd->level;
+		dbg->pos = dd->pos;
+	}
 }
 
 static char __dd_consume_n(struct demangle_data *dd, int n, const char *dbg)
@@ -115,41 +125,46 @@ static char __dd_consume(struct demangle_data *dd, const char *dbg)
 	return __dd_consume_n(dd, 1, dbg);
 }
 
-#define dd_consume(dd)       __dd_consume(dd, __func__)
-#define dd_consume_n(dd, n)  __dd_consume_n(dd, n, __func__)
-#define dd_add_debug(dd)     __dd_add_debug(dd, __func__)
+#define dd_consume(dd) __dd_consume(dd, __func__)
+#define dd_consume_n(dd, n) __dd_consume_n(dd, n, __func__)
+#define dd_add_debug(dd) __dd_add_debug(dd, __func__)
 
-#define DD_DEBUG(dd, exp, inc)						\
-({	dd->func = __func__; dd->line = __LINE__ - 1; dd->pos += inc;	\
-	dd->expected = exp;						\
-	return -1;							\
-})
+#define DD_DEBUG(dd, exp, inc)                                                                     \
+	({                                                                                         \
+		dd->func = __func__;                                                               \
+		dd->line = __LINE__ - 1;                                                           \
+		dd->pos += inc;                                                                    \
+		dd->expected = exp;                                                                \
+		return -1;                                                                         \
+	})
 
-#define DD_DEBUG_CONSUME(dd, exp_c)					\
-({	if (dd_consume(dd) != exp_c) {					\
-		if (!dd->expected) {					\
-			dd->func = __func__;				\
-			dd->line = __LINE__;				\
-			dd->pos--;					\
-			dd->expected = dd_expbuf;			\
-			dd_expbuf[0] = exp_c;				\
-		}							\
-		return -1;						\
-	}								\
-})
+#define DD_DEBUG_CONSUME(dd, exp_c)                                                                \
+	({                                                                                         \
+		if (dd_consume(dd) != exp_c) {                                                     \
+			if (!dd->expected) {                                                       \
+				dd->func = __func__;                                               \
+				dd->line = __LINE__;                                               \
+				dd->pos--;                                                         \
+				dd->expected = dd_expbuf;                                          \
+				dd_expbuf[0] = exp_c;                                              \
+			}                                                                          \
+			return -1;                                                                 \
+		}                                                                                  \
+	})
 
-#define __DD_DEBUG_CONSUME(dd, exp_c)					\
-({	if (__dd_consume(dd, NULL) != exp_c) {				\
-		if (!dd->expected) {					\
-			dd->func = __func__;				\
-			dd->line = __LINE__;				\
-			dd->pos--;					\
-			dd->expected = dd_expbuf;			\
-			dd_expbuf[0] = exp_c;				\
-		}							\
-		return -1;						\
-	}								\
-})
+#define __DD_DEBUG_CONSUME(dd, exp_c)                                                              \
+	({                                                                                         \
+		if (__dd_consume(dd, NULL) != exp_c) {                                             \
+			if (!dd->expected) {                                                       \
+				dd->func = __func__;                                               \
+				dd->line = __LINE__;                                               \
+				dd->pos--;                                                         \
+				dd->expected = dd_expbuf;                                          \
+				dd_expbuf[0] = exp_c;                                              \
+			}                                                                          \
+			return -1;                                                                 \
+		}                                                                                  \
+	})
 
 static void dd_debug_print(struct demangle_data *dd)
 {
@@ -166,99 +181,65 @@ static void dd_debug_print(struct demangle_data *dd)
 	if (dd->func == NULL)
 		dd->func = "demangle";
 
-	if (dbg_domain[DBG_DEMANGLE] <= 1) {
-		pr_dbg("demangle failed: %s\n", dd->old);
+	if (dbg_domain[DBG_DEMANGLE] <= 3) {
+		pr_dbg3("demangle failed: %s\n", dd->old);
 		return;
 	}
 
-	pr_dbg2("simple demangle failed:%s%s\n%s\n%*c\n%s:%d: \"%s\" expected\n",
-		dd_eof(dd) ? " (EOF)" : "", dd->level ? " (not finished)" : "",
-		dd->old, dd->pos + 1, '^', dd->func, dd->line, expected);
+	pr_dbg4("simple demangle failed:%s%s\n%s\n%*c\n%s:%d: \"%s\" expected\n",
+		dd_eof(dd) ? " (EOF)" : "", dd->level ? " (not finished)" : "", dd->old,
+		dd->pos + 1, '^', dd->func, dd->line, expected);
 
-	pr_dbg3("current: %s (pos: %d/%d)\n", dd->new, dd->pos, dd->len);
-	for (i = 0; i < dd->nr_dbg; i++)
-		pr_dbg4("  [%d] %s\n", i, dd->debug[i]);
+	pr_dbg4("current: %s (pos: %d/%d)\n", dd->new, dd->pos, dd->len);
+	for (i = 0; i < dd->nr_dbg; i++) {
+		struct demangle_debug *dbg = &dd->debug[i];
+
+		pr_dbg4("  [%02d] (%03d/%c%c) %*s%s\n", i, dbg->pos,
+			dbg->pos < dd->len ? dd->old[dbg->pos] : ' ',
+			dbg->pos + 1 < dd->len ? dd->old[dbg->pos + 1] : ' ', dbg->level * 2, "",
+			dbg->func);
+	}
 }
 
 static const struct {
 	char op[2];
 	char *name;
 } ops[] = {
-	{ { 'n','w' }, " new" },
-	{ { 'n','a' }, " new[]" },
-	{ { 'd','l' }, " delete" },
-	{ { 'd','a' }, " delete[]" },
-	{ { 'p','s' }, "+" }, /* unary */
-	{ { 'n','g' }, "-" }, /* unary */
-	{ { 'a','d' }, "&" }, /* unary */
-	{ { 'd','e' }, "*" }, /* unary */
-	{ { 'c','o' }, "~" },
-	{ { 'p','l' }, "+" },
-	{ { 'm','i' }, "-" },
-	{ { 'm','l' }, "*" },
-	{ { 'd','v' }, "/" },
-	{ { 'r','m' }, "%" },
-	{ { 'a','n' }, "&" },
-	{ { 'o','r' }, "|" },
-	{ { 'e','o' }, "^" },
-	{ { 'a','S' }, "=" },
-	{ { 'p','L' }, "+=" },
-	{ { 'm','I' }, "-=" },
-	{ { 'm','L' }, "*=" },
-	{ { 'd','V' }, "/=" },
-	{ { 'r','M' }, "%=" },
-	{ { 'a','N' }, "&=" },
-	{ { 'o','R' }, "|=" },
-	{ { 'e','O' }, "^=" },
-	{ { 'l','s' }, "<<" },
-	{ { 'r','s' }, ">>" },
-	{ { 'l','S' }, "<<=" },
-	{ { 'r','S' }, ">>=" },
-	{ { 'e','q' }, "==" },
-	{ { 'n','e' }, "!=" },
-	{ { 'l','t' }, "<" },
-	{ { 'g','t' }, ">" },
-	{ { 'l','e' }, "<=" },
-	{ { 'g','e' }, ">=" },
-	{ { 'n','t' }, "!" },
-	{ { 'a','a' }, "&&" },
-	{ { 'o','o' }, "||" },
-	{ { 'p','p' }, "++" },
-	{ { 'm','m' }, "--" },
-	{ { 'c','m' }, "," },
-	{ { 'p','m' }, "->*" },
-	{ { 'p','t' }, "->" },
-	{ { 'c','l' }, "()" },
-	{ { 'i','x' }, "[]" },
-	{ { 'q','u' }, "?" },
-	{ { 'c','v' }, "(cast)" },
-	{ { 'l','i' }, "\"\"" },
+	{ { 'n', 'w' }, " new" },      { { 'n', 'a' }, " new[]" }, { { 'd', 'l' }, " delete" },
+	{ { 'd', 'a' }, " delete[]" }, { { 'p', 's' }, "+" }, /* unary */
+	{ { 'n', 'g' }, "-" }, /* unary */
+	{ { 'a', 'd' }, "&" }, /* unary */
+	{ { 'd', 'e' }, "*" }, /* unary */
+	{ { 'c', 'o' }, "~" },	       { { 'p', 'l' }, "+" },	   { { 'm', 'i' }, "-" },
+	{ { 'm', 'l' }, "*" },	       { { 'd', 'v' }, "/" },	   { { 'r', 'm' }, "%" },
+	{ { 'a', 'n' }, "&" },	       { { 'o', 'r' }, "|" },	   { { 'e', 'o' }, "^" },
+	{ { 'a', 'S' }, "=" },	       { { 'p', 'L' }, "+=" },	   { { 'm', 'I' }, "-=" },
+	{ { 'm', 'L' }, "*=" },	       { { 'd', 'V' }, "/=" },	   { { 'r', 'M' }, "%=" },
+	{ { 'a', 'N' }, "&=" },	       { { 'o', 'R' }, "|=" },	   { { 'e', 'O' }, "^=" },
+	{ { 'l', 's' }, "<<" },	       { { 'r', 's' }, ">>" },	   { { 'l', 'S' }, "<<=" },
+	{ { 'r', 'S' }, ">>=" },       { { 'e', 'q' }, "==" },	   { { 'n', 'e' }, "!=" },
+	{ { 'l', 't' }, "<" },	       { { 'g', 't' }, ">" },	   { { 'l', 'e' }, "<=" },
+	{ { 'g', 'e' }, ">=" },	       { { 'n', 't' }, "!" },	   { { 'a', 'a' }, "&&" },
+	{ { 'o', 'o' }, "||" },	       { { 'p', 'p' }, "++" },	   { { 'm', 'm' }, "--" },
+	{ { 'c', 'm' }, "," },	       { { 'p', 'm' }, "->*" },	   { { 'p', 't' }, "->" },
+	{ { 'c', 'l' }, "()" },	       { { 'i', 'x' }, "[]" },	   { { 'q', 'u' }, "?" },
+	{ { 'c', 'v' }, "(cast)" },    { { 'l', 'i' }, "\"\"" },
 };
 
 static const struct {
 	char code;
 	char *name;
 } types[] = {
-	{ 'v', "void" },
-	{ 'w', "wchar_t" },
-	{ 'b', "bool" },
-	{ 'c', "char" },
-	{ 'a', "signed char" },
-	{ 'h', "unsigned char" },
-	{ 's', "short" },
-	{ 't', "unsigned short" },
-	{ 'i', "int" },
-	{ 'j', "unsigned int" },
-	{ 'l', "long" },
-	{ 'm', "unsigned long" },
-	{ 'x', "long long" },
-	{ 'y', "unsigned long long" },
-	{ 'n', "__int128" },
-	{ 'o', "unsigned __int128" },
-	{ 'f', "float" },
-	{ 'd', "double" },
-	{ 'e', "long double" },
-	{ 'g', "__float128" },
+	{ 'v', "void" },	{ 'w', "wchar_t" },
+	{ 'b', "bool" },	{ 'c', "char" },
+	{ 'a', "signed char" }, { 'h', "unsigned char" },
+	{ 's', "short" },	{ 't', "unsigned short" },
+	{ 'i', "int" },		{ 'j', "unsigned int" },
+	{ 'l', "long" },	{ 'm', "unsigned long" },
+	{ 'x', "long long" },	{ 'y', "unsigned long long" },
+	{ 'n', "__int128" },	{ 'o', "unsigned __int128" },
+	{ 'f', "float" },	{ 'd', "double" },
+	{ 'e', "long double" }, { 'g', "__float128" },
 	{ 'z', "..." },
 };
 
@@ -276,7 +257,7 @@ static const struct {
 };
 
 static const struct {
-	char *code;  /* surrounded by $..$ */
+	char *code; /* surrounded by $..$ */
 	char *punc;
 } rust_mappings[] = {
 	{ "SP", "@" },
@@ -286,7 +267,7 @@ static const struct {
 	{ "GT", ">" },
 	{ "LP", "(" },
 	{ "RP", ")" },
-	{ "C",  "," },
+	{ "C", "," },
 	/* some selected unicode characters */
 	{ "u20", " " },
 	{ "u22", "\"" },
@@ -307,6 +288,7 @@ static int dd_local_name(struct demangle_data *dd);
 static int dd_source_name(struct demangle_data *dd);
 static int dd_operator_name(struct demangle_data *dd);
 static int dd_nested_name(struct demangle_data *dd);
+static int dd_unqualified_name(struct demangle_data *dd);
 static int dd_type(struct demangle_data *dd);
 static int dd_decltype(struct demangle_data *dd);
 static int dd_expression(struct demangle_data *dd);
@@ -319,7 +301,8 @@ static int dd_append_len(struct demangle_data *dd, char *str, int size)
 		dd->new = xrealloc(dd->new, dd->alloc);
 	}
 
-	strncpy(&dd->new[dd->newpos], str, size);
+	/* copy including the last NUL byte (but usually not) */
+	strncpy(&dd->new[dd->newpos], str, size + 1);
 	dd->newpos += size;
 	dd->new[dd->newpos] = '\0';
 
@@ -372,8 +355,10 @@ static int dd_seq_id(struct demangle_data *dd)
 		return -1;
 
 	/* just skip for now */
-	while (isdigit(c) || isupper(c))
+	while (isdigit(c) || isupper(c)) {
+		dd_add_debug(dd);
 		c = dd->old[++dd->pos];
+	}
 	return 0;
 }
 
@@ -614,6 +599,8 @@ static int dd_unresolved_type(struct demangle_data *dd)
 			return -1;
 		if (dd_curr(dd) == 'I')
 			return dd_template_args(dd);
+		if (isdigit(dd_curr(dd)))
+			return dd_unqualified_name(dd);
 		return 0;
 	}
 	return -1;
@@ -670,32 +657,38 @@ static int dd_unresolved_name(struct demangle_data *dd)
 
 	if (c0 == 's' && c1 == 'r') {
 		dd_consume_n(dd, 2);
-		if (dd_curr(dd) == 'N')
-			__dd_consume(dd, NULL);
 
 		c0 = dd_curr(dd);
 		if (c0 == 'T' || c0 == 'D' || c0 == 'S') {
-			if (dd_unresolved_type(dd) < 0)
+			if (dd_type(dd) < 0)
+				return -1;
+
+			if (dd_base_unresolved_name(dd) < 0)
+				return -1;
+
+			if (dd_curr(dd) == 'I')
+				dd_template_args(dd);
+
+			return 0;
+		}
+
+		if (c0 == 'N') {
+			__dd_consume(dd, NULL);
+			if (dd_type(dd) < 0)
 				return -1;
 		}
 
 		c0 = dd_curr(dd);
-		c1 = dd_peek(dd, 1);
-
-		while (isdigit(c0)) {
+		while (c0 != 'E') {
 			if (dd_simple_id(dd) < 0)
-				return -1;
+				return 0;
 			c0 = dd_curr(dd);
-			c1 = dd_peek(dd, 1);
 		}
 
-		if (c0 == 'E') {
-			dd_consume(dd);
-			return dd_base_unresolved_name(dd);
-		}
-		if ((c0 == 'o' || c0 == 'd') && c1 == 'n')
-			return dd_base_unresolved_name(dd);
-		return 0;
+		if (c0 != 'E')
+			pr_dbg("no E\n");
+
+		__DD_DEBUG_CONSUME(dd, 'E');
 	}
 
 	return dd_base_unresolved_name(dd);
@@ -763,8 +756,8 @@ static int dd_expression(struct demangle_data *dd)
 	char c1 = dd_peek(dd, 1);
 	char *exp = &dd->old[dd->pos];
 	char *unary_ops[] = {
-		"ps", "ng", "ad", "de", "pp_", "mm_", "pp", "mm", "dl", "da",
-		"te", "sz", "az", "nx", "sp", "tw", "nt",
+		"ps", "ng", "ad", "de", "pp_", "mm_", "pp", "mm", "dl",
+		"da", "te", "sz", "az", "nx",  "sp",  "tw", "nt",
 	};
 
 	if (dd_eof(dd))
@@ -965,9 +958,9 @@ static int dd_ptr_to_member_type(struct demangle_data *dd)
 
 	DD_DEBUG_CONSUME(dd, 'M');
 
-	if (dd_type(dd) < 0)  /* class */
+	if (dd_type(dd) < 0) /* class */
 		return -1;
-	return dd_type(dd);   /* member */
+	return dd_type(dd); /* member */
 }
 
 static int dd_decltype(struct demangle_data *dd)
@@ -1025,7 +1018,7 @@ static int dd_type(struct demangle_data *dd)
 	unsigned i;
 	char cv_qual[] = "rVK";
 	char prefix[] = "PROCG";
-	char D_types[] = "defhisacn";
+	char D_types[] = "defhisacnu";
 	char scue[] = "sue"; /* struct, class, union, enum */
 	int done = 0;
 	int ret = -1;
@@ -1035,6 +1028,8 @@ static int dd_type(struct demangle_data *dd)
 
 	/* ignore type names */
 	dd->type++;
+	dd_add_debug(dd);
+	dd->level++;
 
 	while (!done && !dd_eof(dd)) {
 		char c = dd_curr(dd);
@@ -1093,7 +1088,10 @@ static int dd_type(struct demangle_data *dd)
 			done = 1;
 		}
 		else if (c == 'S') {
+			c = dd_peek(dd, 1);
 			ret = dd_substitution(dd);
+			if (!ret && c == 't' && isdigit(dd_curr(dd)))
+				ret = dd_unqualified_name(dd);
 			if (dd_curr(dd) == 'I')
 				ret = dd_template_args(dd);
 			done = 1;
@@ -1130,7 +1128,7 @@ static int dd_type(struct demangle_data *dd)
 			/* builtin types */
 			for (i = 0; i < ARRAY_SIZE(types); i++) {
 				if (c == types[i].code) {
-					dd_consume(dd);
+					__dd_consume(dd, NULL);
 					ret = 0;
 					break;
 				}
@@ -1139,6 +1137,7 @@ static int dd_type(struct demangle_data *dd)
 		}
 	}
 
+	dd->level--;
 	dd->type--;
 	return ret;
 }
@@ -1170,8 +1169,8 @@ static int dd_special_name(struct demangle_data *dd)
 	char c0 = dd_curr(dd);
 	char c1 = dd_peek(dd, 1);
 	char T_type[] = "VTISFJ";
-	char *T_type_name[] = { "vtable", "VTT", "typeinfo_name", "typeinfo",
-				"typeinfo_fn", "java_class" };
+	char *T_type_name[] = { "vtable",   "VTT",	   "typeinfo_name",
+				"typeinfo", "typeinfo_fn", "java_class" };
 
 	if (dd_eof(dd))
 		return -1;
@@ -1218,6 +1217,8 @@ static int dd_special_name(struct demangle_data *dd)
 
 			if (dd_number(dd) < 0)
 				return -1;
+			if (dd_eof(dd))
+				return 0;
 			__DD_DEBUG_CONSUME(dd, '_');
 
 			/*
@@ -1367,7 +1368,7 @@ static int dd_operator_name(struct demangle_data *dd)
 		}
 	}
 	if (c0 == 'v' && isdigit(c1)) {
-		/* vender extended operator */
+		/* vendor extended operator */
 		dd->type++;
 		dd_source_name(dd);
 		dd->type--;
@@ -1383,6 +1384,7 @@ static int dd_source_name(struct demangle_data *dd)
 	char *dollar;
 	char *p, *end;
 	unsigned i;
+	bool add_name = false;
 
 	if (num < 0)
 		return -1;
@@ -1390,6 +1392,8 @@ static int dd_source_name(struct demangle_data *dd)
 	if (dd_eof(dd) || dd->pos + num > dd->len)
 		DD_DEBUG(dd, "shorter name", 0);
 
+	dd_add_debug(dd);
+
 	if (dd->type && !dd->type_info)
 		goto out;
 
@@ -1407,16 +1411,17 @@ static int dd_source_name(struct demangle_data *dd)
 			goto out;
 	}
 
+	add_name = true;
 	dd_append_separator(dd, "::");
 
 	p = dd->old + dd->pos;
 	dollar = strchr(p, '$');
 	if (dollar == NULL)
-		goto out_append;
+		goto out;
 
 	end = p + num;
 	if (dollar > end)
-		goto out_append;
+		goto out;
 
 	/* check special symbol mappings (e.g. '$LT$') for Rust */
 	while (dollar != NULL && dollar < end) {
@@ -1426,10 +1431,11 @@ static int dd_source_name(struct demangle_data *dd)
 		dd_append_len(dd, p, num);
 
 		for (i = 0; i < ARRAY_SIZE(rust_mappings); i++) {
-			if (strncmp(rust_mappings[i].code, dollar+1,
+			if (strncmp(rust_mappings[i].code, dollar + 1,
 				    strlen(rust_mappings[i].code)))
 				continue;
 
+			dd_add_debug(dd);
 			dd_append(dd, rust_mappings[i].punc);
 			num += strlen(rust_mappings[i].code) + 2;
 			__dd_consume_n(dd, num, NULL);
@@ -1447,11 +1453,10 @@ static int dd_source_name(struct demangle_data *dd)
 	}
 	num = end - p;
 
-out_append:
-	dd_append_len(dd, p, num);
 out:
+	if (add_name)
+		dd_append_len(dd, p, num);
 	__dd_consume_n(dd, num, NULL);
-	__dd_add_debug(dd, __func__);
 	return 0;
 }
 
@@ -1514,7 +1519,7 @@ static int dd_unqualified_name(struct demangle_data *dd)
 		ret = dd_operator_name(dd);
 	else {
 		if (c0 == 'L')
-			dd_consume(dd);  /* local-source-name ? */
+			dd_consume(dd); /* local-source-name ? */
 		ret = dd_source_name(dd);
 	}
 
@@ -1552,9 +1557,9 @@ static int dd_nested_name(struct demangle_data *dd)
 		else if (c0 == 'S')
 			ret = dd_substitution(dd);
 		else if (c0 == 'M')
-			dd_consume(dd);  /* assumed data-member-prefix */
+			dd_consume(dd); /* assumed data-member-prefix */
 		else if (c0 == 'L')
-			dd_consume(dd);  /* local-source-name ? */
+			dd_consume(dd); /* local-source-name ? */
 		else if (strchr(qual, c0))
 			dd_qualifier(dd);
 		else
@@ -1631,24 +1636,31 @@ static int dd_name(struct demangle_data *dd)
 static int dd_encoding(struct demangle_data *dd)
 {
 	int ret;
-	char c = dd_curr(dd);
+	char c;
 	char end[] = "E.@";
 
 	if (dd_eof(dd))
 		return -1;
 
-	dd_add_debug(dd);
+	if (dd->pos == 0)
+		dd_consume_n(dd, 2); /* skip initial "_Z" */
+	else
+		dd_add_debug(dd);
 
-	if (c == 'T' || c == 'G')
-		return dd_special_name(dd);
+	dd->level++;
+
+	c = dd_curr(dd);
+	if (c == 'T' || c == 'G') {
+		ret = dd_special_name(dd);
+		dd->level--;
+		return ret;
+	}
 
 	ret = dd_name(dd);
 	if (ret < 0)
 		return ret;
 
 	while (!dd_eof(dd) && !strchr(end, dd_curr(dd))) {
-		__dd_add_debug(dd, "dd_type");
-
 		if (dd_type(dd) < 0)
 			break;
 	}
@@ -1660,6 +1672,7 @@ static int dd_encoding(struct demangle_data *dd)
 	if (dd_curr(dd) == '@')
 		dd->len = dd->pos;
 
+	dd->level--;
 	return 0;
 }
 
@@ -1681,9 +1694,6 @@ static char *demangle_simple(char *str)
 	if (dd.old[0] != '_' || dd.old[1] != 'Z')
 		return xstrdup(str);
 
-	dd.pos = 2;
-	dd.new = xzalloc(0);
-
 	if (dd_encoding(&dd) < 0 || dd.level != 0) {
 		dd_debug_print(&dd);
 		free(dd.new);
@@ -1714,7 +1724,7 @@ static char *demangle_simple(char *str)
 static char *demangle_full(char *str)
 {
 	char *symname;
-	size_t len = 64;  /* minimum length */
+	size_t len = 64; /* minimum length */
 	int status;
 
 	__cxa_demangle(str, NULL, &len, &status);
@@ -1759,12 +1769,13 @@ char *demangle(char *str)
 
 #ifdef UNIT_TEST
 
-#define DEMANGLE_TEST(m, d)			\
-do {						\
-	char *name = demangle_simple(m);	\
-	TEST_STREQ(d, name);			\
-	free(name);				\
-} while (0)
+#define DEMANGLE_TEST(m, d)                                                                        \
+	do {                                                                                       \
+		char *name = demangle_simple(m);                                                   \
+		pr_dbg("%.64s should be converted to %s\n", m, d);                                 \
+		TEST_STREQ(d, name);                                                               \
+		free(name);                                                                        \
+	} while (0)
 
 TEST_CASE(demangle_simple1)
 {
@@ -1779,8 +1790,7 @@ TEST_CASE(demangle_simple1)
 
 TEST_CASE(demangle_simple2)
 {
-	DEMANGLE_TEST("_ZThn8_N13FtraceServiceD0Ev",
-		      "FtraceService::~FtraceService");
+	DEMANGLE_TEST("_ZThn8_N13FtraceServiceD0Ev", "FtraceService::~FtraceService");
 	DEMANGLE_TEST("_ZN2v88internal12ScopedVectorIcEC1Ei",
 		      "v8::internal::ScopedVector::ScopedVector");
 	DEMANGLE_TEST("_ZNSt16allocator_traitsISaISt13_Rb_tree_node"
@@ -1794,8 +1804,7 @@ TEST_CASE(demangle_simple2)
 
 TEST_CASE(demangle_simple3)
 {
-	DEMANGLE_TEST("_ZN4node8Watchdog7DestroyEv.part.0",
-		      "node::Watchdog::Destroy");
+	DEMANGLE_TEST("_ZN4node8Watchdog7DestroyEv.part.0", "node::Watchdog::Destroy");
 	DEMANGLE_TEST("_ZN2v88internal8CodeStub6GetKeyEv.constprop.17",
 		      "v8::internal::CodeStub::GetKey");
 	DEMANGLE_TEST("_ZSteqIPN2v88internal8compiler4NodeERKS4_PS5_E"
@@ -1812,7 +1821,8 @@ TEST_CASE(demangle_simple3)
 	DEMANGLE_TEST("_ZSt3powIidEN9__gnu_cxx11__promote_2IT_T0_NS0_"
 		      "9__promoteIS2_XsrSt12__is_integerIS2_E7__valueEE"
 		      "6__typeENS4_IS3_XsrS5_IS3_E7__valueEE6__typeEE"
-		      "6__typeES2_S3_", "std::pow");
+		      "6__typeES2_S3_",
+		      "std::pow");
 
 	return TEST_OK;
 }
@@ -1833,8 +1843,7 @@ TEST_CASE(demangle_simple4)
 		      "IKS0_EENSt9enable_ifIXntsrNS1_15__select_helper"
 		      "IT_EE5valueES6_E4typeERS6_",
 		      "std::allocator_traits::_S_select");
-	DEMANGLE_TEST("_ZN6icu_5416umtx_loadAcquireERU7_Atomici",
-		      "icu_54::umtx_loadAcquire");
+	DEMANGLE_TEST("_ZN6icu_5416umtx_loadAcquireERU7_Atomici", "icu_54::umtx_loadAcquire");
 
 	return TEST_OK;
 }
@@ -1851,16 +1860,13 @@ TEST_CASE(demangle_simple5)
 		      "1EE18IterateWithWrapperIPFvPPNS0_10HeapObjectE"
 		      "S8_EEEvPNS0_4HeapET_EUlPhE_EEiSE_",
 		      "v8::internal::SlotSet::Iterate");
-	DEMANGLE_TEST("_ZNSt5tupleIJPbSt14default_deleteIA_bEEEC2Ev",
-		      "std::tuple::tuple");
+	DEMANGLE_TEST("_ZNSt5tupleIJPbSt14default_deleteIA_bEEEC2Ev", "std::tuple::tuple");
 	DEMANGLE_TEST("_Z26storageIndexFromLayoutItemRK"
 		      "N51_GLOBAL__N_kernel_qformlayout.cpp_C3DE8A26_2E30FA86"
 		      "17FixedColumnMatrixIP15QFormLayoutItemLi2EEES2_",
 		      "storageIndexFromLayoutItem");
-	DEMANGLE_TEST("_ZGTtNSt11range_errorD1Ev",
-		      "std::range_error::~range_error");
-	DEMANGLE_TEST("_ZNSi6ignoreEl@@GLIBCXX_3.4.5",
-		      "std::basic_istream::ignore");
+	DEMANGLE_TEST("_ZGTtNSt11range_errorD1Ev", "std::range_error::~range_error");
+	DEMANGLE_TEST("_ZNSi6ignoreEl@@GLIBCXX_3.4.5", "std::basic_istream::ignore");
 	DEMANGLE_TEST("_ZN4llvm12function_refIFN5clang12ActionResult"
 		      "IPNS1_4ExprELb1EEES4_EE11callback_fnIZNS1_4Sema"
 		      "25CorrectDelayedTyposInExprES4_PNS1_7VarDeclE"
@@ -1888,16 +1894,12 @@ TEST_CASE(demangle_simple6)
 
 TEST_CASE(demangle_simple7)
 {
-	DEMANGLE_TEST("_ZTSSt12system_error",
-		      "__typeinfo__std::system_error");
-	DEMANGLE_TEST("_ZNSs4nposE",
-		      "std::basic_string<>::npos");
-	DEMANGLE_TEST("_ZNSt14numeric_limitsIoE5radixE",
-		      "std::numeric_limits::radix");
+	DEMANGLE_TEST("_ZTSSt12system_error", "__typeinfo__std::system_error");
+	DEMANGLE_TEST("_ZNSs4nposE", "std::basic_string<>::npos");
+	DEMANGLE_TEST("_ZNSt14numeric_limitsIoE5radixE", "std::numeric_limits::radix");
 	DEMANGLE_TEST("_ZGVNSt7__cxx117collateIcE2idE",
 		      "__guard_variable__std::__cxx11::collate::id");
-	DEMANGLE_TEST("_ZNSbIwSt11char_traitsIwESaIwEE4nposE",
-		      "std::basic_string::npos");
+	DEMANGLE_TEST("_ZNSbIwSt11char_traitsIwESaIwEE4nposE", "std::basic_string::npos");
 
 	return TEST_OK;
 }
@@ -1913,6 +1915,11 @@ TEST_CASE(demangle_simple8)
 		      "__construction_vtable__v8::internal::StdoutStream");
 	DEMANGLE_TEST("_ZGRZNK5blink8Variable27GetPropertyNameAtomicStringEvE4name_",
 		      "__ref_temp__blink::Variable::GetPropertyNameAtomicString::name");
+	DEMANGLE_TEST("_ZNSt14numeric_limitsIDuE8is_exactE", "std::numeric_limits::is_exact");
+	DEMANGLE_TEST("_ZTCSt10istrstream0_Si", "__construction_vtable__std::istrstream");
+	DEMANGLE_TEST(
+		"_ZTCNSt7__cxx1119basic_istringstreamIwSt11char_traitsIwESaIwEEE0_St13basic_istreamIwS2_E",
+		"__construction_vtable__std::__cxx11::basic_istringstream::std::std::allocator");
 
 	return TEST_OK;
 }
@@ -1920,8 +1927,7 @@ TEST_CASE(demangle_simple8)
 TEST_CASE(demangle_rust1)
 {
 	DEMANGLE_TEST("_ZN8$BP$test3fooE", "*test::foo");
-	DEMANGLE_TEST("_ZN35Bar$LT$$u5b$u32$u3b$$u20$4$u5d$$GT$E",
-		      "Bar<[u32; 4]>");
+	DEMANGLE_TEST("_ZN35Bar$LT$$u5b$u32$u3b$$u20$4$u5d$$GT$E", "Bar<[u32; 4]>");
 	DEMANGLE_TEST("_ZN71_$LT$Test$u20$$u2b$$u20$$u27$static"
 		      "$u20$as$u20$foo..Bar$LT$Test$GT$$GT$3barE",
 		      "_<Test + 'static as foo..Bar<Test>>::bar");
