@@ -9,25 +9,20 @@
 #if defined(HAVE_LIBPYTHON2) || defined(HAVE_LIBPYTHON3)
 
 /* This should be defined before #include "utils.h" */
-#define PR_FMT     "script"
-#define PR_DOMAIN  DBG_SCRIPT
+#define PR_FMT "script"
+#define PR_DOMAIN DBG_SCRIPT
 
+#include "utils/script-python.h"
+#include "utils/filter.h"
+#include "utils/fstack.h"
+#include "utils/script.h"
+#include "utils/symbol.h"
+#include "utils/utils.h"
 #include <dlfcn.h>
 #include <pthread.h>
-#include "utils/utils.h"
-#include "utils/symbol.h"
-#include "utils/fstack.h"
-#include "utils/filter.h"
-#include "utils/script.h"
-#include "utils/script-python.h"
 
-#ifdef HAVE_LIBPYTHON2
-/* python library name, it only supports python 2.7 as of now */
-static const char libpython[] = "libpython2.7.so";
-#else
-/* python library name, it should support any version python3 */
-static const char libpython[] = "libpython" stringify(LIBPYTHON3_VERSION) ".so";
-#endif
+/* python library name, it should support any version python v2 or v3 */
+static const char libpython[] = "libpython" stringify(LIBPYTHON_VERSION) ".so";
 
 /* python library handle returned by dlopen() */
 static void *python_handle;
@@ -41,44 +36,69 @@ static bool python_error_reported = false;
 /* whether script_init() was done successfully */
 static bool python_initialized;
 
-static PyAPI_FUNC(void) (*__Py_Initialize)(void);
-static PyAPI_FUNC(void) (*__Py_Finalize)(void);
-static PyAPI_FUNC(void) (*__PySys_SetPath)(char *);
-static PyAPI_FUNC(PyObject *) (*__PyImport_Import)(PyObject *name);
+static void (*__Py_Initialize)(void);
+static void (*__Py_Finalize)(void);
+static void (*__PySys_SetPath)(char *);
+static PyObject *(*__PyImport_Import)(PyObject *name);
 
-static PyAPI_FUNC(PyObject *) (*__PyErr_Occurred)(void);
-static PyAPI_FUNC(void) (*__PyErr_Print)(void);
-static PyAPI_FUNC(void) (*__PyErr_Clear)(void);
+static PyObject *(*__PyErr_Occurred)(void);
+static void (*__PyErr_Print)(void);
+static void (*__PyErr_Clear)(void);
 
-static PyAPI_FUNC(int) (*__PyObject_HasAttrString)(PyObject *, const char *);
-static PyAPI_FUNC(PyObject *) (*__PyObject_GetAttrString)(PyObject *, const char *);
-static PyAPI_FUNC(int) (*__PyCallable_Check)(PyObject *);
-static PyAPI_FUNC(PyObject *) (*__PyObject_CallObject)(PyObject *callable_object, PyObject *args);
-static PyAPI_FUNC(int) (*__PyRun_SimpleStringFlags)(const char *, PyCompilerFlags *);
+static int (*__PyObject_HasAttrString)(PyObject *, const char *);
+static PyObject *(*__PyObject_GetAttrString)(PyObject *, const char *);
+static int (*__PyCallable_Check)(PyObject *);
+static PyObject *(*__PyObject_CallObject)(PyObject *callable_object, PyObject *args);
+static int (*__PyRun_SimpleStringFlags)(const char *, PyCompilerFlags *);
 
-static PyAPI_FUNC(PyObject *) (*__PyString_FromString)(const char *);
-static PyAPI_FUNC(PyObject *) (*__PyInt_FromLong)(long);
-static PyAPI_FUNC(PyObject *) (*__PyLong_FromLong)(long);
-static PyAPI_FUNC(PyObject *) (*__PyLong_FromUnsignedLongLong)(unsigned PY_LONG_LONG);
-static PyAPI_FUNC(PyObject *) (*__PyFloat_FromDouble)(double);
-static PyAPI_FUNC(PyObject *) (*__PyBool_FromLong)(long);
+static PyObject *(*__PyString_FromString)(const char *);
+static PyObject *(*__PyInt_FromLong)(long);
+static PyObject *(*__PyLong_FromLong)(long);
+static PyObject *(*__PyLong_FromUnsignedLongLong)(unsigned PY_LONG_LONG);
+static PyObject *(*__PyFloat_FromDouble)(double);
+static PyObject *(*__PyBool_FromLong)(long);
 
-static PyAPI_FUNC(char *) (*__PyString_AsString)(PyObject *);
-static PyAPI_FUNC(long) (*__PyLong_AsLong)(PyObject *);
+static char *(*__PyString_AsString)(PyObject *);
+static long (*__PyLong_AsLong)(PyObject *);
 
-static PyAPI_FUNC(PyObject *) (*__PyTuple_New)(Py_ssize_t size);
-static PyAPI_FUNC(int) (*__PyTuple_SetItem)(PyObject *, Py_ssize_t, PyObject *);
-static PyAPI_FUNC(PyObject *) (*__PyTuple_GetItem)(PyObject *, Py_ssize_t);
+static PyObject *(*__PyTuple_New)(Py_ssize_t size);
+static int (*__PyTuple_SetItem)(PyObject *, Py_ssize_t, PyObject *);
+static PyObject *(*__PyTuple_GetItem)(PyObject *, Py_ssize_t);
 
-static PyAPI_FUNC(Py_ssize_t) (*__PyList_Size)(PyObject *);
-static PyAPI_FUNC(PyObject *) (*__PyList_GetItem)(PyObject *, Py_ssize_t);
+static Py_ssize_t (*__PyList_Size)(PyObject *);
+static PyObject *(*__PyList_GetItem)(PyObject *, Py_ssize_t);
 
-static PyAPI_FUNC(PyObject *) (*__PyDict_New)(void);
-static PyAPI_FUNC(int) (*__PyDict_SetItem)(PyObject *mp, PyObject *key, PyObject *item);
-static PyAPI_FUNC(int) (*__PyDict_SetItemString)(PyObject *dp, const char *key, PyObject *item);
-static PyAPI_FUNC(PyObject *) (*__PyDict_GetItem)(PyObject *mp, PyObject *key);
+static PyObject *(*__PyDict_New)(void);
+static int (*__PyDict_SetItem)(PyObject *mp, PyObject *key, PyObject *item);
+static int (*__PyDict_SetItemString)(PyObject *dp, const char *key, PyObject *item);
+static PyObject *(*__PyDict_GetItem)(PyObject *mp, PyObject *key);
 
-static PyObject *pModule, *pFuncBegin, *pFuncEntry, *pFuncExit, *pFuncEnd;
+/* for python3.8+ compatibility */
+static void (*__Py_Dealloc)(PyObject *);
+
+#if PY_VERSION_HEX >= 0x03080000
+
+static inline void __Py_DECREF(PyObject *obj)
+{
+	if (--obj->ob_refcnt == 0)
+		__Py_Dealloc(obj);
+}
+
+#undef Py_DECREF
+#define Py_DECREF(obj) __Py_DECREF((PyObject *)obj))
+
+static inline void __Py_XDECREF(PyObject *obj)
+{
+	if (obj)
+		__Py_DECREF(obj);
+}
+
+#undef Py_XDECREF
+#define Py_XDECREF(obj) __Py_XDECREF((PyObject *)obj)
+
+#endif /* PY_VERSION_HEX >= 0x03080000 */
+
+static PyObject *pModule, *pFuncBegin, *pFuncEntry, *pFuncExit, *pFuncEvent, *pFuncEnd;
 
 enum py_context_idx {
 	PY_CTX_TID = 0,
@@ -93,32 +113,25 @@ enum py_context_idx {
 
 /* The order has to be aligned with enum py_args above. */
 static const char *py_context_table[] = {
-	"tid",
-	"depth",
-	"timestamp",
-	"duration",
-	"address",
-	"name",
-	"args",
-	"retval",
+	"tid", "depth", "timestamp", "duration", "address", "name", "args", "retval",
 };
 
-#define INIT_PY_API_FUNC(func) \
-	do { \
-		__##func = dlsym(python_handle, #func); \
-		if (!__##func) { \
-			pr_err("dlsym for \"" #func "\" is failed"); \
-			return -1; \
-		} \
+#define INIT_PY_API_FUNC(func)                                                                     \
+	do {                                                                                       \
+		__##func = dlsym(python_handle, #func);                                            \
+		if (!__##func) {                                                                   \
+			pr_err("dlsym for \"" #func "\" is failed");                               \
+			return -1;                                                                 \
+		}                                                                                  \
 	} while (0)
 
-#define INIT_PY_API_FUNC2(func, name)		\
-	do { \
-		__##func = dlsym(python_handle, #name); \
-		if (!__##func) { \
-			pr_err("dlsym for \"" #name "\" is failed"); \
-			return -1; \
-		} \
+#define INIT_PY_API_FUNC2(func, name)                                                              \
+	do {                                                                                       \
+		__##func = dlsym(python_handle, #name);                                            \
+		if (!__##func) {                                                                   \
+			pr_err("dlsym for \"" #name "\" is failed");                               \
+			return -1;                                                                 \
+		}                                                                                  \
 	} while (0)
 
 static int load_python_api_funcs(void)
@@ -139,11 +152,14 @@ static int load_python_api_funcs(void)
 	INIT_PY_API_FUNC(PyString_FromString);
 	INIT_PY_API_FUNC(PyInt_FromLong);
 	INIT_PY_API_FUNC(PyString_AsString);
+	/* just to suppress compiler warning */
+	__Py_Dealloc = NULL;
 #else
 	INIT_PY_API_FUNC2(PySys_SetPath, Py_SetPath);
 	INIT_PY_API_FUNC2(PyString_FromString, PyUnicode_FromString);
 	INIT_PY_API_FUNC2(PyInt_FromLong, PyLong_FromLong);
 	INIT_PY_API_FUNC2(PyString_AsString, PyUnicode_AsUTF8);
+	INIT_PY_API_FUNC2(Py_Dealloc, _Py_Dealloc);
 #endif
 
 	INIT_PY_API_FUNC(PyErr_Occurred);
@@ -240,14 +256,13 @@ static int import_python_module(char *py_pathname)
 }
 
 union python_val {
-	long			l;
-	unsigned long long	ull;
-	char			*s;
-	double			f;
+	long l;
+	unsigned long long ull;
+	char *s;
+	double f;
 };
 
-static void python_insert_tuple(PyObject *tuple, char type, int idx,
-				union python_val val)
+static void python_insert_tuple(PyObject *tuple, char type, int idx, union python_val val)
 {
 	PyObject *obj;
 
@@ -260,6 +275,11 @@ static void python_insert_tuple(PyObject *tuple, char type, int idx,
 		break;
 	case 's':
 		obj = __PyString_FromString(val.s);
+		if (__PyErr_Occurred()) {
+			Py_XDECREF(obj);
+			obj = __PyString_FromString("<invalid value>");
+			__PyErr_Clear();
+		}
 		break;
 	case 'f':
 		obj = __PyFloat_FromDouble(val.f);
@@ -273,8 +293,7 @@ static void python_insert_tuple(PyObject *tuple, char type, int idx,
 	__PyTuple_SetItem(tuple, idx, obj);
 }
 
-static void python_insert_dict(PyObject *dict, char type, const char *key,
-			       union python_val val)
+static void python_insert_dict(PyObject *dict, char type, const char *key, union python_val val)
 {
 	PyObject *obj;
 
@@ -287,6 +306,11 @@ static void python_insert_dict(PyObject *dict, char type, const char *key,
 		break;
 	case 's':
 		obj = __PyString_FromString(val.s);
+		if (__PyErr_Occurred()) {
+			Py_XDECREF(obj);
+			obj = __PyString_FromString("<invalid value>");
+			__PyErr_Clear();
+		}
 		break;
 	case 'b':
 		obj = __PyBool_FromLong(val.l);
@@ -303,53 +327,69 @@ static void python_insert_dict(PyObject *dict, char type, const char *key,
 
 static void insert_tuple_long(PyObject *tuple, int idx, long v)
 {
-	union python_val val = { .l = v, };
+	union python_val val = {
+		.l = v,
+	};
 	python_insert_tuple(tuple, 'l', idx, val);
 }
 
 static void insert_tuple_ull(PyObject *tuple, int idx, unsigned long long v)
 {
-	union python_val val = { .ull = v, };
+	union python_val val = {
+		.ull = v,
+	};
 	python_insert_tuple(tuple, 'U', idx, val);
 }
 
 static void insert_tuple_string(PyObject *tuple, int idx, char *v)
 {
-	union python_val val = { .s = v, };
+	union python_val val = {
+		.s = v,
+	};
 	python_insert_tuple(tuple, 's', idx, val);
 }
 
 static void insert_tuple_double(PyObject *tuple, int idx, double v)
 {
-	union python_val val = { .f = v, };
+	union python_val val = {
+		.f = v,
+	};
 	python_insert_tuple(tuple, 'f', idx, val);
 }
 
 static void insert_dict_long(PyObject *dict, const char *key, long v)
 {
-	union python_val val = { .l = v, };
+	union python_val val = {
+		.l = v,
+	};
 	python_insert_dict(dict, 'l', key, val);
 }
 
 static void insert_dict_ull(PyObject *dict, const char *key, unsigned long long v)
 {
-	union python_val val = { .ull = v, };
+	union python_val val = {
+		.ull = v,
+	};
 	python_insert_dict(dict, 'U', key, val);
 }
 
 static void insert_dict_string(PyObject *dict, const char *key, char *v)
 {
-	union python_val val = { .s = v, };
+	union python_val val = {
+		.s = v,
+	};
 	python_insert_dict(dict, 's', key, val);
 }
 
 static void insert_dict_bool(PyObject *dict, const char *key, bool v)
 {
-	union python_val val = { .l = v, };
+	union python_val val = {
+		.l = v,
+	};
 	python_insert_dict(dict, 'b', key, val);
 }
 
-#define PYCTX(_item)  py_context_table[PY_CTX_##_item]
+#define PYCTX(_item) py_context_table[PY_CTX_##_item]
 
 static void setup_common_context(PyObject **pDict, struct script_context *sc_ctx)
 {
@@ -360,8 +400,7 @@ static void setup_common_context(PyObject **pDict, struct script_context *sc_ctx
 	insert_dict_string(*pDict, PYCTX(NAME), sc_ctx->name);
 }
 
-static void setup_argument_context(PyObject **pDict, bool is_retval,
-				   struct script_context *sc_ctx)
+static void setup_argument_context(PyObject **pDict, bool is_retval, struct script_context *sc_ctx)
 {
 	struct uftrace_arg_spec *spec;
 	void *data = sc_ctx->argbuf;
@@ -440,8 +479,7 @@ static void setup_argument_context(PyObject **pDict, bool is_retval,
 				dval = (double)val.D;
 				break;
 			default:
-				pr_dbg("invalid floating-point type size %d\n",
-				       spec->size);
+				pr_dbg("invalid floating-point type size %d\n", spec->size);
 				dval = 0;
 				break;
 			}
@@ -461,7 +499,7 @@ static void setup_argument_context(PyObject **pDict, bool is_retval,
 			str[slen] = '\0';
 
 			/* NULL string is encoded as '0xffffffff' */
-			if (!memcmp(str, &null_str, sizeof(null_str)))
+			if (slen == 4 && !memcmp(str, &null_str, sizeof(null_str)))
 				strcpy(str, "NULL");
 
 			insert_tuple_string(args, count++, str);
@@ -478,6 +516,14 @@ static void setup_argument_context(PyObject **pDict, bool is_retval,
 			data += 4;
 			break;
 
+		case ARG_FMT_STRUCT:
+			str = NULL;
+			xasprintf(&str, "struct: %s{}", spec->type_name ? spec->type_name : "");
+			insert_tuple_string(args, count++, str);
+			free(str);
+			data += ALIGN(spec->size, 4);
+			break;
+
 		default:
 			pr_warn("invalid argument format: %d\n", spec->fmt);
 			break;
@@ -497,20 +543,45 @@ static void setup_argument_context(PyObject **pDict, bool is_retval,
 	Py_XDECREF(args);
 }
 
+static void setup_event_argument(PyObject *pDict, struct script_context *sc_ctx)
+{
+	char *data = sc_ctx->argbuf;
+	PyObject *args;
+
+	if (data == NULL)
+		data = "";
+
+	args = __PyString_FromString(data);
+	if (__PyErr_Occurred()) {
+		Py_XDECREF(args);
+		args = __PyString_FromString("<invalid value>");
+		__PyErr_Clear();
+	}
+
+	/* arguments will be returned in a tuple */
+	__PyDict_SetItemString(pDict, PYCTX(ARGS), args);
+
+	Py_XDECREF(args);
+}
+
 int python_uftrace_begin(struct script_info *info)
 {
+	PyObject *dict;
+	PyObject *cmds;
+	PyObject *ctx;
+	int i;
+	char *s;
+
 	if (unlikely(!pFuncBegin))
 		return -1;
 
 	/* python_interpreter_lock is already held */
-	PyObject *dict = __PyDict_New();
+	dict = __PyDict_New();
 
 	insert_dict_bool(dict, "record", info->record);
 	insert_dict_string(dict, "version", info->version);
 
-	int i;
-	char *s;
-	PyObject *cmds = __PyTuple_New(info->cmds.nr);
+	cmds = __PyTuple_New(info->cmds.nr);
 
 	strv_for_each(&info->cmds, s, i)
 		insert_tuple_string(cmds, i, s);
@@ -518,7 +589,7 @@ int python_uftrace_begin(struct script_info *info)
 	__PyDict_SetItemString(dict, "cmds", cmds);
 	Py_XDECREF(cmds);
 
-	PyObject *ctx = __PyTuple_New(1);
+	ctx = __PyTuple_New(1);
 
 	__PyTuple_SetItem(ctx, 0, dict);
 	__PyObject_CallObject(pFuncBegin, ctx);
@@ -536,13 +607,16 @@ int python_uftrace_begin(struct script_info *info)
 
 int python_uftrace_entry(struct script_context *sc_ctx)
 {
+	PyObject *pDict;
+	PyObject *pythonContext;
+
 	if (unlikely(!pFuncEntry))
 		return -1;
 
 	pthread_mutex_lock(&python_interpreter_lock);
 
 	/* Entire arguments are passed into a single dictionary. */
-	PyObject *pDict = __PyDict_New();
+	pDict = __PyDict_New();
 
 	/* Setup common info in both entry and exit into a dictionary */
 	setup_common_context(&pDict, sc_ctx);
@@ -551,7 +625,7 @@ int python_uftrace_entry(struct script_context *sc_ctx)
 		setup_argument_context(&pDict, false, sc_ctx);
 
 	/* Python function arguments must be passed in a tuple. */
-	PyObject *pythonContext = __PyTuple_New(1);
+	pythonContext = __PyTuple_New(1);
 	__PyTuple_SetItem(pythonContext, 0, pDict);
 
 	/* Call python function "uftrace_entry". */
@@ -575,13 +649,16 @@ int python_uftrace_entry(struct script_context *sc_ctx)
 
 int python_uftrace_exit(struct script_context *sc_ctx)
 {
+	PyObject *pDict;
+	PyObject *pythonContext;
+
 	if (unlikely(!pFuncExit))
 		return -1;
 
 	pthread_mutex_lock(&python_interpreter_lock);
 
 	/* Entire arguments are passed into a single dictionary. */
-	PyObject *pDict = __PyDict_New();
+	pDict = __PyDict_New();
 
 	/* Setup common info in both entry and exit into a dictionary */
 	setup_common_context(&pDict, sc_ctx);
@@ -593,7 +670,7 @@ int python_uftrace_exit(struct script_context *sc_ctx)
 		setup_argument_context(&pDict, true, sc_ctx);
 
 	/* Python function arguments must be passed in a tuple. */
-	PyObject *pythonContext = __PyTuple_New(1);
+	pythonContext = __PyTuple_New(1);
 	__PyTuple_SetItem(pythonContext, 0, pDict);
 
 	/* Call python function "uftrace_exit". */
@@ -615,6 +692,46 @@ int python_uftrace_exit(struct script_context *sc_ctx)
 	return 0;
 }
 
+int python_uftrace_event(struct script_context *sc_ctx)
+{
+	PyObject *pDict;
+	PyObject *pythonContext;
+
+	if (unlikely(!pFuncEvent))
+		return -1;
+
+	pthread_mutex_lock(&python_interpreter_lock);
+
+	/* Entire arguments are passed into a single dictionary. */
+	pDict = __PyDict_New();
+
+	/* Setup common info into a dictionary */
+	setup_common_context(&pDict, sc_ctx);
+	setup_event_argument(pDict, sc_ctx);
+
+	/* Python function arguments must be passed in a tuple. */
+	pythonContext = __PyTuple_New(1);
+	__PyTuple_SetItem(pythonContext, 0, pDict);
+
+	/* Call python function "uftrace_exit". */
+	__PyObject_CallObject(pFuncEvent, pythonContext);
+	if (debug) {
+		if (__PyErr_Occurred() && !python_error_reported) {
+			pr_dbg("uftrace_event failed:\n");
+			__PyErr_Print();
+
+			python_error_reported = true;
+		}
+	}
+
+	/* Free PyTuple. */
+	Py_XDECREF(pythonContext);
+
+	pthread_mutex_unlock(&python_interpreter_lock);
+
+	return 0;
+}
+
 int python_uftrace_end(void)
 {
 	if (unlikely(!pFuncEnd))
@@ -650,7 +767,7 @@ int python_atfork_prepare(void)
 	return 0;
 }
 
-static PyObject * get_python_callback(char *name)
+static PyObject *get_python_callback(char *name)
 {
 	PyObject *func;
 
@@ -669,8 +786,7 @@ static PyObject * get_python_callback(char *name)
 	return func;
 }
 
-int script_init_for_python(struct script_info *info,
-			   enum uftrace_pattern_type ptype)
+int script_init_for_python(struct script_info *info, enum uftrace_pattern_type ptype)
 {
 	char *py_pathname = info->name;
 
@@ -679,6 +795,7 @@ int script_init_for_python(struct script_info *info,
 	/* Bind script_uftrace functions to python's. */
 	script_uftrace_entry = python_uftrace_entry;
 	script_uftrace_exit = python_uftrace_exit;
+	script_uftrace_event = python_uftrace_event;
 	script_uftrace_end = python_uftrace_end;
 	script_atfork_prepare = python_atfork_prepare;
 
@@ -705,8 +822,7 @@ int script_init_for_python(struct script_info *info,
 	/* check if script has its own list of functions to run */
 	if (__PyObject_HasAttrString(pModule, "UFTRACE_FUNCS")) {
 		int i, len;
-		PyObject *filter_list = __PyObject_GetAttrString(pModule,
-								 "UFTRACE_FUNCS");
+		PyObject *filter_list = __PyObject_GetAttrString(pModule, "UFTRACE_FUNCS");
 		/* XXX: type checking is hard */
 		len = __PyList_Size(filter_list);
 
@@ -719,8 +835,9 @@ int script_init_for_python(struct script_info *info,
 
 	pFuncBegin = get_python_callback("uftrace_begin");
 	pFuncEntry = get_python_callback("uftrace_entry");
-	pFuncExit  = get_python_callback("uftrace_exit");
-	pFuncEnd   = get_python_callback("uftrace_end");
+	pFuncExit = get_python_callback("uftrace_exit");
+	pFuncEvent = get_python_callback("uftrace_event");
+	pFuncEnd = get_python_callback("uftrace_end");
 
 	/* Call python function "uftrace_begin" immediately if possible. */
 	python_uftrace_begin(info);
