@@ -1,39 +1,39 @@
-#include <stdlib.h>
 #include <stdint.h>
-#include <assert.h>
+#include <stdlib.h>
 #include <unistd.h>
 
-#define PR_FMT     "session"
-#define PR_DOMAIN  DBG_SESSION
+#define PR_FMT "session"
+#define PR_DOMAIN DBG_SESSION
 
-#include "uftrace.h"
-#include "utils/symbol.h"
-#include "utils/rbtree.h"
-#include "utils/utils.h"
-#include "utils/fstack.h"
 #include "libmcount/mcount.h"
+#include "uftrace.h"
+#include "utils/fstack.h"
+#include "utils/rbtree.h"
+#include "utils/symbol.h"
+#include "utils/utils.h"
 
 static void delete_tasks(struct uftrace_session_link *sessions);
 
 /**
  * read_session_map - read memory mappings in a session map file
  * @dirname: directory name of the session
- * @symtabs: symbol table to keep the memory mapping
+ * @addr_space: address space to keep the memory mapping
  * @sid: session id
  *
  * This function reads mapping data from a session map file and
  * construct the address space for a session to resolve symbols
  * in libraries.
  */
-void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
+void read_session_map(char *dirname, struct uftrace_sym_info *sinfo, char *sid)
 {
 	FILE *fp;
 	char buf[PATH_MAX];
 	const char *last_libname = NULL;
-	struct uftrace_mmap **maps = &symtabs->maps;
+	struct uftrace_mmap **maps = &sinfo->maps;
 	struct uftrace_mmap *last_map = NULL;
+	const char build_id_prefix[] = "build-id:";
 
-	snprintf(buf, sizeof(buf), "%s/sid-%.16s.map", dirname, sid);
+	snprintf(buf, sizeof(buf), "%s/sid-%.*s.map", dirname, SESSION_ID_LEN, sid);
 	fp = fopen(buf, "rb");
 	if (fp == NULL)
 		pr_err("cannot open maps file: %s", buf);
@@ -42,18 +42,24 @@ void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
 		uint64_t start, end;
 		char prot[5];
 		char path[PATH_MAX];
+		char build_id[BUILD_ID_STR_SIZE + sizeof(build_id_prefix)];
 		size_t namelen;
 		struct uftrace_mmap *map;
 
+		/* prevent to reuse previous iteration's result */
+		build_id[0] = '\0';
+
 		/* skip anon mappings */
-		if (sscanf(buf, "%"PRIx64"-%"PRIx64" %s %*x %*x:%*x %*d %s\n",
-			   &start, &end, prot, path) != 4)
+		if (sscanf(buf, "%" PRIx64 "-%" PRIx64 " %s %*x %*x:%*x %*d %s %s", &start, &end,
+			   prot, path, build_id) < 4) {
+			pr_dbg("sscanf failed\n");
 			continue;
+		}
 
 		/* skip the [stack] mapping */
 		if (path[0] == '[') {
 			if (strncmp(path, "[stack", 6) == 0)
-				symtabs->kernel_base = guess_kernel_base(buf);
+				sinfo->kernel_base = guess_kernel_base(buf);
 			continue;
 		}
 
@@ -76,10 +82,14 @@ void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
 		memcpy(map->libname, path, namelen);
 		map->libname[strlen(path)] = '\0';
 
+		if (!strncmp(build_id, build_id_prefix, strlen(build_id_prefix))) {
+			memcpy(map->build_id, build_id + strlen(build_id_prefix),
+			       sizeof(build_id) - sizeof(build_id_prefix));
+		}
+
 		/* set mapping of main executable */
-		if (symtabs->exec_map == NULL && symtabs->filename &&
-		    !strcmp(path, symtabs->filename)) {
-			symtabs->exec_map = map;
+		if (sinfo->exec_map == NULL && sinfo->filename && !strcmp(path, sinfo->filename)) {
+			sinfo->exec_map = map;
 		}
 
 		last_libname = map->libname;
@@ -92,25 +102,80 @@ void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
 }
 
 /**
- * delete_session_map - free memory mappings in a symtabs
- * @symtabs: symbol table has the memory mapping
+ * delete_session_map - free memory mappings in an address space
+ * @addr_space: symbol table has the memory mapping
  *
  * This function releases mapping data in a symbol table which
  * was read by read_session_map().
  */
-void delete_session_map(struct symtabs *symtabs)
+void delete_session_map(struct uftrace_sym_info *sinfo)
 {
 	struct uftrace_mmap *map, *tmp;
 
-	map = symtabs->maps;
+	map = sinfo->maps;
 	while (map) {
 		tmp = map->next;
 		free(map);
 		map = tmp;
 	}
 
-	symtabs->maps = NULL;
-	symtabs->exec_map = NULL;
+	sinfo->maps = NULL;
+	sinfo->exec_map = NULL;
+}
+
+/**
+ * update_session_map - rewrite map files to have build-id
+ * @filename - name of map file
+ *
+ * This function updates @filename map file to add build-id at the end
+ * of each line.
+ */
+void update_session_map(const char *filename)
+{
+	FILE *ifp, *ofp;
+	char buf[PATH_MAX];
+	const char build_id_prefix[] = "build-id:";
+
+	ifp = fopen(filename, "r");
+	if (ifp == NULL)
+		pr_err("cannot open map file: %s", filename);
+
+	snprintf(buf, sizeof(buf), "%s.tmp", filename);
+	ofp = fopen(buf, "w");
+	if (ofp == NULL)
+		pr_err("cannot create new map file: %s", buf);
+
+	while (fgets(buf, sizeof(buf), ifp) != NULL) {
+		char path[PATH_MAX];
+		char build_id[BUILD_ID_STR_SIZE];
+		int len;
+
+		len = strlen(buf);
+		if (len > 0 && buf[len - 1] == '\n')
+			buf[--len] = '\0';
+		fwrite_all(buf, len, ofp);
+
+		/* skip anon mappings */
+		if (sscanf(buf, "%*x-%*x %*s %*x %*x:%*x %*d %s", path) != 1)
+			goto next;
+
+		/* skip the special mappings like [stack] */
+		if (path[0] == '[')
+			goto next;
+
+		if (read_build_id(path, build_id, sizeof(build_id)) == 0)
+			fprintf(ofp, " %s%s", build_id_prefix, build_id);
+
+next:
+		fputc('\n', ofp);
+	}
+
+	fclose(ifp);
+	fclose(ofp);
+
+	snprintf(buf, sizeof(buf), "%s.tmp", filename);
+	if (rename(buf, filename) < 0)
+		pr_err("cannot rename map file: %s", filename);
 }
 
 /**
@@ -118,6 +183,7 @@ void delete_session_map(struct symtabs *symtabs)
  * @sessions: session link to manage sessions and tasks
  * @msg: uftrace session message read from task file
  * @dirname: uftrace data directory name
+ * @symdir: symbol directory name
  * @exename: executable name started this session
  * @sym_rel_addr: whether symbol table uses relative address
  * @needs_symtab: whether symbol table loading is needed
@@ -127,9 +193,9 @@ void delete_session_map(struct symtabs *symtabs)
  * session will be added to sessions tree sorted by pid and timestamp.
  * Also it loads symbol table and debug info if needed.
  */
-void create_session(struct uftrace_session_link *sessions,
-		    struct uftrace_msg_sess *msg, char *dirname, char *exename,
-		    bool sym_rel_addr, bool needs_symtab, bool needs_srcline)
+void create_session(struct uftrace_session_link *sessions, struct uftrace_msg_sess *msg,
+		    char *dirname, char *symdir, char *exename, bool sym_rel_addr,
+		    bool needs_symtab, bool needs_srcline)
 {
 	struct uftrace_session *s;
 	struct uftrace_task *t;
@@ -162,20 +228,22 @@ void create_session(struct uftrace_session_link *sessions,
 	s->filters = RB_ROOT;
 	INIT_LIST_HEAD(&s->dlopen_libs);
 
-	pr_dbg2("new session: pid = %d, session = %.16s\n",
-		s->pid, s->sid);
+	pr_dbg2("new session: pid = %d, session = %.*s\n", s->pid, SESSION_ID_LEN, s->sid);
 
 	if (needs_symtab) {
-		s->symtabs.dirname = dirname;
-		s->symtabs.filename = s->exename;
-		s->symtabs.flags = SYMTAB_FL_USE_SYMFILE | SYMTAB_FL_DEMANGLE;
+		s->sym_info.dirname = dirname;
+		s->sym_info.filename = s->exename;
+		s->sym_info.symdir = symdir;
+		s->sym_info.flags = SYMTAB_FL_USE_SYMFILE | SYMTAB_FL_DEMANGLE;
 		if (sym_rel_addr)
-			s->symtabs.flags |= SYMTAB_FL_ADJ_OFFSET;
+			s->sym_info.flags |= SYMTAB_FL_ADJ_OFFSET;
+		if (strcmp(dirname, symdir))
+			s->sym_info.flags |= SYMTAB_FL_SYMS_DIR;
 
-		read_session_map(dirname, &s->symtabs, s->sid);
+		read_session_map(dirname, &s->sym_info, s->sid);
 
-		load_module_symtabs(&s->symtabs);
-		load_debug_info(&s->symtabs, needs_srcline);
+		load_module_symtabs(&s->sym_info);
+		load_debug_info(&s->sym_info, needs_srcline);
 	}
 
 	if (sessions->first == NULL)
@@ -191,8 +259,8 @@ void create_session(struct uftrace_session_link *sessions,
 	rb_insert_color(&s->node, &sessions->root);
 }
 
-static struct uftrace_session *find_session(struct uftrace_session_link *sessions,
-					    int pid, uint64_t timestamp)
+static struct uftrace_session *find_session(struct uftrace_session_link *sessions, int pid,
+					    uint64_t timestamp)
 {
 	struct uftrace_session *iter;
 	struct uftrace_session *s = NULL;
@@ -228,8 +296,7 @@ static struct uftrace_session *find_session(struct uftrace_session_link *session
  * @arg.  As the @callback returns a non-zero value, it'll stop and
  * return in the middle.
  */
-void walk_sessions(struct uftrace_session_link *sessions,
-		   walk_sessions_cb_t callback, void *arg)
+void walk_sessions(struct uftrace_session_link *sessions, walk_sessions_cb_t callback, void *arg)
 {
 	struct rb_node *n = rb_first(&sessions->root);
 	struct uftrace_session *s;
@@ -251,8 +318,7 @@ void walk_sessions(struct uftrace_session_link *sessions,
  *
  * This function returns a matching session or %NULL.
  */
-struct uftrace_session *
-get_session_from_sid(struct uftrace_session_link *sessions, char sid[])
+struct uftrace_session *get_session_from_sid(struct uftrace_session_link *sessions, char sid[])
 {
 	struct rb_node *n = rb_first(&sessions->root);
 	struct uftrace_session *s;
@@ -273,22 +339,24 @@ get_session_from_sid(struct uftrace_session_link *sessions, char sid[])
  * @sess: pointer to a current session
  * @timestamp: timestamp at the dlopen call
  * @base_addr: load address of text segment of the library
- * @libname: name of the librarry
+ * @libname: name of the library
  *
  * This functions adds the info of a library which was loaded by dlopen.
  * Instead of creating a new session, it just adds the library information
  * to the @sess.
  */
-void session_add_dlopen(struct uftrace_session *sess, uint64_t timestamp,
-			unsigned long base_addr, const char *libname)
+void session_add_dlopen(struct uftrace_session *sess, uint64_t timestamp, unsigned long base_addr,
+			const char *libname)
 {
 	struct uftrace_dlopen_list *udl, *pos;
+	char build_id[BUILD_ID_STR_SIZE];
 
 	udl = xmalloc(sizeof(*udl));
 	udl->time = timestamp;
 	udl->base = base_addr;
 
-	udl->mod = load_module_symtab(&sess->symtabs, libname);
+	read_build_id(libname, build_id, sizeof(build_id));
+	udl->mod = load_module_symtab(&sess->sym_info, libname, build_id);
 	list_for_each_entry(pos, &sess->dlopen_libs, list) {
 		if (pos->time > timestamp)
 			break;
@@ -306,11 +374,11 @@ void session_add_dlopen(struct uftrace_session *sess, uint64_t timestamp,
  * @sess using @addr.  The @timestamp is needed to determine which
  * library should be searched.
  */
-struct sym * session_find_dlsym(struct uftrace_session *sess, uint64_t timestamp,
-				unsigned long addr)
+struct uftrace_symbol *session_find_dlsym(struct uftrace_session *sess, uint64_t timestamp,
+					  unsigned long addr)
 {
 	struct uftrace_dlopen_list *pos;
-	struct sym *sym;
+	struct uftrace_symbol *sym;
 
 	list_for_each_entry_reverse(pos, &sess->dlopen_libs, list) {
 		if (pos->time > timestamp)
@@ -336,15 +404,15 @@ void delete_session(struct uftrace_session *sess)
 		free(udl);
 	}
 
-	finish_debug_info(&sess->symtabs);
-	delete_session_map(&sess->symtabs);
+	finish_debug_info(&sess->sym_info);
+	delete_session_map(&sess->sym_info);
 	uftrace_cleanup_filter(&sess->filters);
 	uftrace_cleanup_filter(&sess->fixups);
 	free(sess);
 }
 
 /**
- * delete_sessions - free all resouces in the @sessions
+ * delete_sessions - free all resources in the @sessions
  * @sessions: session link to manage sessions and tasks
  *
  * This function removes all session-related data structure in
@@ -386,8 +454,7 @@ static void add_session_ref(struct uftrace_task *task, struct uftrace_session *s
 	sref->start = timestamp;
 	sref->end = -1ULL;
 
-	pr_dbg2("task session: tid = %d, session = %.16s\n",
-		task->tid, sess->sid);
+	pr_dbg2("task session: tid = %d, session = %.*s\n", task->tid, SESSION_ID_LEN, sess->sid);
 	task->sref_last = sref;
 }
 
@@ -399,12 +466,11 @@ static void add_session_ref(struct uftrace_task *task, struct uftrace_session *s
  *
  * This function searches the sessions tree using @task and @timestamp.
  * The most recent session that has a smaller than the @timestamp will
- * be returned.  If it didn't find a session tries to search sesssion
+ * be returned.  If it didn't find a session tries to search session
  * list of parent or thread-leader.
  */
 struct uftrace_session *find_task_session(struct uftrace_session_link *sessions,
-					  struct uftrace_task *task,
-					  uint64_t timestamp)
+					  struct uftrace_task *task, uint64_t timestamp)
 {
 	int parent_id;
 	struct uftrace_sess_ref *ref;
@@ -441,8 +507,7 @@ struct uftrace_session *find_task_session(struct uftrace_session_link *sessions,
  * The newly created task will have a reference to a session if
  * @needs_session is %true.
  */
-void create_task(struct uftrace_session_link *sessions,
-		 struct uftrace_msg_task *msg, bool fork)
+void create_task(struct uftrace_session_link *sessions, struct uftrace_msg_task *msg, bool fork)
 {
 	struct uftrace_task *t;
 	struct uftrace_task *pt;
@@ -494,9 +559,8 @@ void create_task(struct uftrace_session_link *sessions,
 		t->comm[sizeof(t->comm) - 1] = '\0';
 	}
 
-	pr_dbg2("new task: tid = %d (%.*s), session = %-.16s\n",
-		t->tid, sizeof(t->comm), s ? t->comm : "unknown",
-		s ? s->sid : "unknown");
+	pr_dbg2("new task: tid = %d (%.*s), session = %-.*s\n", t->tid, sizeof(t->comm),
+		s ? t->comm : "unknown", SESSION_ID_LEN, s ? s->sid : "unknown");
 
 	if (sessions->first_task == NULL)
 		sessions->first_task = t;
@@ -568,8 +632,7 @@ struct uftrace_task *find_task(struct uftrace_session_link *sessions, int tid)
  * @arg.  As the @callback returns a non-zero value, it'll stop and
  * return in the middle.
  */
-void walk_tasks(struct uftrace_session_link *sessions,
-		walk_tasks_cb_t callback, void *arg)
+void walk_tasks(struct uftrace_session_link *sessions, walk_tasks_cb_t callback, void *arg)
 {
 	struct rb_node *n = rb_first(&sessions->tasks);
 	struct uftrace_task *t;
@@ -592,13 +655,12 @@ void walk_tasks(struct uftrace_session_link *sessions,
  *
  * This function looks up symbol table in current session.
  */
-struct sym * task_find_sym(struct uftrace_session_link *sessions,
-			   struct uftrace_task_reader *task,
-			   struct uftrace_record *rec)
+struct uftrace_symbol *task_find_sym(struct uftrace_session_link *sessions,
+				     struct uftrace_task_reader *task, struct uftrace_record *rec)
 {
 	struct uftrace_session *sess;
-	struct symtabs *symtabs;
-	struct sym *sym = NULL;
+	struct uftrace_sym_info *sinfo;
+	struct uftrace_symbol *sym = NULL;
 	uint64_t addr = rec->addr;
 
 	sess = find_task_session(sessions, task->t, rec->time);
@@ -606,14 +668,14 @@ struct sym * task_find_sym(struct uftrace_session_link *sessions,
 	if (is_kernel_record(task, rec)) {
 		if (sess == NULL)
 			sess = sessions->first;
-		addr = get_kernel_address(&sess->symtabs, addr);
+		addr = get_kernel_address(&sess->sym_info, addr);
 	}
 
 	if (sess == NULL)
 		return NULL;
 
-	symtabs = &sess->symtabs;
-	sym = find_symtabs(symtabs, addr);
+	sinfo = &sess->sym_info;
+	sym = find_symtabs(sinfo, addr);
 
 	if (sym == NULL)
 		sym = session_find_dlsym(sess, rec->time, addr);
@@ -622,7 +684,7 @@ struct sym * task_find_sym(struct uftrace_session_link *sessions,
 }
 
 /**
- * task_find_sym - find a symbol that matches to @addr
+ * task_find_sym_addr - find a symbol that matches to @addr
  * @sessions: session link to manage sessions and tasks
  * @task: handle for functions in a task
  * @time: timestamp of the @addr
@@ -630,36 +692,105 @@ struct sym * task_find_sym(struct uftrace_session_link *sessions,
  *
  * This function looks up symbol table in current session.
  */
-struct sym * task_find_sym_addr(struct uftrace_session_link *sessions,
-				struct uftrace_task_reader *task,
-				uint64_t time, uint64_t addr)
+struct uftrace_symbol *task_find_sym_addr(struct uftrace_session_link *sessions,
+					  struct uftrace_task_reader *task, uint64_t time,
+					  uint64_t addr)
 {
 	struct uftrace_session *sess;
-	struct sym *sym = NULL;
+	struct uftrace_symbol *sym = NULL;
 
 	sess = find_task_session(sessions, task->t, time);
 
 	if (sess == NULL) {
 		struct uftrace_session *fsess = sessions->first;
 
-		if (is_kernel_address(&fsess->symtabs, addr))
+		if (is_kernel_address(&fsess->sym_info, addr))
 			sess = fsess;
 		else
 			return NULL;
 	}
 
-	sym = find_symtabs(&sess->symtabs, addr);
+	sym = find_symtabs(&sess->sym_info, addr);
 	if (sym == NULL)
 		sym = session_find_dlsym(sess, time, addr);
 
+	if (sym == NULL) {
+		if (EVENT_ID_PERF_SCHED_IN == addr || EVENT_ID_PERF_SCHED_OUT == addr ||
+		    EVENT_ID_PERF_SCHED_BOTH == addr)
+			return &sched_sym;
+		else if (EVENT_ID_PERF_SCHED_OUT_PREEMPT == addr ||
+			 EVENT_ID_PERF_SCHED_BOTH_PREEMPT == addr)
+			return &sched_preempt_sym;
+	}
+
 	return sym;
 }
 
+/**
+ * task_find_loc_addr - find a debug location that matches to @addr
+ * @sessions: session link to manage sessions and tasks
+ * @task: handle for functions in a task
+ * @time: timestamp of the @addr
+ * @addr: instruction address
+ *
+ * This function returns a debug location of symbol
+ * that looked up in symbol table in current session
+ */
+struct uftrace_dbg_loc *task_find_loc_addr(struct uftrace_session_link *sessions,
+					   struct uftrace_task_reader *task, uint64_t time,
+					   uint64_t addr)
+{
+	struct uftrace_session *sess;
+	struct uftrace_symbol *sym = NULL;
+	struct uftrace_mmap *map;
+	struct uftrace_dbg_info *dinfo;
+	struct uftrace_dbg_loc *loc;
+	ptrdiff_t sym_idx;
+
+	sess = find_task_session(sessions, task->t, time);
+
+	if (sess == NULL) {
+		struct uftrace_session *fsess = sessions->first;
+
+		if (is_kernel_address(&fsess->sym_info, addr))
+			sess = fsess;
+		else
+			return NULL;
+	}
+
+	sym = find_symtabs(&sess->sym_info, addr);
+	if (sym == NULL)
+		sym = session_find_dlsym(sess, time, addr);
+
+	if (sym == NULL)
+		return NULL;
+
+	if (sym->type == ST_LOCAL_FUNC || sym->type == ST_GLOBAL_FUNC) {
+		map = find_map(&sess->sym_info, addr);
+		if (map == NULL)
+			return NULL;
+
+		dinfo = &(map->mod->dinfo);
+		if (dinfo == NULL || dinfo->nr_locs_used == 0)
+			return NULL;
+
+		sym_idx = sym - map->mod->symtab.sym;
+		loc = &dinfo->locs[sym_idx];
+		if (loc->file != NULL)
+			return loc;
+	}
+
+	return NULL;
+}
+
 #ifdef UNIT_TEST
 
 static struct uftrace_session_link test_sessions;
-static const char session_map[] =
-	"00400000-00401000 r-xp 00000000 08:03 4096 unittest\n"
+static const char session_map[] = "00400000-00401000 r-xp 00000000 08:03 4096 unittest\n"
+				  "bfff0000-bffff000 rw-p 00000000 08:03 4096 [stack]\n";
+static const char session_map_with_build_id[] =
+	"00400000-00401000 r-xp 00000000 08:03 4096 unittest build-id:1234567890abcdef\n"
+	"5f98a000-5fa8c000 r-xp 00000000 08:03 4096 libc.so\n"
 	"bfff0000-bffff000 rw-p 00000000 08:03 4096 [stack]\n";
 
 TEST_CASE(session_search)
@@ -669,6 +800,7 @@ TEST_CASE(session_search)
 
 	TEST_EQ(test_sessions.first, NULL);
 
+	pr_dbg("create same session %d times\n", NUM_TEST);
 	for (i = 0; i < NUM_TEST; i++) {
 		struct uftrace_msg_sess msg = {
 			.task = {
@@ -682,10 +814,9 @@ TEST_CASE(session_search)
 		int fd;
 
 		fd = creat("sid-test.map", 0400);
-		write_all(fd, session_map, sizeof(session_map)-1);
+		write_all(fd, session_map, sizeof(session_map) - 1);
 		close(fd);
-		create_session(&test_sessions, &msg, ".", "unittest",
-			       false, false, false);
+		create_session(&test_sessions, &msg, ".", ".", "unittest", false, false, false);
 		remove("sid-test.map");
 	}
 
@@ -693,6 +824,7 @@ TEST_CASE(session_search)
 	TEST_EQ(test_sessions.first->pid, 1);
 	TEST_EQ(test_sessions.first->start_time, 0);
 
+	pr_dbg("find sessions including random timestamp\n");
 	for (i = 0; i < NUM_TEST; i++) {
 		int t;
 		struct uftrace_session *s;
@@ -718,7 +850,7 @@ TEST_CASE(task_search)
 	struct uftrace_session *sess;
 	int fd;
 
-	/* 1. create initial task */
+	pr_dbg("create initial task\n");
 	{
 		struct uftrace_msg_sess smsg = {
 			.task = {
@@ -736,10 +868,9 @@ TEST_CASE(task_search)
 		};
 
 		fd = creat("sid-initial.map", 0400);
-		write_all(fd, session_map, sizeof(session_map)-1);
+		write_all(fd, session_map, sizeof(session_map) - 1);
 		close(fd);
-		create_session(&test_sessions, &smsg, ".", "unittest",
-			       false, false, false);
+		create_session(&test_sessions, &smsg, ".", ".", "unittest", false, false, false);
 		create_task(&test_sessions, &tmsg, false);
 		remove("sid-initial.map");
 
@@ -756,11 +887,11 @@ TEST_CASE(task_search)
 		TEST_EQ(sess->tid, task->tid);
 	}
 
-	/* 2. fork child task */
+	pr_dbg("fork child task\n");
 	{
 		struct uftrace_msg_task tmsg = {
-			.pid = 1,  /* ppid */
-			.tid = 2,  /* pid */
+			.pid = 1, /* ppid */
+			.tid = 2, /* pid */
 			.time = 200,
 		};
 
@@ -778,7 +909,7 @@ TEST_CASE(task_search)
 		TEST_LE(sess->start_time, tmsg.time);
 	}
 
-	/* 3. create parent thread */
+	pr_dbg("create parent thread\n");
 	{
 		struct uftrace_msg_task tmsg = {
 			.pid = 1,
@@ -800,7 +931,7 @@ TEST_CASE(task_search)
 		TEST_LE(sess->start_time, tmsg.time);
 	}
 
-	/* 4. create child thread */
+	pr_dbg("create child thread\n");
 	{
 		struct uftrace_msg_task tmsg = {
 			.pid = 2,
@@ -823,7 +954,7 @@ TEST_CASE(task_search)
 		TEST_LE(sess->start_time, tmsg.time);
 	}
 
-	/* 5. exec from child */
+	pr_dbg("exec from child\n");
 	{
 		struct uftrace_msg_sess smsg = {
 			.task = {
@@ -841,10 +972,9 @@ TEST_CASE(task_search)
 		};
 
 		fd = creat("sid-after_exec.map", 0400);
-		write_all(fd, session_map, sizeof(session_map)-1);
+		write_all(fd, session_map, sizeof(session_map) - 1);
 		close(fd);
-		create_session(&test_sessions, &smsg, ".", "unittest",
-			       false, false, false);
+		create_session(&test_sessions, &smsg, ".", ".", "unittest", false, false, false);
 		create_task(&test_sessions, &tmsg, false);
 		remove("sid-after_exec.map");
 
@@ -860,11 +990,11 @@ TEST_CASE(task_search)
 		TEST_LE(sess->start_time, tmsg.time);
 	}
 
-	/* 6. fork grand-child task */
+	pr_dbg("fork grand-child task\n");
 	{
 		struct uftrace_msg_task tmsg = {
-			.pid = 4,  /* ppid */
-			.tid = 5,  /* pid */
+			.pid = 4, /* ppid */
+			.tid = 5, /* pid */
 			.time = 600,
 		};
 
@@ -881,7 +1011,7 @@ TEST_CASE(task_search)
 		TEST_LE(sess->start_time, tmsg.time);
 	}
 
-	/* 7. create grand-child thread */
+	pr_dbg("create grand-child thread\n");
 	{
 		struct uftrace_msg_task tmsg = {
 			.pid = 5,
@@ -903,6 +1033,7 @@ TEST_CASE(task_search)
 		TEST_LE(sess->start_time, tmsg.time);
 	}
 
+	pr_dbg("finding tasks in the initial session\n");
 	task = find_task(&test_sessions, 1);
 	sess = find_task_session(&test_sessions, task, 100);
 	TEST_NE(sess, NULL);
@@ -918,6 +1049,7 @@ TEST_CASE(task_search)
 	TEST_NE(sess, NULL);
 	TEST_STREQ(sess->sid, "initial");
 
+	pr_dbg("finding tasks in the session after exec\n");
 	sess = find_task_session(&test_sessions, task, 500);
 	TEST_NE(sess, NULL);
 	TEST_STREQ(sess->sid, "after_exec");
@@ -941,7 +1073,7 @@ TEST_CASE(task_search)
 
 TEST_CASE(task_symbol)
 {
-	struct sym *sym;
+	struct uftrace_symbol *sym;
 	struct uftrace_msg_sess msg = {
 		.task = {
 			.pid = 1,
@@ -961,6 +1093,7 @@ TEST_CASE(task_symbol)
 	};
 	FILE *fp;
 
+	pr_dbg("creating symbol and map files\n");
 	fp = fopen("sid-test.map", "w");
 	TEST_NE(fp, NULL);
 	fprintf(fp, "%s", session_map);
@@ -975,8 +1108,7 @@ TEST_CASE(task_symbol)
 	fprintf(fp, "00000500 T __sym_end\n");
 	fclose(fp);
 
-	create_session(&test_sessions, &msg, ".", "unittest",
-		       false, true, false);
+	create_session(&test_sessions, &msg, ".", ".", "unittest", false, true, false);
 	create_task(&test_sessions, &tmsg, false);
 	remove("sid-test.map");
 	remove("unittest.sym");
@@ -984,6 +1116,7 @@ TEST_CASE(task_symbol)
 	TEST_NE(test_sessions.first, NULL);
 	TEST_EQ(test_sessions.first->pid, 1);
 
+	pr_dbg("try to find a symbol from a mapped address\n");
 	task.t = find_task(&test_sessions, 1);
 	sym = task_find_sym_addr(&test_sessions, &task, 100, 0x400410);
 
@@ -998,7 +1131,7 @@ TEST_CASE(task_symbol)
 
 TEST_CASE(task_symbol_dlopen)
 {
-	struct sym *sym;
+	struct uftrace_symbol *sym;
 	struct uftrace_msg_sess msg = {
 		.task = {
 			.pid = 1,
@@ -1015,6 +1148,7 @@ TEST_CASE(task_symbol_dlopen)
 	fprintf(fp, "%s", session_map);
 	fclose(fp);
 
+	pr_dbg("creating symbol for the dlopen library\n");
 	fp = fopen("libuftrace-test.so.0.sym", "w");
 	TEST_NE(fp, NULL);
 	fprintf(fp, "0100 P __tls_get_addr\n");
@@ -1024,18 +1158,19 @@ TEST_CASE(task_symbol_dlopen)
 	fprintf(fp, "0500 T __sym_end\n");
 	fclose(fp);
 
-	create_session(&test_sessions, &msg, ".", "unittest",
-		       false, true, false);
+	create_session(&test_sessions, &msg, ".", ".", "unittest", false, true, false);
 	remove("sid-test.map");
 
 	TEST_NE(test_sessions.first, NULL);
 	TEST_EQ(test_sessions.first->pid, 1);
 
+	pr_dbg("add dlopen info message\n");
 	session_add_dlopen(test_sessions.first, 200, 0x7003000, "libuftrace-test.so.0");
 	remove("libuftrace-test.so.0.sym");
 
 	TEST_EQ(list_empty(&test_sessions.first->dlopen_libs), false);
 
+	pr_dbg("try to find a symbol from the dlopen address\n");
 	sym = session_find_dlsym(test_sessions.first, 250, 0x7003410);
 
 	TEST_NE(sym, NULL);
@@ -1047,4 +1182,39 @@ TEST_CASE(task_symbol_dlopen)
 	return TEST_OK;
 }
 
+TEST_CASE(session_map_build_id)
+{
+	FILE *fp;
+	struct uftrace_mmap *map;
+	struct uftrace_sym_info test_sinfo = {
+		.loaded = false,
+	};
+
+	fp = fopen("sid-test.map", "w");
+	TEST_NE(fp, NULL);
+	fprintf(fp, "%s", session_map_with_build_id);
+	fclose(fp);
+
+	pr_dbg("read map file with build-id info\n");
+	read_session_map(".", &test_sinfo, "test");
+
+	pr_dbg("first entry should have a build-id\n");
+	map = test_sinfo.maps;
+	TEST_NE(map, NULL);
+	TEST_STREQ(map->libname, "unittest");
+	TEST_STREQ(map->build_id, "1234567890abcdef");
+
+	pr_dbg("next entry should not have one\n");
+	map = map->next;
+	TEST_NE(map, NULL);
+	TEST_STREQ(map->libname, "libc.so");
+	TEST_STREQ(map->build_id, "");
+
+	map = map->next;
+	TEST_EQ(map, NULL);
+
+	delete_session_map(&test_sinfo);
+	return TEST_OK;
+}
+
 #endif /* UNIT_TEST */
