@@ -1,18 +1,22 @@
-#include <stdio.h>
 #include <dirent.h>
-#include <stdbool.h>
-#include <unistd.h>
-#include <signal.h>
 #include <errno.h>
-#include <sys/uio.h>
-#include <sys/stat.h>
 #include <libgen.h>
+#include <signal.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <sys/stat.h>
+#include <sys/uio.h>
+#include <unistd.h>
+
+#ifdef HAVE_LIBUNWIND
+#define UNW_LOCAL_ONLY
+#include <libunwind.h>
+#endif
 
-#include "uftrace.h"
-#include "utils/utils.h"
-#include "utils/kernel.h"
 #include "libtraceevent/event-parse.h"
-
+#include "uftrace.h"
+#include "utils/kernel.h"
+#include "utils/utils.h"
 
 volatile bool uftrace_done;
 
@@ -26,7 +30,7 @@ void sighandler(int sig)
 
 void setup_signal(void)
 {
-	signal(SIGINT,  sighandler);
+	signal(SIGINT, sighandler);
 	signal(SIGTERM, sighandler);
 	signal(SIGPIPE, sighandler);
 }
@@ -59,9 +63,9 @@ int pread_all(int fd, void *buf, size_t size, off_t off)
 		if (ret <= 0)
 			return -1;
 
-		buf  += ret;
+		buf += ret;
 		size -= ret;
-		off  += ret;
+		off += ret;
 	}
 	return 0;
 }
@@ -78,7 +82,7 @@ int fread_all(void *buf, size_t size, FILE *fp)
 		if (ferror(fp))
 			return -1;
 
-		buf  += ret;
+		buf += ret;
 		size -= ret;
 	}
 	return 0;
@@ -131,7 +135,25 @@ int writev_all(int fd, struct iovec *iov, int count)
 		}
 
 		iov->iov_base += ret;
-		iov->iov_len  -= ret;
+		iov->iov_len -= ret;
+	}
+	return 0;
+}
+
+int fwrite_all(const void *buf, size_t size, FILE *fp)
+{
+	size_t ret;
+
+	while (size) {
+		if (feof(fp))
+			return -1;
+
+		ret = fwrite(buf, 1, size, fp);
+		if (ferror(fp))
+			return -1;
+
+		buf += ret;
+		size -= ret;
 	}
 	return 0;
 }
@@ -152,8 +174,7 @@ int remove_directory(const char *dirname)
 	pr_dbg("removing %s directory\n", dirname);
 
 	while ((ent = readdir(dp)) != NULL) {
-		if (!strcmp(ent->d_name, ".") ||
-		    !strcmp(ent->d_name, ".."))
+		if (!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, ".."))
 			continue;
 
 		snprintf(buf, sizeof(buf), "%s/%s", dirname, ent->d_name);
@@ -186,7 +207,9 @@ static bool is_uftrace_directory(const char *path)
 	int fd;
 	bool ret = false;
 	char *info_path = NULL;
-	char sig[UFTRACE_MAGIC_LEN] = {0,};
+	char sig[UFTRACE_MAGIC_LEN] = {
+		0,
+	};
 
 	/* ensure that there is "info" file in the recorded directory */
 	xasprintf(&info_path, "%s/info", path);
@@ -346,7 +369,7 @@ char *read_exename(void)
 	static char exename[PATH_MAX];
 
 	if (!*exename) {
-		len = readlink("/proc/self/exe", exename, sizeof(exename)-1);
+		len = readlink("/proc/self/exe", exename, sizeof(exename) - 1);
 		if (len < 0)
 			pr_err("cannot read executable name");
 
@@ -356,6 +379,29 @@ char *read_exename(void)
 	return exename;
 }
 
+clockid_t clock_source = CLOCK_MONOTONIC;
+
+static const struct {
+	const char *name;
+	clockid_t clock_id;
+} trace_clocks[] = {
+	{ "mono", CLOCK_MONOTONIC },
+	{ "mono_raw", CLOCK_MONOTONIC_RAW },
+	{ "boot", CLOCK_BOOTTIME },
+};
+
+void setup_clock_id(const char *clock_str)
+{
+	size_t i;
+
+	for (i = 0; i < ARRAY_SIZE(trace_clocks); i++) {
+		if (!strcmp(clock_str, trace_clocks[i].name)) {
+			clock_source = trace_clocks[i].clock_id;
+			break;
+		}
+	}
+}
+
 bool check_time_range(struct uftrace_time_range *range, uint64_t timestamp)
 {
 	/* maybe it's called before first timestamp set */
@@ -429,24 +475,23 @@ uint64_t parse_time(char *arg, int limited_digits)
 	char *unit, *pos;
 	int i, decimal_places = 0, exp = 0;
 	uint64_t limited, decimal = 0;
-	uint64_t val = strtoull(arg, &unit, 0);
+	uint64_t val = strtoull(arg, &unit, 10);
 
 	pos = strchr(arg, '.');
 	if (pos != NULL) {
 		while (*(++pos) == '0')
 			decimal_places++;
-		decimal = strtoull(pos, &unit, 0);
+		decimal = strtoull(pos, &unit, 10);
 	}
 
 	limited = 10;
 	for (i = 1; i < limited_digits; i++)
 		limited *= 10;
 	if (val >= limited)
-		pr_err_ns("Limited %d digits (before and after decimal point)\n",
-			  limited_digits);
+		pr_err_ns("Limited %d digits (before and after decimal point)\n", limited_digits);
 	/* ignore more digits than limited digits before decimal point */
 	while (decimal >= limited)
-		decimal /=10;
+		decimal /= 10;
 
 	/*
 	 * if the unit is omitted, it is regarded as default unit 'ns'.
@@ -487,12 +532,12 @@ uint64_t parse_timestamp(char *arg)
 	uint64_t ts, tmp;
 	int len;
 
-	tmp = strtoull(arg, &sep, 0);
+	tmp = strtoull(arg, &sep, 10);
 	ts = tmp * NSEC_PER_SEC;
 
 	if (*sep == '.') {
 		arg = sep + 1;
-		tmp = strtoull(arg, &sep, 0);
+		tmp = strtoull(arg, &sep, 10);
 
 		len = 0;
 		while (isdigit(*arg)) {
@@ -529,7 +574,7 @@ uint64_t parse_timestamp(char *arg)
  * a copy of @right will be returned.  @left must be dynamically allocated
  * buffer so that it can be passed to realloc.
  */
-char * strjoin(char *left, char *right, const char *delim)
+char *strjoin(char *left, char *right, const char *delim)
 {
 	size_t llen = left ? strlen(left) : 0;
 	size_t rlen = strlen(right);
@@ -555,7 +600,7 @@ char * strjoin(char *left, char *right, const char *delim)
  * @str:   input string
  * @delim: delimiter to split the string
  *
- * This function build a string vector using @str splitted by @delim.
+ * This function builds a string vector using @str split by @delim.
  */
 void strv_split(struct strv *strv, const char *str, const char *delim)
 {
@@ -571,7 +616,7 @@ void strv_split(struct strv *strv, const char *str, const char *delim)
 	}
 
 	strv->nr = c;
-	strv->p = xcalloc(c + 1, sizeof(*strv->p));  /* including NULL at last */
+	strv->p = xcalloc(c + 1, sizeof(*strv->p)); /* including NULL at last */
 
 	c = 0;
 	tmp = saved_str;
@@ -645,7 +690,7 @@ void strv_replace(struct strv *strv, int idx, const char *str)
  * @strv with @delim.  Note that if @strv contains a single string,
  * @delim will be omitted and a copy of @right will be returned.
  */
-char * strv_join(struct strv *strv, const char *delim)
+char *strv_join(struct strv *strv, const char *delim)
 {
 	int i;
 	char *s;
@@ -681,7 +726,7 @@ void strv_free(struct strv *strv)
 #define QUOTES "\'\""
 
 /* escape double-quote with backslash - caller should free the returned string */
-char * json_quote(char *str, int *len)
+char *json_quote(char *str, int *len)
 {
 	char *p = str;
 	int quote = 0;
@@ -725,7 +770,7 @@ static int setargs(char *args, char **argv)
 		/* consider quotes and update argv */
 		if (*args == QUOTE) {
 			++args;
-			if (*args == QUOTE )
+			if (*args == QUOTE)
 				continue;
 			if (argv)
 				argv[count] = args;
@@ -816,7 +861,6 @@ char **parse_cmdline(char *cmd, int *argc)
 	return &argv[1];
 }
 
-
 /**
  * free_parsed_cmdline - free memory that was allocated by parse_cmdline
  * @argv: result of parse_cmdline
@@ -863,6 +907,100 @@ char *absolute_dirname(const char *path, char *resolved_path)
 	return resolved_path;
 }
 
+char *uftrace_strerror(int errnum, char *buf, size_t buflen)
+{
+	long result = (long)strerror_r(errnum, buf, buflen);
+
+	if (result == 0)
+		/* XSI-compliant strerror_r succeed */
+		return buf;
+	else if (result < 4096) {
+		/* XSI-compliant strerror_r failed */
+		snprintf(buf, buflen, "error: %d", errnum);
+		return buf;
+	}
+	else
+		/* GNU-specific strerror_r */
+		return (char *)result;
+}
+
+void stacktrace(void)
+{
+#ifdef HAVE_LIBUNWIND
+	const int max_depth = 64;
+	int i = 0;
+	bool out = false;
+	unw_cursor_t cursor;
+	unw_context_t context;
+
+	unw_getcontext(&context);
+	unw_init_local(&cursor, &context);
+
+	pr_yellow("Stack trace:\n");
+	while (unw_step(&cursor) && i < max_depth && !out) {
+		char symbol[256] = { "<unknown>" };
+		char *name = symbol;
+		unw_word_t ip, off;
+
+		unw_get_reg(&cursor, UNW_REG_IP, &ip);
+		if (!ip)
+			break;
+
+		if (!unw_get_proc_name(&cursor, symbol, sizeof(symbol), &off))
+			name = symbol;
+
+		pr_yellow("  #%-2d 0x%012" PRIxPTR " %s + 0x%" PRIxPTR "\n", ++i, (uintptr_t)(ip),
+			  name, (uintptr_t)(off));
+
+		/*
+		 * plt_hooker goes into infinite loop with unwinding.
+		 * so stop following the stack below.
+		 */
+		if (!strcmp(name, "plt_hooker"))
+			out = true;
+
+		if (name != symbol)
+			free(name);
+	}
+#endif
+	pr_out("\n");
+}
+
+int copy_file(const char *path_in, const char *path_out)
+{
+	char buf[4096];
+	FILE *ifp, *ofp;
+	int n;
+
+	ifp = fopen(path_in, "r");
+	if (ifp == NULL) {
+		pr_warn("cannot open file: %s: %m\n", path_in);
+		return -1;
+	}
+
+	ofp = fopen(path_out, "w");
+	if (ofp == NULL) {
+		pr_warn("cannot create file: %s: %m\n", path_out);
+		fclose(ifp);
+		return -1;
+	}
+
+	while (true) {
+		n = fread(buf, 1, sizeof(buf), ifp);
+		if (n == 0)
+			break;
+
+		if (fwrite_all(buf, n, ofp) < 0) {
+			pr_warn("cannot write to file: %m\n");
+			break;
+		}
+	}
+
+	fclose(ifp);
+	fclose(ofp);
+	return 0;
+}
+
 #ifdef UNIT_TEST
 TEST_CASE(utils_parse_cmdline)
 {
@@ -872,6 +1010,7 @@ TEST_CASE(utils_parse_cmdline)
 	cmdv = parse_cmdline(NULL, NULL);
 	TEST_EQ(cmdv, NULL);
 
+	pr_dbg("parse_cmdline() should handle quoted stringss\n");
 	cmdv = parse_cmdline("uftrace recv --run-cmd 'uftrace replay'", &argc);
 	TEST_NE(cmdv, NULL);
 	TEST_EQ(argc, 4);
@@ -891,15 +1030,17 @@ TEST_CASE(utils_strv)
 	int i;
 
 	const char test_str[] = "abc;def;xyz";
-	const char * test_array[] = { "abc", "def", "xyz" };
+	const char *test_array[] = { "abc", "def", "xyz" };
 
 	TEST_EQ(strv.nr, 0);
 	TEST_EQ(strv.p, NULL);
 
+	pr_dbg("split string into a vector using ';' delimiter\n");
 	strv_split(&strv, test_str, ";");
 	strv_for_each(&strv, s, i)
 		TEST_STREQ(s, test_array[i]);
 
+	pr_dbg("join the string vector into a single string\n");
 	s = strv_join(&strv, ";");
 	TEST_STREQ(s, test_str);
 	free(s);
@@ -908,6 +1049,7 @@ TEST_CASE(utils_strv)
 	strv_free(&strv);
 	TEST_EQ(strv.nr, 0);
 
+	pr_dbg("append string items to string vector\n");
 	for (i = 0; i < 3; i++) {
 		strv_append(&strv, test_array[i]);
 		TEST_STREQ(strv.p[i], test_array[i]);
@@ -923,4 +1065,59 @@ TEST_CASE(utils_strv)
 
 	return TEST_OK;
 }
+
+TEST_CASE(utils_parse_time)
+{
+	struct TC {
+		char *arg;
+		int limit;
+		uint64_t val;
+	} tc[] = {
+		{ "10.123ns", 5, 10 },
+		{ "50.987us", 5, 50987 },
+		{ "100.654ms", 5, 100654000 },
+		{ "123.123s", 5, 123123000000 },
+		{ "17.1m", 5, 1026000000000 },
+		{ "8960070.725168293s", 9, 8960070725168293 },
+		{ "8959832.082682731s", 9, 8959832082682731 },
+		{ "426883.003490100s", 9, 426883003490100 },
+		{ "8107.0641850s", 9, 8107064185000 },
+		{ "8107.006418501s", 9, 8107006418501 },
+	};
+	uint64_t time;
+	int i;
+
+	pr_dbg("parsing time with a unit\n");
+	for (i = 0; i < sizeof(tc) / sizeof(struct TC); i++) {
+		time = parse_time(tc[i].arg, tc[i].limit);
+		TEST_EQ(time, tc[i].val);
+	}
+
+	return TEST_OK;
+}
+
+TEST_CASE(utils_parse_timestamp)
+{
+	struct TC {
+		char *arg;
+		uint64_t val;
+	} tc[] = {
+		{ "8960070.725168293", 8960070725168293 },
+		{ "8959832.082682731", 8959832082682731 },
+		{ "426883.003490100", 426883003490100 },
+		{ "8107.0641850", 8107064185000 },
+		{ "8107.006418501234567890", 8107006418501 },
+	};
+	uint64_t time;
+	int i;
+
+	pr_dbg("parsing timestamp in various precisions with leading zeros\n");
+	for (i = 0; i < sizeof(tc) / sizeof(struct TC); i++) {
+		time = parse_timestamp(tc[i].arg);
+		TEST_EQ(time, tc[i].val);
+	}
+
+	return TEST_OK;
+}
+
 #endif /* UNIT_TEST */
